      subroutine hz98210 (iflag)
**********************************************************************
* Running: Ee=27.5(positrons), Ep=820, no polarization
*
* purpose: produce histograms for:
*
*          INTEGRATED JET SHAPE (psi) in Breit frame
*          (using different jetfinders, in bins of Et(jet) and eta(jet))
*       &
*          SUBJET MULTIPLICITIES in Breit frame
*          (using KTCLUS, in bins of Et(jet) and eta(jet))
*
*
*          event selection:
*            i)    energy of scattered lepton >  11 GeV
*            ii)   156 < polar scattering angle of lepton < 173
*            iii)  y > 0.15
*            iv)   Q**2 > 10 GeV**2
*
*          jet selection:
*            Ji)   number of jets >= 2
*            only the two jets with highest Et are considered
*            Jii)  for both jets: Et(jet) in Breit frame > 5 GeV
*            Jiii) for both jets: -1 < eta(jet) in lab. frame < 2
*
*          used Jetalgorithms:
*            Jetfinder 1: KTCLUS
*            Jetfinder 2: PXCONE with cone radius = 1.0
*            Jetfinder 3: PXCONE with cone radius = 0.7
*
*          Et-binning:
*            Et bin 1:   5 GeV < Et(jet) in Breit frame < 8 GeV
*            Et bin 2:   Et(jet) in Breit frame > 8 GeV
*          eta-binning:
*            eta bin 1:  eta(jet) in Breit frame < 1.5
*            eta bin 2:  1.5 < eta(jet) in Breit frame < 2.2
*            eta bin 3:  eta(jet) in Breit frame > 2.2
*
*
* produced histos:
*
*          MC histos for subjet multiplicities:
*             id = 120+(Etbin-1)*10+etabin
*          MC histos for integrated jet shapes:
*             id = 100*jetfinder+Etbin*10+etabin
*
*          Histos with H1 data have corresponding negative numbers plus
*             offset = 0:      for combined errors
*             offset = -1000:  for statistical errors only
*             offset = -2000:  for systematical errors only
*
* arguments: iflag=1 initialisation
*            iflag=2 filling
*            iflag=3 termination
*
* comments:   - be careful with (Snowmass) convention with massive hadrons
*             (Et not exactly equal to pt); this subroutine uses the pt-scheme
*             - with the definition of the integrated jet shape psi in
*             the referenced paper, psi(r/R=1) is in general not equal to 1
*
* structure:  called subroutines and functions:
*             from hbook lib:  hcdir,hmdir,hbookb,hbook1,
*                              hpak,hpake,hfill,hbarx
*             from HzTool lib: HzSvhepc, HzRchepc, HzBrtini, HzBrt,
*                              HzJtfind, HzBrtoLa, HzSubjm, HzJetshp,
*                              HzMeanHi, HzHinfo
*                              HzDiskin,HzIdelec,HzTheta, HzIbeam,
*                              HzIpgam
*             and some subroutines contained in this file
*
* written by: Andreas von Manteuffel
*
* reference:  'Untersuchung interner Jet-Struktur in tiefinelastischer
*              Streuung im H1-Experiment', Lars Sonnenschein,
*              III. Physikalisches Institut der Technischen Hochschule
*              Aachen, March 1998 (PITHA 98/4)
*
**********************************************************************
      IMPLICIT NONE
**********************************************************************
*     DECLARATIONS for HzTool stuff
**********************************************************************
*
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
*********************************************************************
*     DECLARATIONS for H1 histos
*********************************************************************
*
* jet shape and subjet multiplicity variables for H1 data:
      Real rtoRbins(11)
*
      Real psiA1a(10), psiaA1a(10), psiyA1a(10)
      Real psiA1b(10), psiaA1b(10), psiyA1b(10)
      Real psiA1c(10), psiaA1c(10), psiyA1c(10)
      Real psiA2a(10), psiaA2a(10), psiyA2a(10)
      Real psiA2b(10), psiaA2b(10), psiyA2b(10)
      Real psiA2c(10), psiaA2c(10), psiyA2c(10)

      Real psiB1a(10), psiaB1a(10), psiyB1a(10)
      Real psiB1b(10), psiaB1b(10), psiyB1b(10)
      Real psiB1c(10), psiaB1c(10), psiyB1c(10)
      Real psiB2a(10), psiaB2a(10), psiyB2a(10)
      Real psiB2b(10), psiaB2b(10), psiyB2b(10)
      Real psiB2c(10), psiaB2c(10), psiyB2c(10)

      Real psiC1a(10), psiaC1a(10), psiyC1a(10)
      Real psiC1b(10), psiaC1b(10), psiyC1b(10)
      Real psiC1c(10), psiaC1c(10), psiyC1c(10)
      Real psiC2a(10), psiaC2a(10), psiyC2a(10)
      Real psiC2b(10), psiaC2b(10), psiyC2b(10)
      Real psiC2c(10), psiaC2c(10), psiyC2c(10)

      Real lycutbns(14)

      Real smpl1a(13), smpla1a(13), smply1a(13)
      Real smpl1b(13), smpla1b(13), smply1b(13)
      Real smpl1c(13), smpla1c(13), smply1c(13)
      Real smpl2a(13), smpla2a(13), smply2a(13)
      Real smpl2b(13), smpla2b(13), smply2b(13)
      Real smpl2c(13), smpla2c(13), smply2c(13)


*********************************************************************
*     H1 DATA
*********************************************************************

* H1 integrated jet shape (psi)

* use following 'binning' for r/R in order to get points in histo
* at right positions
      Data rtoRbins  / 0.05, 0.15, 0.25, 0.35, 0.45, 0.55,
     .                 0.65, 0.75, 0.85, 0.95, 1.05 /

* suffix in variable labels mean:

*     A.. : jetfinder KTCLUS
*     B.. : jetfinder PXCONE with cone radius = 1.0
*     C.. : jetfinder PXCONE with cone radius = 0.7

*     .1. : 5 < Et < 8 GeV
*     .2. : Et > 8 GeV

*     ..a : eta < 1.5
*     ..b : 1.5 < eta < 2.2
*     ..c : eta > 2.2


      Data psiA1a  / 0.1503, 0.3612, 0.5539, 0.7003, 0.7998,
     .               0.8658, 0.9139, 0.9430, 0.9654, 0.9803 /
      Data psiaA1a / 0.0078, 0.0094, 0.0086, 0.0070, 0.0053,
     .               0.0038, 0.0028, 0.0020, 0.0015, 0.0010 /
      Data psiyA1a / 0.0159, 0.0188, 0.0165, 0.0092, 0.0090,
     .               0.0073, 0.0044, 0.0033, 0.0032, 0.0026 /

      Data psiA1b  / 0.1188, 0.3289, 0.5188, 0.6552, 0.7627,
     .               0.8398, 0.8965, 0.9361, 0.9609, 0.9753 /
      Data psiaA1b / 0.0085, 0.0116, 0.0108, 0.0091, 0.0069,
     .               0.0052, 0.0039, 0.0027, 0.0019, 0.0014 /
      Data psiyA1b / 0.0120, 0.0172, 0.0178, 0.0125, 0.0106,
     .               0.0075, 0.0058, 0.0051, 0.0034, 0.0036 /

      Data psiA1c  / 0.1474, 0.2832, 0.4644, 0.6170, 0.7317,
     .               0.8232, 0.8917, 0.9345, 0.9587, 0.9762 /
      Data psiaA1c / 0.0110, 0.0109, 0.0106, 0.0091, 0.0073,
     .               0.0054, 0.0040, 0.0030, 0.0022, 0.0016 /
      Data psiyA1c / 0.0275, 0.0201, 0.0207, 0.0155, 0.0120,
     .               0.0092, 0.0071, 0.0058, 0.0051, 0.0050 /

      Data psiA2a  / 0.1888, 0.4392, 0.6265, 0.7475, 0.8306,
     .               0.8804, 0.9174, 0.9449, 0.9644, 0.9799 /
      Data psiaA2a / 0.0109, 0.0121, 0.0104, 0.0083, 0.0064,
     .               0.0047, 0.0036, 0.0027, 0.0019, 0.0013 /
      Data psiyA2a / 0.0252, 0.0188, 0.0179, 0.0126, 0.0134,
     .               0.0116, 0.0070, 0.0066, 0.0031, 0.0018 /

      Data psiA2b  / 0.1317, 0.3946, 0.5829, 0.7107, 0.7983,
     .               0.8638, 0.9078, 0.9397, 0.9610, 0.9763 /
      Data psiaA2b / 0.0083, 0.0120, 0.0105, 0.0083, 0.0064,
     .               0.0047, 0.0035, 0.0027, 0.0019, 0.0013 /
      Data psiyA2b / 0.0094, 0.0153, 0.0181, 0.0140, 0.0096,
     .               0.0085, 0.0077, 0.0049, 0.0042, 0.0039 /

      Data psiA2c  / 0.1934, 0.4649, 0.6076, 0.7178, 0.7946,
     .               0.8524, 0.8986, 0.9292, 0.9557, 0.9710 /
      Data psiaA2c / 0.0124, 0.0138, 0.0111, 0.0087, 0.0065,
     .               0.0050, 0.0037, 0.0029, 0.0022, 0.0016 /
      Data psiyA2c / 0.0178, 0.0297, 0.0193, 0.0150, 0.0113,
     .               0.0084, 0.0069, 0.0070, 0.0037, 0.0029 /


      Data psiB1a  / 0.1260, 0.3150, 0.4868, 0.6247, 0.7301,
     .               0.8093, 0.8726, 0.9221, 0.9653, 0.9999 /
      Data psiaB1a / 0.0063, 0.0080, 0.0080, 0.0072, 0.0062,
     .               0.0051, 0.0040, 0.0029, 0.0019, 0.0001 /
      Data psiyB1a / 0.0081, 0.0150, 0.0151, 0.0167, 0.0122,
     .               0.0089, 0.0072, 0.0053, 0.0035, 0.0001 /

      Data psiB1b  / 0.0905, 0.2554, 0.4075, 0.5527, 0.6641,
     .               0.7501, 0.8331, 0.8970, 0.9527, 0.9994 /
      Data psiaB1b / 0.0065, 0.0090, 0.0094, 0.0091, 0.0078,
     .               0.0064, 0.0051, 0.0036, 0.0022, 0.0002 /
      Data psiyB1b / 0.0118, 0.0158, 0.0151, 0.0172, 0.0131,
     .               0.0158, 0.0169, 0.0124, 0.0042, 0.0006 /

      Data psiB1c  / 0.0841, 0.2483, 0.4068, 0.5375, 0.6648,
     .               0.7585, 0.8343, 0.8984, 0.9543, 0.9995 /
      Data psiaB1c / 0.0064, 0.0096, 0.0102, 0.0094, 0.0087,
     .               0.0071, 0.0059, 0.0044, 0.0026, 0.0003 /
      Data psiyB1c / 0.0137, 0.0316, 0.0305, 0.0248, 0.0251,
     .               0.0198, 0.0152, 0.0092, 0.0060, 0.0004 /

      Data psiB2a  / 0.1792, 0.4149, 0.6091, 0.7270, 0.8132,
     .               0.8694, 0.9121, 0.9478, 0.9753, 0.9998 /
      Data psiaB2a / 0.0100, 0.0120, 0.0108, 0.0088, 0.0073,
     .               0.0056, 0.0042, 0.0030, 0.0018, 0.0001 /
      Data psiyB2a / 0.0187, 0.0177, 0.0136, 0.0117, 0.0110,
     .               0.0104, 0.0070, 0.0054, 0.0020, 0.0002 /

      Data psiB2b  / 0.1378, 0.3891, 0.5800, 0.7030, 0.7915,
     .               0.8587, 0.9062, 0.9419, 0.9765, 0.9997 /
      Data psiaB2b / 0.0080, 0.0108, 0.0101, 0.0089, 0.0074,
     .               0.0056, 0.0042, 0.0030, 0.0016, 0.0001 /
      Data psiyB2b / 0.0093, 0.0134, 0.0146, 0.0130, 0.0096,
     .               0.0074, 0.0056, 0.0048, 0.0022, 0.0003 /

      Data psiB2c  / 0.1666, 0.3841, 0.5429, 0.6472, 0.7308,
     .               0.7995, 0.8617, 0.9080, 0.9568, 0.9990 /
      Data psiaB2c / 0.0104, 0.0118, 0.0109, 0.0094, 0.0081,
     .               0.0066, 0.0050, 0.0037, 0.0021, 0.0002 /
      Data psiyB2c / 0.0158, 0.0183, 0.0172, 0.0131, 0.0108,
     .               0.0088, 0.0068, 0.0051, 0.0044, 0.0005 /


      Data psiC1a  / 0.0996, 0.2627, 0.4326, 0.5817, 0.7006,
     .               0.7791, 0.8559, 0.9191, 0.9661, 1.0000 /
      Data psiaC1a / 0.0066, 0.0096, 0.0100, 0.0092, 0.0077,
     .               0.0065, 0.0050, 0.0035, 0.0022, 0.0001 /
      Data psiyC1a / 0.0116, 0.0176, 0.0213, 0.0174, 0.0144,
     .               0.0115, 0.0081, 0.0059, 0.0030, 0.0002 /

      Data psiC1b  / 0.1047, 0.2640, 0.4167, 0.5540, 0.6813,
     .               0.7562, 0.8342, 0.9005, 0.9601, 0.9995 /
      Data psiaC1b / 0.0092, 0.0121, 0.0124, 0.0115, 0.0102,
     .               0.0086, 0.0066, 0.0047, 0.0028, 0.0005 /
      Data psiyC1b / 0.0168, 0.0271, 0.0281, 0.0290, 0.0253,
     .               0.0222, 0.0174, 0.0065, 0.0052, 0.0002 /

      Data psiC1c  / 0.0779, 0.2146, 0.3714, 0.4924, 0.5953,
     .               0.7070, 0.7926, 0.8846, 0.9457, 1.0000 /
      Data psiaC1c / 0.0078, 0.0110, 0.0120, 0.0111, 0.0100,
     .               0.0087, 0.0072, 0.0054, 0.0032, 0.0001 /
      Data psiyC1c / 0.0123, 0.0200, 0.0307, 0.0197, 0.0190,
     .               0.0243, 0.0188, 0.0103, 0.0059, 0.0007 /

      Data psiC2a  / 0.1799, 0.3720, 0.5477, 0.6899, 0.7859,
     .               0.8499, 0.9006, 0.9369, 0.9707, 0.9996 /
      Data psiaC2a / 0.0131, 0.0141, 0.0134, 0.0118, 0.0089,
     .               0.0068, 0.0054, 0.0040, 0.0023, 0.0000 /
      Data psiyC2a / 0.0228, 0.0170, 0.0153, 0.0138, 0.0110,
     .               0.0099, 0.0063, 0.0044, 0.0040, 0.0004 /

      Data psiC2b  / 0.1155, 0.3333, 0.5338, 0.6572, 0.7586,
     .               0.8444, 0.8911, 0.9402, 0.9746, 1.0000 /
      Data psiaC2b / 0.0089, 0.0129, 0.0124, 0.0105, 0.0087,
     .               0.0066, 0.0051, 0.0033, 0.0021, 0.0000 /
      Data psiyC2b / 0.0144, 0.0177, 0.0183, 0.0144, 0.0113,
     .               0.0099, 0.0072, 0.0067, 0.0037, 0.0003 /

      Data psiC2c  / 0.1646, 0.3922, 0.5781, 0.7067, 0.7898,
     .               0.8430, 0.8968, 0.9372, 0.9715, 1.0000 /
      Data psiaC2c / 0.0129, 0.0159, 0.0144, 0.0119, 0.0098,
     .               0.0077, 0.0060, 0.0042, 0.0024, 0.0000 /
      Data psiyC2c / 0.0230, 0.0226, 0.0293, 0.0312, 0.0221,
     .               0.0152, 0.0099, 0.0080, 0.0041, 0.0001 /


* H1 mean subjet multiplicity (using KTCLUS) vs lg y_cut

* use following 'binning' for log(y_cut) in order to get points
* in histo at right positions
      Data lycutbns  / -3.125, -2.875, -2.625, -2.375,
     .                 -2.125, -1.875, -1.625, -1.375,
     .                 -1.125, -0.875, -0.625, -0.375,
     .                 -0.125,  0.125 /


* suffix in variable labels mean:

*     .1. : 5 < Et < 8 GeV
*     .2. : Et > 8 GeV

*     ..a : eta < 1.5
*     ..b : 1.5 < eta < 2.2
*     ..c : eta > 2.2


      Data smpl1a  / 4.144, 3.602, 3.155, 2.750, 2.330,
     .               1.932, 1.611, 1.340, 1.131, 1.012,
     .               1.000, 1.000, 1.000 /
      Data smpla1a / 0.043, 0.038, 0.033, 0.028, 0.024,
     .               0.021, 0.017, 0.015, 0.011, 0.004,
     .               0.000, 0.000, 0.000 /
      Data smply1a / 0.081, 0.088, 0.065, 0.042, 0.039,
     .               0.037, 0.032, 0.032, 0.020, 0.007,
     .               0.000, 0.000, 0.000 /

      Data smpl1b  / 4.374, 3.841, 3.328, 2.830, 2.391,
     .               2.029, 1.698, 1.391, 1.144, 1.025,
     .               1.000, 1.000, 1.000 /
      Data smpla1b / 0.054, 0.048, 0.042, 0.035, 0.030,
     .               0.026, 0.022, 0.020, 0.015, 0.005,
     .               0.000, 0.000, 0.000 /
      Data smply1b / 0.112, 0.085, 0.077, 0.070, 0.051,
     .               0.043, 0.054, 0.036, 0.032, 0.012,
     .               0.000, 0.000, 0.000 /

      Data smpl1c  / 4.589, 3.930, 3.400, 2.957, 2.451,
     .               2.082, 1.788, 1.440, 1.186, 1.042,
     .               1.000, 1.000, 1.000 /
      Data smpla1c / 0.052, 0.046, 0.041, 0.035, 0.031,
     .               0.027, 0.024, 0.021, 0.016, 0.007,
     .               0.000, 0.000, 0.000 /
      Data smply1c / 0.078, 0.067, 0.071, 0.053, 0.053,
     .               0.041, 0.039, 0.030, 0.025, 0.018,
     .               0.000, 0.000, 0.000 /

      Data smpl2a  / 4.143, 3.593, 3.114, 2.611, 2.126,
     .               1.733, 1.458, 1.220, 1.065, 1.005,
     .               1.000, 1.000, 1.000 /
      Data smpla2a / 0.056, 0.047, 0.042, 0.035, 0.030,
     .               0.025, 0.021, 0.017, 0.011, 0.005,
     .               0.000, 0.000, 0.000 /
      Data smply2a / 0.086, 0.083, 0.069, 0.078, 0.065,
     .               0.039, 0.035, 0.025, 0.014, 0.008,
     .               0.000, 0.000, 0.000 /

      Data smpl2b  / 4.480, 3.759, 3.179, 2.690, 2.232,
     .               1.893, 1.529, 1.302, 1.082, 1.000,
     .               1.000, 1.000, 1.000 /
      Data smpla2b / 0.053, 0.046, 0.041, 0.036, 0.030,
     .               0.026, 0.022, 0.019, 0.012, 0.004,
     .               0.000, 0.000, 0.000 /
      Data smply2b / 0.087, 0.076, 0.058, 0.054, 0.042,
     .               0.044, 0.032, 0.028, 0.020, 0.009,
     .               0.000, 0.000, 0.000 /

      Data smpl2c  / 4.424, 3.767, 3.187, 2.655, 2.192,
     .               1.796, 1.499, 1.275, 1.091, 1.000,
     .               1.000, 1.000, 1.000 /
      Data smpla2c / 0.055, 0.049, 0.044, 0.038, 0.031,
     .               0.026, 0.022, 0.019, 0.013, 0.005,
     .               0.000, 0.000, 0.000 /
      Data smply2c / 0.081, 0.087, 0.080, 0.054, 0.054,
     .               0.059, 0.037, 0.028, 0.022, 0.014,
     .               0.000, 0.000, 0.000 /

*********************************************************************
*     DECLARATIONS for MC histos
*********************************************************************

* function declaration
      integer getBinNo

*     variable names: i.... -> integer
*                     d.... -> double precision
*                     c.... -> character
*                     no special prefix -> real

* constants
      Real PI,RD
      Parameter (PI=3.1415927,RD=180./PI)

* other variables

      Character*8 cxxxx
      Data cxxxx/'hz98210'/

      Logical lp
      Integer iid, iEtbin, ietabin, iycut
      Integer ilept, ierr, ibeam, idum, imu, ijf, ijetfid, ijt
      Integer ihad, ir, ieta, iEt, iflag, iNhad, icount
      Integer isubjm(13)

      Character*80 chtitl
      Character*10 rout
      Data rout/'**hzjtshap'/
      Real y, Q2, thetae
      Real N(3,2,3), EtBins(2), etaBins(2)
      Real EtBrt (2), etaBrt (2), etaLab (2)
      Real ycut(13)
      Double Precision dpLab (4), dpBrt (4)
      Double Precision dpBeam(4), dpGam(4), dp5(5)
      Double Precision dconeR
      Double Precision djetsBrt (MAXHZJETS, 8)
      Double Precision djetsLab (MAXHZJETS, 8)
      Double Precision djshpRho(10,MAXHZJETS)
      Double Precision djshpPsi(10,MAXHZJETS)
      Double Precision dsubjmpl
      Double Precision drtoRval(10), dlycutvl(13)


* save contents
      save cxxxx
      save N, drtoRval, dlycutvl, ycut, EtBins, etaBins, lp

* settings
      Data lp /.false./
      Data EtBins / 5., 8. /
      Data etaBins / 1.5, 2.2 /

*   for histo filling:
      Data drtoRval  / 0.1D0, 0.2D0, 0.3D0, 0.4D0, 0.5D0,
     .                 0.6D0, 0.7D0, 0.8D0, 0.9D0, 1.0D0 /

*     note: lycutsm = lg y_cut
      Data dlycutvl / -3.00D0, -2.75D0, -2.50D0, -2.25D0,
     .                -2.00D0, -1.75D0, -1.50D0, -1.25D0,
     .                -1.00D0, -0.75D0, -0.50D0, -0.25D0,
     .                 0.00D0 /
      Data ycut / 1.E-3, 1.7782794E-3, 3.1622777E-3, 5.6234133E-3,
     .            1.E-2, 1.7782794E-2, 3.1622777E-2, 5.6234133E-2,
     .            1.E-1, 1.7782794E-1, 3.1622777E-1, 5.6234133E-1,
     .            1. /


*********************************************************************
*
*                      Initialization
*
*********************************************************************


*

      if (iflag.eq.1) then
*
*
*      Initialisation: The following MUST always be done
*      (i) make subdirectory in PAWC
*       - use the name as the xxxxxx in HZxxxxxx subroutine
*      (ii) make subdirectory in o/p file
*

         call hcdir('//PAWC',' ')
         call hmdir(cxxxx,'S')
         call hcdir('//HISTO',' ')
         call hmdir(cxxxx,'S')

         write (6,*) '****************************************'
         write (6,*) ' Hz',cxxxx,' initialization step'
         write (6,*) '****************************************'

*
* remind:
*     HistoIDs:  ID < 0  for H1 data
*                ID > 0  for corresponding generator histos
*               |ID| = i        for combined errors
*               |ID| = 1000 + i  for statist. errors only
*               |ID| = 2000 + i  for system. errors only
*
*

* build up H1-histos and prepare MC-histos

* KTCLUS
*     integrated jet shape psi
         write(chtitl,*)
     .        'Psi Breit (KTCLUS, 5.lt.Et.lt.8, eta.lt.1.5)'
         call H1Histo(111,chtitl,10,rtoRbins,psiA1a,psiaA1a,psiyA1a)
         call MCpsiHi(111,chtitl)
         write(chtitl,*)
     .        'Psi Breit (KTCLUS, 5.lt.Et.lt.8, 1.5.lt.eta.lt.2.2)'
         call H1Histo(112,chtitl,10,rtoRbins,psiA1b,psiaA1b,psiyA1b)
         call MCpsiHi(112,chtitl)
         write(chtitl,*)
     .        'Psi Breit (KTCLUS, 5.lt.Et.lt.8, 2.2.lt.eta)'
         call H1Histo(113,chtitl,10,rtoRbins,psiA1c,psiaA1c,psiyA1c)
         call MCpsiHi(113,chtitl)
         write(chtitl,*)
     .        'Psi Breit (KTCLUS, 8.lt.Et, eta.lt.1.5)'
         call H1Histo(121,chtitl,10,rtoRbins,psiA2a,psiaA2a,psiyA2a)
         call MCpsiHi(121,chtitl)
         write(chtitl,*)
     .        'Psi Breit (KTCLUS, 8.lt.Et, 1.5.lt.eta.lt.2.2)'
         call H1Histo(122,chtitl,10,rtoRbins,psiA2b,psiaA2b,psiyA2b)
         call MCpsiHi(122,chtitl)
         write(chtitl,*)
     .        'Psi Breit (KTCLUS, 8.lt.Et, 2.2.lt.eta)'
         call H1Histo(123,chtitl,10,rtoRbins,psiA2c,psiaA2c,psiyA2c)
         call MCpsiHi(123,chtitl)
*     subjet multiplicity
         write(chtitl,*) 'mean N(subjet) vs log(ycut) in Breit fr.'//
     .                   '(5.lt.Et.lt.8,eta.lt.1.5)'
         call H1Histo(131,chtitl,13,lycutbns,smpl1a,smpla1a,smply1a)
         call MCsmHi(131,chtitl)
         write(chtitl,*) 'mean N(subjet) vs log(ycut) in Breit fr.'//
     .                   '(5.lt.Et.lt.8, 1.5.lt.eta.lt.2.2)'
         call H1Histo(132,chtitl,13,lycutbns,smpl1b,smpla1b,smply1b)
         call MCsmHi(132,chtitl)
         write(chtitl,*) 'mean N(subjet) vs log(ycut) in Breit fr.'//
     .                   '(5.lt.Et.lt.8, 2.2.lt.eta)'
         call H1Histo(133,chtitl,13,lycutbns,smpl1c,smpla1c,smply1c)
         call MCsmHi(133,chtitl)
         write(chtitl,*) 'mean N(subjet) vs log(ycut) in Breit fr.'//
     .                   '(8.lt.Et, eta.lt.1.5)'
         call H1Histo(141,chtitl,13,lycutbns,smpl2a,smpla2a,smply2a)
         call MCsmHi(141,chtitl)
         write(chtitl,*) 'mean N(subjet) vs log(ycut) in Breit fr.'//
     .                   '(8.lt.Et, 1.5.lt.eta.lt.2.2)'
         call H1Histo(142,chtitl,13,lycutbns,smpl2b,smpla2b,smply2b)
         call MCsmHi(142,chtitl)
         write(chtitl,*) 'mean N(subjet) vs log(ycut) in Breit fr.'//
     .                   '(8.lt.Et, 2.2.lt.eta)'
         call H1Histo(143,chtitl,13,lycutbns,smpl2c,smpla2c,smply2c)
         call MCsmHi(143,chtitl)

* PXCONE with cone radius = 1.0
         write(chtitl,*)
     .       'Psi Breit (PXCONE(R=1.0), 5.lt.Et.lt.8, eta.lt.1.5)'
         call H1Histo(211,chtitl,10,rtoRbins,psiB1a,psiaB1a,psiyB1a)
         call MCpsiHi(211,chtitl)
         write(chtitl,*)
     .    'Psi Breit (PXCONE(R=1.0), 5.lt.Et.lt.8, 1.5.lt.eta.lt.2.2)'
         call H1Histo(212,chtitl,10,rtoRbins,psiB1b,psiaB1b,psiyB1b)
         call MCpsiHi(212,chtitl)
         write(chtitl,*)
     .       'Psi Breit (PXCONE(R=1.0), 5.lt.Et.lt.8, 2.2.lt.eta)'
         call H1Histo(213,chtitl,10,rtoRbins,psiB1c,psiaB1c,psiyB1c)
         call MCpsiHi(213,chtitl)
         write(chtitl,*)
     .       'Psi Breit (PXCONE(R=1.0), 8.lt.Et, eta.lt.1.5)'
         call H1Histo(221,chtitl,10,rtoRbins,psiB2a,psiaB2a,psiyB2a)
         call MCpsiHi(221,chtitl)
         write(chtitl,*)
     .       'Psi Breit (PXCONE(R=1.0), 8.lt.Et, 1.5.lt.eta.lt.2.2)'
         call H1Histo(222,chtitl,10,rtoRbins,psiB2b,psiaB2b,psiyB2b)
         call MCpsiHi(222,chtitl)
         write(chtitl,*)
     .       'Psi Breit (PXCONE(R=1.0), 8.lt.Et, 2.2.lt.eta)'
         call H1Histo(223,chtitl,10,rtoRbins,psiB2c,psiaB2c,psiyB2c)
         call MCpsiHi(223,chtitl)

* PXCONE with cone radius = 0.7
         write(chtitl,*)
     .       'Psi Breit (PXCONE(R=0.7), 5.lt.Et.lt.8, eta.lt.1.5)'
         call H1Histo(311,chtitl,10,rtoRbins,psiC1a,psiaC1a,psiyC1a)
         call MCpsiHi(311,chtitl)
         write(chtitl,*)
     .    'Psi Breit (PXCONE(R=0.7), 5.lt.Et.lt.8, 1.5.lt.eta.lt.2.2)'
         call H1Histo(312,chtitl,10,rtoRbins,psiC1b,psiaC1b,psiyC1b)
         call MCpsiHi(312,chtitl)
         write(chtitl,*)
     .       'Psi Breit (PXCONE(R=0.7), 5.lt.Et.lt.8, 2.2.lt.eta)'
         call H1Histo(313,chtitl,10,rtoRbins,psiC1c,psiaC1c,psiyC1c)
         call MCpsiHi(313,chtitl)
         write(chtitl,*)
     .       'Psi Breit (PXCONE(R=0.7), 8.lt.Et, eta.lt.1.5)'
         call H1Histo(321,chtitl,10,rtoRbins,psiC2a,psiaC2a,psiyC2a)
         call MCpsiHi(321,chtitl)
         write(chtitl,*)
     .       'Psi Breit (PXCONE(R=0.7), 8.lt.Et, 1.5.lt.eta.lt.2.2)'
         call H1Histo(322,chtitl,10,rtoRbins,psiC2b,psiaC2b,psiyC2b)
         call MCpsiHi(322,chtitl)
         write(chtitl,*)
     .       'Psi Breit (PXCONE(R=0.7), 8.lt.Et, 2.2.lt.eta)'
         call H1Histo(323,chtitl,10,rtoRbins,psiC2c,psiaC2c,psiyC2c)
         call MCpsiHi(323,chtitl)

* prepare control histos

         write(chtitl,*) 'dn/dQ2 in lab. fr. - control plot'
         call hbook1(1, chtitl, 100, 0., 10000., 0.)
         call hbarx(1)

         write(chtitl,*) 'dn/dEe in lab. fr. - control plot'
         call hbook1(2, chtitl, 100, 0., 100., 0.)
         call hbarx(2)

         write(chtitl,*) 'dn/dthetae in lab. fr. - control plot'
         call hbook1(3, chtitl, 50, 0., 180., 0.)
         call hbarx(3)

         write(chtitl,*) 'dn/dy in lab. fr. - control plot'
         call hbook1(4, chtitl, 100, 0., 1., 0.)
         call hbarx(4)

         write(chtitl,*) 'dn/dEt(jet) in Breit fr. - control plot'
         call hbook1(5, chtitl, 20, 0., 20., 0.)
         call hbarx(5)

         write(chtitl,*) 'dn/deta(jet) in lab. fr. - control plot'
         call hbook1(6, chtitl, 20, -4., 4., 0.)
         call hbarx(6)

         write(chtitl,*) 'dn/deta(jet) in Breit fr. - control plot'
         call hbook1(7, chtitl, 20, -4., 4., 0.)
         call hbarx(7)

         write(chtitl,*) 'reached checkpoints - control plot'
         call hbook1(10, chtitl, 50, -0.5, 49.5, 0.)


* init number of jets
         do ijt = 1,3
            do iEt = 1,2
               do ieta = 1,3
                  N(ijt,iEt,ieta) = 0.
               enddo
            enddo
         enddo

*********************************************************************
*
*                      Event Processing
*
*********************************************************************

      else if (iflag.eq.2) then

*      Filling: The following MUST always be done
*      (i) move to the correct sub-directory in PAWC
*
         call hcdir('//PAWC/'//cxxxx,' ')

* first of all: save the HEP event prime common
         call HzSvhepc

****************************************************
* Part A: KINEMATIC CUTS
****************************************************
         call hfill(10,1.,0.,1.)

* get Q2
         Q2 = real(HzDiskin(1))
         if (Q2.le.0.) then
            write(6,*) 'HZ'//cxxxx,' ERROR: unphysical Q2 !!!'
            goto 9990
         endif


*     fill control histo: Q2 distribution
         call hfill(1, Q2, 0., 1.)


* CUT (iv): Q2 > 10 GeV
*----------
         if (Q2.le.10.) goto 9990
         call hfill(10,2.,0.,1.)

* detect scattered lepton
         ilept = HzIdelec(idum)
         if (ilept.eq.-1) then
            write(6,*) 'Hz'//cxxxx,' ERROR: lepton not found !!!'
            goto 9990
         endif

*     fill control histo: lepton energy distribution
         call hfill(2, real(PHEP(4,ilept)), 0., 1.)

* CUT (i): energy of scattered lepton > 11 GeV
*---------
         if (real(PHEP(4,ilept)).le.11.) goto 9990
         call hfill(10,3.,0.,1.)

* get polar angle of scattered lepton
         thetae = RD * real(HzTheta(ilept))
*     fill control histo: polar angle of scattered lepton
         call hfill(3, thetae, 0., 1.)

* CUT (ii): polar angle of scattered lepton within (156, 173)
*----------
         if ((thetae.le.156.).or.(thetae.ge.173.)) goto 9990
         call hfill(10,4.,0.,1.)

* get y
         y = real(HzDiskin(3))

*     fill control histo: y distribution
         call hfill(4, y, 0., 1.)


* CUT (iii):  y > 0.15
*-----------
         if (y.le.0.15) goto 9990
         call hfill(10,5.,0.,1.)


****************************************************
* Part B: BOOST TO BREIT FRAME
****************************************************

* detect beam proton
         ierr = HzIbeam(ibeam,idum)
         if (.not.(ierr.eq.1)) then
            write(6,*) 'HZ'//cxxxx,' ERROR: beams not found ! '
            goto 9990
         else
            do imu = 1, 4
               dpBeam(imu)=PHEP(imu,IBEAM)
            enddo
         endif

* detect virtual boson
         ierr = HzIpgam(dp5)
         if (ierr.eq.-1) then
            write(6,*) 'HZ'//cxxxx,' ERROR: boson vector not found ! '
            goto 9990
         else
            do imu = 1, 4
               dpGam(imu) = dp5(imu)
            enddo
         endif

* init boost to Breit frame
         call HzBrtini (dpBeam, dpGam, ierr)
         if (ierr.eq.-1) then
            write(6,*)
     .          'HZ'//cxxxx,' problem with boost to Breit frame !'
            goto 9990
         endif


* boost to Breit frame

*     loop over particle array
C         if (lp) then
C          write(6,*) rout,' PHEP in LAB: '
C          call hzlihep(2)
C         endif
         do ihad = 1, NHEP
*     skip unstable particles and lepton
            if ((isthep(ihad).eq.1).and.(ihad.ne.ilept)) then
*     perform boost to Breit frame
               do imu = 1, 4
                  dpLab(imu) = PHEP(imu, ihad)
               enddo
               call HzBrt(dpLab, dpBrt, ierr)
               if (ierr.eq.-1) then
                  write(6,*) 'HZ'//cxxxx,
     .                 ' problem with boost to Breit frame ! '
                  goto 9990
               endif
*     overwrite HEP particle array with 4-vectors boosted to Breit frame
               do imu = 1, 4
                  PHEP(imu, ihad) = dpBrt(imu)
               enddo
            endif
         enddo
*
* loop over 3 different jetalgorithms and -settings
*--------------------------------------------------
 3333    continue
C         if (lp) then
C          write(6,*) rout,' PHEP in BREIT: '
C          call hzlihep(2)
C         endif
*
         do 6000 ijf = 1, 3

****************************************************
* Part C: JET FINDING
****************************************************
            call hfill(10,6.,0.,1.)


            if (ijf.eq.1) then
*     KTCLUS
               ijetfid = 3
               dconeR = 1.D0
            elseif (ijf.eq.2) then
*     PXCONE with cone radius = 1.0
               ijetfid = 2
               dconeR = 1.D0
            elseif(ijf.eq.3) then
*     PXCONE with cone radius = 0.7
               ijetfid = 2
               dconeR = 7.D-1
            endif

* jet finding
            call hzjtfind(ijetfid,dconeR,idum,djetsBrt)
            if (lp) then
             write(6,*) rout,' ijet= ',ijetfid
             call hzlijet(numjets,jets)
            endif
****************************************************
* Part D: JET CUTS
****************************************************
* note: jetfinder is assumed to sort jets by Et

* CUT (Ji): number of jets >= 2
*----------
            if (NUMJETS.lt.2) goto 6000
            call hfill(10,7.,0.,1.)


*     get kinematic variables of first two jets
            do ijt = 1,2
               EtBrt(ijt) = real(JETS(ijt,3))
               call hfill(5,EtBrt(ijt),0.,1.)
               etaBrt(ijt) = real(JETS(ijt,1))
               call hfill(7,etaBrt(ijt),0.,1.)
            enddo

* CUT (Jii): there must be two jets with Et > 5 GeV each
*-----------
            if ((EtBrt(1).le.5.).or.(EtBrt(2).le.5.)) goto 6000
            call hfill(10,8.,0.,1.)

* boost jets from Breit to lab. frame
            call hzBrtoLa(2,djetsBrt,djetsLab)
            if (lp) then
             write(6,*) rout,' jets in lab '
             call hzlijet(2,djetsLab)
            endif
* CUT (Jiii): pseudo-rapidity eta of the two jets with highest Et within (-1,2)
*------------
            do ijt = 1,2
              etaLab(ijt)=real(djetsLab(ijt,1))
              call hfill(6,etaLab(ijt),0.,1.)
              if ((etaLab(ijt).le.-1.).or.(etaLab(ijt).ge.2.)) goto 6000
            enddo

            call hfill(10,9.,0.,1.)


*******************************************************************
* Part E: CALCULATION OF PSI, SUBJET MULTIPLICITY AND HISTOFILLING
*******************************************************************

            if (ijf.eq.1) then
*     prepare calculation of subjet multiplicity
               call HzSubjm(1, 0, 13, ycut, isubjm, ierr)
               if (ierr.eq.-1) goto 9990
            endif

*     calculate jet shapes
            call HzJetshp(0, dconeR, djshpRho, djshpPsi, ierr)
            if (ierr.eq.-1) goto 9990

* loop over jets
            do ijt = 1, 2

* get Et bin-no (=1,2) and eta bin-no (=1,2,3)
               iEtbin = getBinNo(1, EtBins, EtBrt(ijt))
               ietaBin = getBinNo(1, etaBins, etaBrt(ijt)) + 1
               if ((iEtbin.lt.1).or.(iEtbin.gt.2).or.
     &             (ietabin.lt.1).or.(ietabin.gt.3)) then
*     note: this should never happen (due to cuts above)
                  write(6,*) 'HZ'//cxxxx, ' ERROR: invalid bin number !'
                  goto 6000
               endif
               call hfill(10,10.+real((iEtbin-1)*3+ietaBin),0.,1.)

* increase normalization constant (no of jets)
               N(ijf,iEtbin,ietabin) = N(ijf,iEtbin,ietabin)+wtx

* fill histo for integrated jet shape
               iid = ijf*100+iEtbin*10+ietabin
               do ir = 1, 10
C                  write(6,*) ir,' x,y= ',drtoRval(ir),djshpPsi(ir,ijt)
                  call HzMeanHi(iid,drtoRval(ir),djshpPsi(ir,ijt),wtx)
               enddo

               if (ijf.eq.1) then
* calculate subjet multiplicity
                  call HzSubjm(2, ijt, 13, ycut, isubjm, ierr)
C                  write(6,*)'subjm(1),(13):',isubjm(1),isubjm(13)
                  if (ierr.eq.-1) goto 9990
* fill histo for subjet multiplicity
                  iid = 120+iEtbin*10+ietabin
C                 write(6,*) ' filling = ',iid
                  do iycut = 1, 13
                     dsubjmpl = dble(isubjm(iycut))
C                     write(6,*) iid,' dly= ',dlycutvl(iycut),'wtx= ',wtx
                     call HzMeanHi(iid,dlycutvl(iycut),dsubjmpl,wtx)
                  enddo
               endif
            enddo
 6000    continue

 9990    continue

* recover the HEP event prime common !
         call HzRchepc

*********************************************************************
*
*                      Termination
*
*********************************************************************

      else if(iflag.eq.3) then

*      Termination: The following MUST always be done
*      (i) Move to the correct PAW subdirectory
*
         call hcdir('//PAWC/'//cxxxx,' ')
         call hfill(10,43.,0.,1.)

         do ijf = 1, 3
          do iEt = 1, 2
           do ieta = 1, 3
              if (N(ijf, iEt, ieta).gt.0.) then
* write number of jets
                 iid = ijf*100+iEt*10+ieta
                 call HzHinfo(iid, N(ijf, iEt, ieta))
                 if (ijf.eq.1) then
                    iid = 120+iEt*10+ieta
                    call HzHinfo(iid, N(ijf, iEt, ieta))
                 endif
*     note: simple chi-square calculation doesn't make sense due
*           to correlations
              endif
           enddo
          enddo
         enddo

      endif

      return

      end
