*CMZ :  2.00/03 31/05/2000  15.33.39  by  Klaus Rabbertz
*-- Author : Klaus Rabbertz
      SUBROUTINE HZ99193(IFLAG)
**********************************************************************
* Running: Ee=27.5, Ep=820, no polarization
*
* Purpose: Produce histograms for:
*
*          Event shape variables: Two versions of thrust tau_C = 1-T_C,
*          ---------------------- tau = 1-T, jet broadening B, jet mass rho,
*                                 C parameter, two versions of diff. two-jet
*                                 rate y_kt and y_fJ
*
*          Q-bins: Low Q sample:
*                  1) 7 < Q < 8 GeV
*                  2) 8 < Q < 10 GeV
*                  High Q sample:
*                  3) 14 < Q < 16 GeV
*                  4) 16 < Q < 20 GeV
*                  5) 20 < Q < 30 GeV
*                  6) 30 < Q < 50 GeV
*                  7) 50 < Q < 70 GeV
*                  8) 70 < Q < 100 GeV
*
*          Event selection (phase space cuts only):
*          i)    Energy of scattered lepton:
*                E_e' > 14 GeV (low Q)
*                E_e' > 11 GeV (high Q)
*          ii)   Polar angle of scattered lepton:
*                157 deg < theta_e' < 173 deg (low Q)
*                 30 deg < theta_e' < 150 deg (high Q)
*          iii)  Inelasticity y_e (from lepton) and y_h (from hadr.fin.state):
*                0.05 < y_e < 0.80
*                0.05 < y_h        ( = y_e = y on generator level)
*          iv)   Angle of 'quark' direction as deduced from the scattered
*                lepton in QPM:
*                theta_q > 20 deg
*          v)    Hadronic energy in forward region (polar angle within (4,15)):
*                E_forw > 0.5 GeV
*          vi)   Total hadronic energy in Breit current hemisphere:
*                E_CH > 0.1 * Q (part of the def., NOT for y_kt, y_fJ)
*          vii)  No. of hadronic objects in Breit current hemisphere:
*                N_CH >= 2 (NOT for y_kt, y_fJ)
*          viii) Note:
*                There is a usually ineffective cutoff (s. statistics in
*                log file) of O(10^-5) to stay away from the exact left and
*                right borders of the distributions. For y_kt overflow events
*                may occur. In case of the mean values this cut is NOT
*                active.
*
* Produced histos: Mean values:   ID = 10:          <tau>
*                                 ID = 20:          <B>
*                                 ID = 30:          <tau_C>
*                                 ID = 40:          <rho>
*                                 ID = 50:          <C>
*                                 ID = 60:          <y_fJ>
*                                 ID = 70:          <y_kt>
*
*                  Distributions: (QbinNo = 1..8)
*                                 ID = 10 + QbinNo: 1/N dn/dtau
*                                 ID = 20 + QbinNo: 1/N dn/dB
*                                 ID = 30 + QbinNo: 1/N dn/dtau_C
*                                 ID = 40 + QbinNo: 1/N dn/drho
*                                 ID = 50 + QbinNo: 1/N dn/dC
*                                 ID = 60 + QbinNo: 1/N dn/dy_fJ
*                                 ID = 70 + QbinNo: 1/N dn/dy_kt
*
*                  H1 data histograms have corresponding negative numbers.
*                  Data histos with systematic uncertainties only and total
*                  uncertainties are stored with offset -100 and -200.
*
* Arguments: IFLAG = 1: Initialization
*            IFLAG = 2: Filling
*            IFLAG = 3: Termination
*
* Called subroutines and functions:
*            From HBOOK lib:  HCDIR,HMDIR,HBOOK1,HBOOKB,
*                             HFILL,HPAK,HPAKE,HBARX
*            From HzTool lib: DEVSHP
*                             HzDiskin,HzIdelec,HzIpgam,HzIbeam,
*                             HzBrtini,HzBrt,HzHinrm,HzHinfo,HzChisq
*
* Adapted by: Klaus Rabbertz
*
* Reference: Eur. Phys. J. C 14 (2000) 255, DESY 99-193.
*
**********************************************************************

      IMPLICIT NONE

**********************************************************************
*
*                      Declarations
*
**********************************************************************

#include "hzfunc.inc"
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"

*
*---Subroutine-specific settings
*-------------------------------
*---Constants:
      INTEGER NB,NQ,NE
      PARAMETER (NB = 10, NQ = 8, NE = 7)
      REAL PI,RD,EE
      PARAMETER (PI = 3.1415927, RD = 180./PI, EE = 27.5)

*---General variables:
*---DESY Preprint number
      CHARACTER*6 XXXX
      DATA XXXX/'99193'/

      LOGICAL LP
      DATA LP /.TRUE./
      SAVE LP

*---Histo variables:
      CHARACTER*80 CHMEAN(7),CHDIST(7)
      DATA CHMEAN/'<tau>','<B>','<tau_C>','<rho>','<C>',
     &     '<y_fJ>','<y_kt>'/
      DATA CHDIST/'1/N dn/d(tau)','1/N dn/dB',
     &     '1/N dn/d(tau_C)','1/N dn/d(rho)',
     &     '1/N dn/dC','1/N dn/d(y_fJ)','1/N dn/d(y_kt)'/

*---Kinematic variables:
      LOGICAL LOWQ,HIGHQ
      REAL Q2,Q,X,Y
      REAL COSTHETA,THETAE,THETAH,TEMP
      REAL FTQ,BTQ,CTQ,THQ
      DOUBLE PRECISION EHADFWD,ECQ
      DOUBLE PRECISION PLAB (4)
      DOUBLE PRECISION PBRT (4)
      DOUBLE PRECISION PHAD (4,NMXHEP)
      DOUBLE PRECISION PBEAM(4),PGAM(4),P5(5)

*---Event shape variables:
      REAL QL8(NQ),QDT8(NQ),QR8(NQ)
      REAL DM(0:NB,NQ,NE),DDMST(0:NB,NQ,NE)
      REAL DDMSYP(0:NB,NQ,NE),DDMSYM(0:NB,NQ,NE)
      REAL FBLD(NB,NE),FBCD(NB,NE),FBRD(NB,NE)
      REAL DMTMP(NB),DDMTMP(NB)
      REAL QBINNING(10)
      DATA QBINNING/7.0,8.0,10.0,14.0,16.0,20.0,30.0,50.0,70.0,100.0/

*---For MC calculation:
      REAL ESMEAN(7,8)
      REAL ESMEANE(7,8)
      REAL ESMTMP(9)
      REAL ESMETMP(9)

      DOUBLE PRECISION NL
      PARAMETER (NL=5.D-5)
      DOUBLE PRECISION DLIM(7)
      DATA DLIM/0.D05,0.D025,0.D025,0.D0125,0.D05,0.D03,0.D025/
      SAVE DLIM

      DOUBLE PRECISION LOLIMIT (7)
      DOUBLE PRECISION HILIMIT (7)
      DOUBLE PRECISION EVSHPMC (7)
      DOUBLE PRECISION EVSHPSR (13)
      DOUBLE PRECISION ESSUM (7,8)
      DOUBLE PRECISION ESSQSUM (7,8)
      SAVE ESSUM
      SAVE ESSQSUM

*---Counting, index and errorcode variables:
      INTEGER I,IB,IQ,IE
      INTEGER ID,IH,IM
      INTEGER IHEP,IEL,IBEAM,IGAM
      INTEGER IDUM,IERR,IFLAG
      INTEGER ICHN,CURQBIN
      INTEGER IHAD,ICOMP
      INTEGER NCURR
      REAL NEVT(7,8)
      REAL LOCUT(7,8)
      REAL HICUT(7,8)
      DOUBLE PRECISION DDUM
      SAVE NEVT

*********************************************************************
*
*                      Initialization
*
*********************************************************************
      IF (IFLAG.EQ.1) THEN

*---Initialization: The following MUST always be done
*      (i) make subdirectory in PAWC
*       - use the name as the xxxxxx in HZxxxxxx subroutine
*      (ii) make subdirectory in o/p file
         CALL HCDIR('//PAWC',' ')
         CALL HMDIR(XXXX,'S')
         CALL HCDIR('//HISTO',' ')
         CALL HMDIR(XXXX,'S')

*---Hello world!
	 WRITE(6,*)'***************************************************'
	 WRITE(6,*)'HZ'//xxxx//' called'
         WRITE(6,*)'***************************************************'

*---Read H1 Data
         OPEN (UNIT=20,FILE='hz99193mean.dat',
     &        STATUS='OLD',FORM='FORMATTED')
 1001    FORMAT(2X,3(F6.1,5X),3(F7.4,4X),F7.4)
         IB = 0
         DO IE=1,NE
            DO I=1,3
               READ(20,*)
            ENDDO
            DO IQ=1,NQ
               READ(20,1001)QL8(IQ),QDT8(IQ),QR8(IQ),
     &              DM(IB,IQ,IE),DDMST(IB,IQ,IE),
     &              DDMSYP(IB,IQ,IE),DDMSYM(IB,IQ,IE)
            ENDDO
            READ(20,*)
         ENDDO
         CLOSE(20)
         OPEN (UNIT=20,FILE='hz99193dist.dat',
     &        STATUS='OLD',FORM='FORMATTED')
 1002    FORMAT(2X,3(F6.3,5X),3(G10.3,1X),G10.3)
         DO IE=1,NE
            READ(20,*)
            DO IQ=1,NQ
               DO I=1,6
                  READ(20,*)
               ENDDO
               DO IB=1,NB
                  READ(20,1002)FBLD(IB,IE),FBCD(IB,IE),FBRD(IB,IE),
     &                 DM(IB,IQ,IE),DDMST(IB,IQ,IE),
     &                 DDMSYP(IB,IQ,IE),DDMSYM(IB,IQ,IE)
               ENDDO
            ENDDO
            READ(20,*)
         ENDDO

*---Book histograms, fill data histograms
*---Reminder: HistoID's:  ID > 0  : MC, stat. uncertainty
*                         ID < 0  : H1 data, stat. uncertainty
*                         ID <-100: H1 data, syst. uncertainty (symm. overlay)
*                         ID <-200: H1 data, tot. uncertainty (symm. overlay)
         DO IE=1,NE
            IM = 10*IE
            CALL HBOOKB(IM,CHMEAN(IE),NQ+1,QBINNING,0.)
            CALL HBARX(IM)
            DO IH=0,2
               ID = -(100*IH+10*IE)
               CALL HBOOKB(ID,CHMEAN(IE),NQ+1,QBINNING,0.)
               IF (IH.EQ.0) THEN
                  DO IQ=1,2
                     DMTMP(IQ)  = DM(0,IQ,IE)
                     DDMTMP(IQ) = DDMST(0,IQ,IE)
                  ENDDO
                  DMTMP(3)  = 0.
                  DDMTMP(3) = 0.
                  DO IQ=3,8
                     DMTMP(IQ+1)  = DM(0,IQ,IE)
                     DDMTMP(IQ+1) = DDMST(0,IQ,IE)
                  ENDDO
               ELSEIF (IH.EQ.1) THEN
                  DO IQ=1,2
                     DMTMP(IQ)  = DM(0,IQ,IE) + 0.5 *
     &                    (DDMSYP(0,IQ,IE)+DDMSYM(0,IQ,IE))
                     DDMTMP(IQ) = 0.5 *
     &                    (DDMSYP(0,IQ,IE)-DDMSYM(0,IQ,IE))
                  ENDDO
                  DMTMP(3)  = 0.
                  DDMTMP(3) = 0.
                  DO IQ=3,8
                     DMTMP(IQ+1)  = DM(0,IQ,IE) + 0.5 *
     &                    (DDMSYP(0,IQ,IE)+DDMSYM(0,IQ,IE))
                     DDMTMP(IQ+1) = 0.5 *
     &                    (DDMSYP(0,IQ,IE)-DDMSYM(0,IQ,IE))
                  ENDDO
               ELSE
                  DO IQ=1,2
                     DMTMP(IQ)  = DM(0,IQ,IE) + 0.5 *
     &                    (SQRT(DDMST(0,IQ,IE)**2.+DDMSYP(0,IQ,IE)**2.)-
     &                    SQRT(DDMST(0,IQ,IE)**2.+DDMSYM(0,IQ,IE)**2.))
                     DDMTMP(IQ) = 0.5 *
     &                    (SQRT(DDMST(0,IQ,IE)**2.+DDMSYP(0,IQ,IE)**2.)+
     &                    SQRT(DDMST(0,IQ,IE)**2.+DDMSYM(0,IQ,IE)**2.))
                  ENDDO
                  DMTMP(3)  = 0.
                  DDMTMP(3) = 0.
                  DO IQ=3,8
                     DMTMP(IQ+1)  = DM(0,IQ,IE) + 0.5 *
     &                    (SQRT(DDMST(0,IQ,IE)**2.+DDMSYP(0,IQ,IE)**2.)-
     &                    SQRT(DDMST(0,IQ,IE)**2.+DDMSYM(0,IQ,IE)**2.))
                     DDMTMP(IQ+1) = 0.5 *
     &                    (SQRT(DDMST(0,IQ,IE)**2.+DDMSYP(0,IQ,IE)**2.)+
     &                    SQRT(DDMST(0,IQ,IE)**2.+DDMSYM(0,IQ,IE)**2.))
                  ENDDO
               ENDIF
               CALL HPAK(ID,DMTMP)
               CALL HPAKE(ID,DDMTMP)
            ENDDO
            DO IQ=1,NQ
               IM = 10*IE+IQ
               CALL HBOOK1(IM,CHDIST(IE),
     &              NB,FBLD(1,IE),FBRD(NB,IE),0.)
               CALL HBARX(IM)
               DO IH=0,2
                  ID = -(100*IH+10*IE+IQ)
                  CALL HBOOK1(ID,CHDIST(IE),
     &                 NB,FBLD(1,IE),FBRD(NB,IE),0.)
                  IF (IH.EQ.0) THEN
                     DO IB=1,NB
                        DMTMP(IB)  = DM(IB,IQ,IE)
                        DDMTMP(IB) = DDMST(IB,IQ,IE)
                     ENDDO
                  ELSEIF (IH.EQ.1) THEN
                     DO IB=1,NB
                        DMTMP(IB)  = DM(IB,IQ,IE) + 0.5 *
     &                       (DDMSYP(IB,IQ,IE)+DDMSYM(IB,IQ,IE))
                        DDMTMP(IB) = 0.5 *
     &                       (DDMSYP(IB,IQ,IE)-DDMSYM(IB,IQ,IE))
                     ENDDO
                  ELSE
                     DO IB=1,NB
                        DMTMP(IB)  = DM(IB,IQ,IE) + 0.5 *
     &                       (SQRT(DDMST(IB,IQ,IE)**2. +
     &                       DDMSYP(IB,IQ,IE)**2.) -
     &                       SQRT(DDMST(IB,IQ,IE)**2. +
     &                       DDMSYM(IB,IQ,IE)**2.))
                        DDMTMP(IB) = 0.5 *
     &                       (SQRT(DDMST(IB,IQ,IE)**2. +
     &                       DDMSYP(IB,IQ,IE)**2.)+
     &                       SQRT(DDMST(IB,IQ,IE)**2. +
     &                       DDMSYM(IB,IQ,IE)**2.))
                     ENDDO
                  ENDIF
                  CALL HPAK(ID,DMTMP)
                  CALL HPAKE(ID,DDMTMP)
               ENDDO
            ENDDO
         ENDDO

*---Initialize counter variables
         DO IE=1,NE
            DO IQ=1,NQ
               ESSUM(IE,IQ) = 0.D0
               ESSQSUM(IE,IQ) = 0.D0
               NEVT(IE,IQ) = 0.
               LOCUT(IE,IQ) = 0.
               HICUT(IE,IQ) = 0.
            ENDDO
         ENDDO

*---Initialize exclusive limits for event shapes in MC calculation
         DO IE=1,NE
            LOLIMIT(IE) = DBLE(FBLD(1,IE))  + 1.D-3*DLIM(IE)
            HILIMIT(IE) = DBLE(FBRD(NB,IE)) - 1.D-3*DLIM(IE)
         ENDDO

*********************************************************************
*
*                      Event Processing
*
*********************************************************************

      ELSEIF (IFLAG.EQ.2) THEN


*---Filling: The following MUST always be done
*   (i) move to the correct sub-directory in PAWC
*
         CALL HCDIR('//PAWC/'//XXXX,' ')

*----------------
* Event selection
*----------------

*---Get Q2
         Q2 = REAL(HZDISKIN(1))

*---Get Q
         IF (Q2.LE.0.) THEN
            WRITE(6,*) 'Hz'//xxxx,' ERROR: Q2 less or equal zero !!!'
            GOTO 9990
         ELSE
            Q = SQRT(Q2)
ckr            CALL HFILL(1,Q,0.,WTX)
         ENDIF

*---Q-binning: Q has to be within [7 GeV, 10 GeV] (low Q) or
*------------              within [14 GeV, 100 GeV] (high Q)

         LOWQ  = .FALSE.
         HIGHQ = .FALSE.
         IF (7..LE.Q.AND.Q.LE.10.) THEN
            LOWQ = .TRUE.
            IF (Q.LT.8.)  THEN
               CURQBIN = 1
            ELSE
               CURQBIN = 2
            ENDIF
         ELSEIF (14..LE.Q.AND.Q.LE.100.) THEN
            HIGHQ = .TRUE.
            IF (Q.LT.16.) THEN
               CURQBIN = 3
            ELSEIF (Q.LT.20.) THEN
               CURQBIN = 4
            ELSEIF (Q.LT.30.) THEN
               CURQBIN = 5
            ELSEIF (Q.LT.50.) THEN
               CURQBIN = 6
            ELSEIF (Q.LT.70.) THEN
               CURQBIN = 7
            ELSE
               CURQBIN = 8
            ENDIF
         ELSE
            GOTO 9990
         ENDIF

*---Detect lepton
         IEL = HZIDELEC(IDUM)
         IF ( (IEL.EQ.-1).OR.
     &        (IEL.LT.1).OR.(IEL.GT.NMXHEP) ) THEN
            WRITE(6,*) 'Hz'//xxxx,' ERROR: lepton not found !!!'
            GOTO 9990
         ENDIF

*---CUT (i): energy of scattered lepton
*----------
         IF (LOWQ.AND.PHEP(4,IEL).LE.14.D0) GOTO 9990
         IF (HIGHQ.AND.PHEP(4,IEL).LE.11.D0) GOTO 9990

*---Get polar angle of scattered lepton
         COSTHETA =  REAL(PHEP(3,IEL)) /
     &        SQRT( REAL(PHEP(1,IEL))**2 + REAL(PHEP(2,IEL))**2
     &        + REAL(PHEP(3,IEL))**2 )
         IF (ABS(COSTHETA).GT.1.) THEN
            WRITE(6,*) 'Hz'//xxxx,' ERROR: abs(cos(thetae)) > 1 !!!'
            GOTO 9990
         ENDIF
         THETAE = RD * ACOS(COSTHETA)

*---CUT (ii): polar angle of scattered lepton
*-----------
         IF (LOWQ) THEN
            IF (THETAE.LE.157..OR.THETAE.GE.173.) THEN
               GOTO 9990
            ENDIF
         ELSEIF (HIGHQ) THEN
            IF (THETAE.LE.30..OR.THETAE.GE.150.) THEN
               GOTO 9990
            ENDIF
         ENDIF

*---Get y
         Y = REAL(HZDISKIN(3))

*---CUT (iii): inelasticity y
*------------
         IF (Y.LE.0.05.OR.Y.GE.0.80) GOTO 9990

*---Get theta_q
         FTQ = Q2*(1.-Y)
         BTQ = 4.*Y**2*EE**2
         CTQ = (FTQ-BTQ)/(FTQ+BTQ)
         THQ = 0.
         IF (ABS(CTQ).LE.1.) THEN
            THQ = RD*ACOS(CTQ)
         ELSEIF (CTQ.LT.-1.) THEN
            THQ = 180.
         ENDIF

*---CUT (iv): angle of 'quark' direction
*-----------
         IF (THQ.LE.20.) GOTO 9990

*---Init boost to Breit frame
         IERR = HZIBEAM(IBEAM,IDUM)
         IF ( (IERR.NE.1).OR.
     &        (IBEAM.LT.1).OR.(IBEAM.GT.NMXHEP).OR.
     &        (IDUM.LT.1).OR.(IDUM.GT.NMXHEP)       ) THEN
            WRITE(6,*) 'HZ'//xxxx,' ERROR: beams not found !!!'
            GOTO 9990
         ELSE
            DO ICOMP = 1,4
               PBEAM(ICOMP)=PHEP(ICOMP,IBEAM)
            ENDDO
         ENDIF

         IERR = HZIPGAM(P5)
         IF (IERR.EQ.-1) THEN
            WRITE(6,*) 'HZ'//xxxx,' ERROR: boson vector not found !!!'
            GOTO 9990
         ELSE
            DO ICOMP = 1,4
               PGAM(ICOMP) = P5(ICOMP)
            ENDDO
         ENDIF

         CALL HZBRTINI (PBEAM,PGAM,IERR)
         IF (IERR.EQ.-1) THEN
            WRITE(6,*)
     &           'HZ'//xxxx,' problem with boost to Breit frame !'
            GOTO 9990
         ENDIF

*---Loop over HEP common
*-----------------------
         IHAD = 0
         EHADFWD = 0.D0
         DO 5000 IHEP = 1,NHEP
*---Skip lepton and unstable particles
            IF ((ISTHEP(IHEP).NE.1).OR.(IHEP.EQ.IEL))  GOTO 5000

*---Get polar angle of hadron
            TEMP =  SQRT( REAL(PHEP(1,IHEP))**2 + REAL(PHEP(2,IHEP))**2
     &           + REAL(PHEP(3,IHEP))**2 )
            IF (TEMP.NE.0.) THEN
               COSTHETA =  REAL(PHEP(3,IHEP)) / TEMP
               IF (ABS(COSTHETA).GT.1.) THEN
                  WRITE(6,*) 'Hz'//xxxx,'ERROR: abs(cos(thetah))>1 !!!'
                  GOTO 9990
               ENDIF
               THETAH = RD * ACOS(COSTHETA)
            ELSE
               THETAH = 0.
            ENDIF

*---Sum up hadronic energy in forward direction
            IF ((THETAH.GT.4.).AND.(THETAH.LT.15.))  THEN
               EHADFWD = EHADFWD + PHEP(4,IHEP)
            ENDIF

*---Perform boost to Breit frame
            DO ICOMP = 1,4
               PLAB(ICOMP) = PHEP(ICOMP,IHEP)
            ENDDO
            CALL HZBRT(PLAB,PBRT,IERR)
            IF (IERR.EQ.-1) THEN
               WRITE(6,*) 'HZ'//xxxx,
     &              ' problem with boost to Breit frame !'
               GOTO 9990
            ENDIF

*---Copy to array suitable for subroutine devshp
            IHAD = IHAD + 1
            PHAD(1,IHAD) = PBRT(4)
            PHAD(2,IHAD) = PBRT(1)
            PHAD(3,IHAD) = PBRT(2)
            PHAD(4,IHAD) = PBRT(3)

 5000    CONTINUE

*---CUT (v): hadronic energy in forward region
*----------
         IF (REAL(EHADFWD).LE.0.5) GOTO 9990


*-------------------------------------
* Calculation of event shape variables
*-------------------------------------

*---Get event shape variables
         CALL DEVSHP(PHAD,IHAD,DBLE(Q2),EVSHPSR,NCURR,ECQ)

*---Copy into internal array
         EVSHPMC(1) = EVSHPSR(3)
         EVSHPMC(2) = EVSHPSR(5)
         EVSHPMC(3) = EVSHPSR(1)
         EVSHPMC(4) = EVSHPSR(7)
         EVSHPMC(5) = EVSHPSR(9)
         EVSHPMC(6) = EVSHPSR(12)
         EVSHPMC(7) = EVSHPSR(10)

*--------------
* Histo filling
*--------------

*---Fill histos and increase counters for number of accepted events
         DO IE=1,NE
            IF (IE.LE.5) THEN
*---CUT (vi): total hadronic energy in Breit current hemisphere
*-----------
               IF (REAL(ECQ).LE.0.1) GOTO 999
*---CUT (vii): no. of hadronic objects in Breit current hemisphere
*------------
               IF (NCURR.LT.2) GOTO 999
            ENDIF
*---Calculate sums for mean values and their standard deviations
            ESSUM(IE,CURQBIN) = ESSUM(IE,CURQBIN)
     &           + DBLE(WTX)*EVSHPMC(IE)
            ESSQSUM(IE,CURQBIN) = ESSQSUM(IE,CURQBIN)
     &           + DBLE(WTX)*(EVSHPMC(IE)**2.)
*---CUT (viii): values of evt.shp.var. in allowed regions (s.a.)
*-------------
            IF (EVSHPMC(IE).GT.LOLIMIT(IE).AND.
     &           EVSHPMC(IE).LT.HILIMIT(IE)) THEN
               CALL HFILL(IE*10+CURQBIN,
     &              REAL(EVSHPMC(IE)),0.,WTX)
               NEVT(IE,CURQBIN) = NEVT(IE,CURQBIN) + WTX
            ELSEIF (EVSHPMC(IE).LE.LOLIMIT(IE)) THEN
               LOCUT(IE,CURQBIN) = LOCUT(IE,CURQBIN) + WTX
            ELSEIF (EVSHPMC(IE).GE.HILIMIT(IE)) THEN
               HICUT(IE,CURQBIN) = HICUT(IE,CURQBIN) + WTX
            ENDIF
 999     ENDDO

 9990    CONTINUE

*********************************************************************
*
*                      Termination
*
*********************************************************************

      ELSEIF (IFLAG.EQ.3) THEN

*---Termination: The following MUST always be done
*   (i) Move to the correct PAW subdirectory
*
         CALL HCDIR('//PAWC/'//XXXX,' ')

*---Write out number of accepted events to histos
         DO IE=1,NE
            DO IQ=1,NQ
               IF (NEVT(IE,IQ).GT.0.) THEN
                  CALL HZHINRM(IE*10+IQ,0,NEVT(IE,IQ),1)
               ENDIF
               CALL HZHINFO(IE*10+IQ,NEVT(IE,IQ))
            ENDDO
         ENDDO

*---Get mean values and errors of event shape variables
         DO IE=1,NE
            DO IQ=1,NQ
               IF (NEVT(IE,IQ).LE.0.) THEN
                  ESMEAN(IE,IQ) = 0.
                  ESMEANE(IE,IQ) = 0.
               ELSE
                  ESMEAN(IE,IQ) = REAL(ESSUM(IE,IQ))
     &                 / NEVT(IE,IQ)
                  IF (NEVT(IE,IQ).GE.2.) THEN
                     ESMEANE(IE,IQ) =
     &                    SQRT(  ( REAL(ESSQSUM(IE,IQ)) -
     &                    REAL(ESSUM(IE,IQ)**2)
     &                    / NEVT(IE,IQ) )
     &                    /((NEVT(IE,IQ)-1)*NEVT(IE,IQ)))
                  ELSE
                     ESMEANE(IE,IQ) = 0.
                     WRITE(6,*)
     &                    'Hz'//xxxx,' only one event in Q-bin No.:',IQ
                  ENDIF
               ENDIF
            ENDDO
         ENDDO

*---Fill mean histos
*---(3rd channel contains zeros)
         DO IE=1,NE
            DO ICHN = 1,2
               ESMTMP(ICHN) = ESMEAN(IE,ICHN)
               ESMETMP(ICHN) = ESMEANE(IE,ICHN)
            ENDDO
            ESMTMP(3) = 0.
            ESMETMP(3) = 0.
            DO ICHN = 4,9
               ESMTMP(ICHN) = ESMEAN(IE,ICHN-1)
               ESMETMP(ICHN) = ESMEANE(IE,ICHN-1)
            ENDDO
            CALL HPAK(IE*10,ESMTMP)
            CALL HPAKE(IE*10,ESMETMP)
         ENDDO

*---Get chi-squares and write them to n-tuple
         DO IE=1,NE
            DO IQ=0,NQ
               CALL HZCHISQ(-(IE*10+IQ),IE*10+IQ)
            ENDDO
         ENDDO

*---Write out statistics
         IF (LP) THEN
            WRITE(6,*)' '
            WRITE(6,*)'Hz'//xxxx,'statistics:'
            WRITE(6,*)'-------------------'
            DO IE=1,NE
               WRITE(6,*) '----- for event shape variable',IE
               DO IQ=1,NQ
                  IF (NEVT(IE,IQ).NE.0.) THEN
                     TEMP = (HICUT(IE,IQ)+LOCUT(IE,IQ))/
     &                    NEVT(IE,IQ)
                  ELSE
                     TEMP = 0.
                  ENDIF
                  WRITE(6,*)'Q bin:',IQ,'   # accepted:',NEVT(IE,IQ)
                  WRITE(6,*)'  # hicut:',HICUT(IE,IQ),
     &                 '   # locut:',LOCUT(IE,IQ),
     &                 '   (# hicut + # locut) / # total :',TEMP
               ENDDO
            ENDDO
         ENDIF
      ENDIF

      RETURN
      END
