*Author : Albert Knutsson, February-2007
      subroutine hzh0512014(iflag)
**********************************************************************
*
* reference: DESY 05-225, Eur.Phys.J.C46:343-356,2006., hep-ex/0512014
*
* purpose: produce histograms for
*	
*	 
*          event shape variables: thrust (T)
*				  thrust with respect to thrust axis (Tc)
*				  jet broadening (B)
*				  jet mass    (rho)
*				  C-parameter (C)
*
*          Q-bins: 
*                  1)  14 < Q <  16 GeV
*                  2)  16 < Q <  20 GeV
*                  3)  20 < Q <  30 GeV
*                  4)  30 < Q <  50 GeV
*                  5)  50 < Q <  70 GeV
*                  6)  70 < Q < 100 GeV
*                  7) 100 < Q < 200 GeV
*
*          event selection:
*          	   i)   196.0 < Q2 < 40000.0 GeV2
*          	   ii)  0.10 < y < 0.70
*          	   iii) total hadronic energy in Breit current
*                       hemisphere > 0.1*Q GeV
*
*
* produced histos:
*
*            distributions: (Qbini = 1-7)
*            ID = 1000 + Qbini:     1/s ds/d(1-Tc)
*            ID = 1010 + Qbini:     1/s ds/d(1-T)
*            ID = 1020 + Qbini:     1/s ds/dB
*            ID = 1030 + Qbini:     1/s ds/d(rho)
*            ID = 1040 + Qbini:     1/s ds/dC
*            mean event shape variables as a function of Q:
*            ID = 1100:  <1-Tc>
*            ID = 1110:  <1-T>
*            ID = 1120:  <B>
*            ID = 1130:  <rho>
*            ID = 1140:  <C>
*
*            H1 data histograms have corresponding negative numbers.
*            Data histos are stored with total errors
*
* Arguments: iflag=1 initialisation
*            iflag=2 filling
*            iflag=3 termination
*
* called subroutines and functions:
*            from hbook lib:  hcdir,hmdir,hbookb,hbook1,
*                             hpak,hpake,hfill
*            from HzTool lib: DGTHRU
*                             HzIdelec,HzIpgam,HzIbeam,
*                             HzBrtini,HzBrt,HzHinrm
*
**********************************************************************

      Implicit None

#include "hzfunc.inc"
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"

      integer iflag
      integer i, j, ii, ij, ihep
      integer idum, iel, ierr, ipbeam, iebeam

      real lumnb, lumpb, Nevent

*histo name
      Character*80 htit

*hep-ex Preprint number
      character*8 xxxx
      data xxxx/'h051201'/

* kinematic variables
      double precision  Ee, The, Q, y
      double precision pbeam(4),ebeam(4), pelec(4), pgam(4)
      double precision pchbrt(4,nmxhep), plab(4),pbrt(4)

* event shapes
      integer ncurhem, nypart, ncurrh
      double precision  tau, tauc, B, rho, C
      double precision  psum, Esum, pxsum, pysum, pzsum, ptsum
      double precision  cpr, costheta, pnorm
      double precision   tausum(7), taucsum(7), Bsum(7), rhosum(7),
     &	   Csum(7), Qsum(7)
      double precision   tausqsum(7), taucsqsum(7), Bsqsum(7), 
     &	   rhosqsum(7), Csqsum(7), Qsqsum(7)
      real   NeventQ(7), Nevtau(7), Nevtauc(7), 
     &			 NevB(7), Nevrho(7), NevC(7)
      real   XsecQ
     
      real   tauav(7), taucav(7), Bav(7), rhoav(7),
     &	 	Cav(7), Qav(7)
      real   tauerr(7), taucerr(7), Berr(7), rhoerr(7), 
     &		Cerr(7)

C------------ /QJMACS/ --- Buffer for jet and topological routines -----
C only  jtbi, qpi(i,j) and  qtbor(1,1) used in this routine
C needed when cacluating thrust axis using DGTHRU.F 
        INTEGER JTBIMX,JTBOMX,JPARMX,JTBI,JTBIT,JTBO,JTBOF
        DOUBLE PRECISION QPI,QTBIR,QPO,QTBOR
        PARAMETER (JTBIMX =10000,JTBOMX = 30,JPARMX = 5)
        COMMON /QCTBUF/QPI(5,JTBIMX),QTBIR(JPARMX,JTBIMX),
     +                  QPO(5,JTBOMX),QTBOR(JPARMX,JTBOMX),
     +                  JTBI,JTBIT(JTBIMX),
     +                  JTBO,JTBOF(JPARMX,JTBIMX)
C     jtbi       number of input vectors (max : jtbimx).
C     qpi(i,j)   particle input vector j; i=1,2,3,4:px,py,pz,e; i=5 free
C     jtbit(j)   pointer to corresponding q-vector
C     qtbir(k,j) scalar input k for each input vector j
C     jtbo       number of output jets   (max : jtbomx)
C     qpo(i,j)   jet output vector (jet axis) i=1,2,3,4:px,py,pz,e
C     jtbof(k,i) number of jet for input particle i in multiplet k
C     qtbor(k,j) scalar output result k for each jet j (k_max : jparmx)
C                (will be stored as user words in Q_vector)

        save taucsum, tausum, bsum, rhosum, csum, neventq, nevtauc, 
     &        nevtau, nevb, nevrho, nevc

*********************************************************
*							*
*                     Binning for histos		*
*						  	*
*********************************************************

      !Q-binning
      real    binQ(8)
      data    binQ /14.0, 16.0, 20.0, 30.0, 50.0, 70.0, 100.0, 200.0/	
      integer ibinQ    !the actually Q bin for the event	
	
      !cross-section histos	
      real    binsxstc9(10)
      data    binsxstc9 /0.0, 0.05, 0.1, 0.15, 
     &			 0.2, 0.25, 0.3, 0.35, 0.4, 0.45/
      real    binsxstc8(9)
      data    binsxstc8 /0.0, 0.05, 0.1, 0.15, 
     &			 0.2, 0.25, 0.3, 0.35, 0.4/
      real    binsxstc7(8)
      data    binsxstc7 /0.0, 0.05, 0.1, 0.15, 
     &			 0.2, 0.25, 0.3, 0.35/
      real    binsxst10(11)
      data    binsxst10 /0.0, 0.1, 0.2, 0.3, 
     &			 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0/
      real    binsxst9(10)
      data    binsxst9 /0.0, 0.1, 0.2, 0.3, 
     &			0.4, 0.5, 0.6, 0.7, 0.8, 0.9/
      real    binsxst7(8)
      data    binsxst7 /0.0, 0.1, 0.2, 0.3, 
     &			0.4, 0.5, 0.6, 0.7/
      real    binsxsb9(10)
      data    binsxsb9 /0.0, 0.1, 0.15, 0.2,
     & 		        0.25, 0.3, 0.35, 0.4, 0.45, 0.5/
      real    binsxsb7(8)
      data    binsxsb7 /0.0, 0.1, 0.15, 0.2,
     & 		        0.25, 0.3, 0.35, 0.4/
      real    binsxsm8(9)
      data    binsxsm8 /0.0, 0.031, 0.062,
     &		       0.093, 0.125, 0.156, 0.187, 0.218, 0.250/
      real    binsxsm7(8)
      data    binsxsm7 /0.0, 0.031, 0.062,
     &		        0.093, 0.125, 0.156, 0.187, 0.218/
      real    binsxsc10(11)
      data    binsxsc10 /0.0, 0.1, 0.2, 0.3, 
     &		         0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0/
      real    binsxsc9(10)
      data    binsxsc9 /0.0, 0.1, 0.2, 0.3, 
     &		        0.4, 0.5, 0.6, 0.7, 0.8, 0.9/

      !upper limits
       double precision taucmaxQ(7),taumaxQ(7), BmaxQ(7), 
     & 	                rhomaxQ(7), CmaxQ(7)
      
      data  taucmaxQ /0.45, 0.45, 0.45, 0.45, 0.4, 0.35, 0.35/
      data  taumaxQ  /1.0,  1.0,  1.0,  1.0,  0.9, 0.9,  0.7/
      data  BmaxQ    /0.5,  0.5,  0.5,  0.5,  0.5, 0.5,  0.4/
      data  rhomaxQ   /0.250,0.250,0.250,0.250,0.250,0.250,0.218/ 
      data  CmaxQ    /1.0,  1.0,  1.0,  1.0,  1.0, 1.0,  0.9/
      
*********************************************************
*							*
*                     H1 Data Points			*
*						  	*
*********************************************************

*****Cross-sections as a function of event shape variables
*****for different Q

      real Xsec1(9), Xsec2(9), Xsec3(9), Xsec4(9), Xsec5(8), Xsec6(8),
     & 	   Xsec7(7), Xsec8(10), Xsec9(10), Xsec10(10), Xsec11(10), 
     &     Xsec12(10), Xsec13(9), Xsec14(7), Xsec15(9), Xsec16(9), 
     &     Xsec17(9), Xsec18(9), Xsec19(9), Xsec20(9), Xsec21(8), 
     &     Xsec22(8), Xsec23(8), Xsec24(8), Xsec25(8), Xsec26(8), 
     &     Xsec27(8), Xsec28(8), Xsec29(10), Xsec30(10), Xsec31(10), 
     &     Xsec32(10), Xsec33(10), Xsec34(10), Xsec35(9)

      real Err1(9), Err2(9), Err3(9), Err4(9), Err5(8), Err6(8), 
     &     Err7(7), Err8(10), Err9(10), Err10(10), Err11(10), 
     &     Err12(10), Err13(9), Err14(7), Err15(9), Err16(9), 
     &     Err17(9), Err18(9), Err19(9), Err20(9), Err21(8), 
     &     Err22(8), Err23(8), Err24(8), Err25(8), Err26(8), 
     &     Err27(8), Err28(8), Err29(10), Err30(10), Err31(10), 
     &     Err32(10), Err33(10), Err34(10), Err35(9)


      Data Xsec1 /0.1499, 0.2723, 0.2399, 0.1545, 0.0933, 0.0558,
     &						0.0268, 0.0069, 0.0005/
      Data Err1 /0.0087, 0.0126, 0.0071, 0.0066, 0.0063, 0.0043,
     &						 0.0037, 0.0013, 0.0002/

      Data Xsec2 /0.1697, 0.3008, 0.2333, 0.1388, 0.0800, 0.0472,
     &						 0.0238, 0.0060, 0.0005/
      Data Err2 /0.0075, 0.0157, 0.0062, 0.0060, 0.0062, 0.0045,
     & 						 0.0024, 0.0013, 0.0003/

      Data Xsec3 /0.2300, 0.3295, 0.2049, 0.1127, 0.0661, 0.0370,
     & 						0.0164, 0.0032, 0.0003/
      Data Err3 /0.0087, 0.0127, 0.0056, 0.0073, 0.0063, 0.0032, 
     &						0.0027, 0.0007, 0.0003/

      Data Xsec4 /0.3303, 0.3280, 0.1551, 0.0845, 0.0533, 0.0312, 
     &						0.0145, 0.0029, 0.0002/
      Data Err4 /0.0140, 0.0127, 0.0066, 0.0050, 0.0053, 0.0041, 
     &						0.0024, 0.0009, 0.0003/

      Data Xsec5 /0.4530, 0.2792, 0.1281, 0.0779, 0.0388, 0.0173, 
     &						0.0053, 0.0005/
      Data Err5 /0.0423, 0.0230, 0.0161, 0.0109, 0.0082, 0.0040, 
     &						0.0025, 0.0008/

      Data Xsec6 /0.5946, 0.1953, 0.0902, 0.0575, 0.0379, 0.0182, 
     &						0.0047, 0.0016/
      Data Err6 /0.0556, 0.0257, 0.0147, 0.0130, 0.0093, 0.0056, 
     & 						0.0030, 0.0018/

      Data Xsec7 /0.6520, 0.0920, 0.0946, 0.0421, 0.1011,
     & 						 0.0175, 0.0007/
      Data Err7 /0.1640, 0.0623, 0.0420, 0.0303, 0.0502, 
     & 						0.0132, 0.0007/

      Data Xsec8 /0.1383, 0.2644, 0.1872, 0.1224, 0.0815, 
     & 	          0.0654, 0.0548, 0.0449, 0.0302, 0.0109/
      Data Err8 /0.0280, 0.0273, 0.0070, 0.0087, 0.0079,
     & 		 0.0088, 0.0067, 0.0093, 0.0145, 0.0143/

      Data Xsec9 /0.1897, 0.2801, 0.1673, 0.1058, 0.0759, 
     & 		  0.0613, 0.0458, 0.0371, 0.0280, 0.0091/
      Data Err9 /0.0275, 0.0132, 0.0052, 0.0073, 0.0055, 
     &            0.0031, 0.0039, 0.0052, 0.0144, 0.0143/

      Data Xsec10 /0.2949, 0.2875, 0.1376, 0.0851, 0.0601, 
     &              0.0416, 0.0342, 0.0298, 0.0208, 0.0083/
      Data Err10 /0.0291, 0.0073, 0.0052, 0.0040, 0.0038, 
     &             0.0034, 0.0025, 0.0054, 0.0144, 0.0178/

      Data Xsec11 /0.4536, 0.2525, 0.1091, 0.0604, 0.0403, 
     &              0.0287, 0.0202, 0.0167, 0.0140, 0.0045/
      Data Err11 /0.0313, 0.0120, 0.0059, 0.0068, 0.0049, 
     & 		  0.0041, 0.0035, 0.0051, 0.0143, 0.0075/

      Data Xsec12 /0.6102, 0.2119, 0.0732, 0.0363, 0.0234, 
     &              0.0215, 0.0120, 0.0062, 0.0047, 0.0007/
      Data Err12 /0.0444, 0.0212, 0.0094, 0.0067, 0.0054, 
     &             0.0054, 0.0040, 0.0038, 0.0029, 0.0006/

      Data Xsec13 /0.7457, 0.1300, 0.0614, 0.0333, 0.0142, 
     &              0.0072, 0.0039, 0.0024, 0.0018/
      Data Err13 /0.0644, 0.0188, 0.0128, 0.0091, 0.0059, 
     &             0.0040, 0.0030, 0.0019, 0.0042/

      Data Xsec14 /0.6526, 0.1401, 0.1377, 0.0350, 
     &                      0.0093, 0.0242, 0.0011/
      Data Err14 /0.1732, 0.0589, 0.0629, 0.0273, 
     &                     0.0077, 0.0220, 0.0013/

      Data Xsec15 /0.0162, 0.0714, 0.1366, 0.1575, 
     &              0.1488, 0.1317, 0.1124, 0.1066, 0.1188/
      Data Err15 /0.0054, 0.0162, 0.0214, 0.0121, 
     &             0.0071, 0.0090, 0.0070, 0.0116, 0.0360/

      Data Xsec16 /0.0232, 0.1022, 0.1640, 0.1549, 
     &              0.1345, 0.1171, 0.1029, 0.0988, 0.1025/
      Data Err16 /0.0056, 0.0147, 0.0151, 0.0051, 
     & 		  0.0054, 0.0072, 0.0065, 0.0046, 0.0293/

      Data Xsec17 /0.0553, 0.1598, 0.1822, 0.1561, 
     &              0.1198, 0.0890, 0.0811, 0.0752, 0.0813/
      Data Err17 /0.0101, 0.0154, 0.0070, 0.0085, 
     &             0.0050, 0.0056, 0.0048, 0.0048, 0.0288/

      Data Xsec18 /0.1577, 0.2178, 0.1732, 0.1246, 
     &              0.0989, 0.0753, 0.0557, 0.0455, 0.0513/
      Data Err18 /0.0188, 0.0157, 0.0120, 0.0063, 
     &             0.0088, 0.0072, 0.0069, 0.0062, 0.0232/

      Data Xsec19 /0.3276, 0.2215, 0.1470, 0.1056, 
     &              0.0604, 0.0467, 0.0368, 0.0347, 0.0198/
      Data Err19 /0.0534, 0.0282, 0.0191, 0.0135, 
     &             0.0085, 0.0087, 0.0079, 0.0083, 0.0071/

      Data Xsec20 /0.5151, 0.1791, 0.0969, 0.0609, 
     &              0.0609, 0.0367, 0.0309, 0.0135, 0.0061/
      Data Err20 /0.0659, 0.0331, 0.0157, 0.0131, 
     &             0.0134, 0.0100, 0.0103, 0.0066, 0.0051/

      Data Xsec21 /0.4766, 0.1196, 0.0963, 0.0861, 
     &              0.0856, 0.0960, 0.0394, 0.0003/
      Data Err21 /0.1735, 0.0597, 0.0491, 0.0470, 
     &             0.0474, 0.0564, 0.0328, 0.0003/

      Data Xsec22 /0.1918, 0.3348, 0.2161, 0.1181, 
     &              0.0670, 0.0408, 0.0240, 0.0073/
      Data Err22 /0.0158, 0.0168, 0.0059, 0.0071, 
     &             0.0087, 0.0062, 0.0056, 0.0030/

      Data Xsec23 /0.2283, 0.3479, 0.1925, 0.1049, 
     &              0.0588, 0.0371, 0.0235, 0.0070/
      Data Err23 /0.0194, 0.0122, 0.0052, 0.0076, 
     &             0.0061, 0.0076, 0.0066, 0.0028/

      Data Xsec24 /0.2947, 0.3554, 0.1602, 0.0833, 
     &              0.0460, 0.0308, 0.0205, 0.0091/
      Data Err24 /0.0173, 0.0124, 0.0057, 0.0057, 
     &             0.0062, 0.0052, 0.0059, 0.0049/

      Data Xsec25 /0.4173, 0.3078, 0.1175, 0.0666, 
     &              0.0401, 0.0282, 0.0164, 0.0060/
      Data Err25 /0.0202, 0.0114, 0.0053, 0.0066, 
     &             0.0054, 0.0069, 0.0068, 0.0039/

      Data Xsec26 /0.5389, 0.2592, 0.1001, 0.0495, 
     &              0.0198, 0.0173, 0.0138, 0.0014/
      Data Err26 /0.0435, 0.0242, 0.0124, 0.0091, 
     &             0.0049, 0.0053, 0.0050, 0.0011/

      Data Xsec27 /0.6703, 0.1673, 0.0703, 0.0474, 
     &              0.0304, 0.0094, 0.0032, 0.0018/
      Data Err27 /0.0576, 0.0224, 0.0127, 0.0100, 
     &             0.0085, 0.0040, 0.0023, 0.0014/

      Data Xsec28 /0.6474, 0.1253, 0.0502, 0.1026, 
     &              0.0382, 0.0064, 0.0291, 0.0008/
      Data Err28 /0.1716, 0.0508, 0.0350, 0.0535, 
     &             0.0290, 0.0051, 0.0251, 0.0009/

      Data Xsec29 /0.0333, 0.0765, 0.1205, 0.1583, 0.1675, 
     &              0.1561, 0.1222, 0.0954, 0.0576, 0.0125/
      Data Err29 /0.0064, 0.0069, 0.0115, 0.0085, 0.0085, 
     &             0.0065, 0.0061, 0.0079, 0.0055, 0.0017/

      Data Xsec30 /0.0377, 0.0888, 0.1421, 0.1735, 0.1679, 
     &              0.1396, 0.1073, 0.0813, 0.0511, 0.0107/
      Data Err30 /0.0057, 0.0061, 0.0133, 0.0097, 0.0049, 
     &             0.0049, 0.0066, 0.0060, 0.0055, 0.0016/

      Data Xsec31 /0.0463, 0.1235, 0.1837, 0.1891, 0.1523, 
     &              0.1119, 0.0819, 0.0663, 0.0389, 0.0061/
      Data Err31 /0.0070, 0.0083, 0.0114, 0.0084, 0.0054, 
     &             0.0053, 0.0056, 0.0061, 0.0050, 0.0012/

      Data Xsec32 /0.0667, 0.2093, 0.2194, 0.1615, 0.1095, 
     &              0.0783, 0.0649, 0.0564, 0.0294, 0.0047/
      Data Err32 /0.0075, 0.0121, 0.0115, 0.0071, 0.0056, 
     &             0.0044, 0.0044, 0.0061, 0.0040, 0.0010/

      Data Xsec33 /0.1384, 0.2782, 0.1916, 0.1250, 0.0949, 
     &              0.0717, 0.0539, 0.0340, 0.0105, 0.0018/
      Data Err33 /0.0246, 0.0304, 0.0179, 0.0151, 0.0122, 
     &             0.0109, 0.0095, 0.0062, 0.0035, 0.0016/

      Data Xsec34 /0.2375, 0.3227, 0.1469, 0.0782, 0.0572, 
     &              0.0489, 0.0462, 0.0458, 0.0150, 0.0016/
      Data Err34 /0.0368, 0.0367, 0.0207, 0.0155, 0.0114, 
     &             0.0107, 0.0119, 0.0102, 0.0049, 0.0015/

      Data Xsec35 /0.2247, 0.3999, 0.0994, 0.0354, 0.0275, 
     &              0.0558, 0.0351, 0.1166, 0.0056/
      Data Err35 /0.1569, 0.1155, 0.0489, 0.0242, 0.0198, 
     &             0.0303, 0.0258, 0.0575, 0.0043/



*****Events shape variables as a function of Q
      real Q1(7), Q2(7), Q3(7), Q4(7), Q5(7)

      real Mean1(7), Mean2(7), Mean3(7), Mean4(7), Mean5(7)

      real ErrMean1(7), ErrMean2(7), ErrMean3(7),
     & 			ErrMean4(7), ErrMean5(7)

      Data Q1 /14.9, 17.7, 23.8, 36.9, 57.6, 80.6, 115.6/
      Data Mean1 /0.129, 0.122, 0.108, 0.093, 0.076, 0.065, 0.065/
      Data ErrMean1 /0.004, 0.004, 0.003, 0.003, 0.005, 0.005, 0.013/

      Data Q2 /14.9, 17.7, 23.8, 36.9, 57.6, 80.6, 115.6/
      Data Mean2 /0.299, 0.275, 0.232, 0.173, 0.124, 0.092, 0.095/
      Data ErrMean2 /0.026, 0.018, 0.015, 0.011, 0.008, 0.007, 0.020/

      Data Q3 /14.9, 17.7, 23.8, 36.9, 57.6, 80.6, 115.6/
      Data Mean3 /0.290, 0.275, 0.247, 0.205, 0.165, 0.135, 0.137/
      Data ErrMean3 /0.025, 0.017, 0.015, 0.014, 0.016, 0.014, 0.032/

      Data Q4 /14.9, 17.7, 23.8, 36.9, 57.6, 80.6, 115.6/
      Data Mean4 /0.070, 0.066, 0.060, 0.051, 0.041, 0.034, 0.038/
      Data ErrMean4 /0.003, 0.003, 0.003, 0.002, 0.003, 0.002, 0.008/


      Data Q5 /14.9, 17.7, 23.8, 36.9, 57.6, 80.6, 115.6/
      Data Mean5 /0.473, 0.448, 0.406, 0.352, 0.292, 0.252, 0.248/
      Data ErrMean5 /0.020, 0.018, 0.016, 0.012, 0.021, 0.018, 0.056/


*****Events shape variables as a function of Q
*
* remind:
*     HistoIDs:  ID < 0  for H1 data
*                ID > 0  for corresponding generator histos

 	

*********************************************************
*							*
*                      Initialization			*
*							*
*********************************************************

      if (iflag.eq.1) then

***** create directory
         call hcdir('//PAWC',' ')
         call hmdir(xxxx,'S')
         call hcdir('//HISTO',' ')
         call hmdir(xxxx,'S')

	 WRITE(6,*)'***************************************************'
	 WRITE(6,*)'HZ'//xxxx//' called'
     	 WRITE(6,*)'***************************************************'

*****Create and Fill H1 histos
	
	! thrust_c - xsec
        call hbookb(-1001,'t_c - Q1', 9, binsxstc9, 0.0)
        call hbookb(-1002,'t_c - Q2', 9, binsxstc9, 0.0)
        call hbookb(-1003,'t_c - Q3', 9, binsxstc9, 0.0)
        call hbookb(-1004,'t_c - Q4', 9, binsxstc9, 0.0)	
        call hbookb(-1005,'t_c - Q5', 8, binsxstc8, 0.0)
        call hbookb(-1006,'t_c - Q6', 7, binsxstc7, 0.0)
        call hbookb(-1007,'t_c - Q7', 7, binsxstc7, 0.0)
        call hpak (-1001,Xsec1)
        call hpak (-1002,Xsec2)
        call hpak (-1003,Xsec3)
        call hpak (-1004,Xsec4)
        call hpak (-1005,Xsec5)
        call hpak (-1006,Xsec6)
        call hpak (-1007,Xsec7)
        call hpake (-1001,Err1)
        call hpake (-1002,Err2)
        call hpake (-1003,Err3)
        call hpake (-1004,Err4)
        call hpake (-1005,Err5)
        call hpake (-1006,Err6)
        call hpake (-1007,Err7)

	! thrust - xsec 
        call hbookb(-1011,'t - Q1', 10, binsxst10, 0.0)
        call hbookb(-1012,'t - Q2', 10, binsxst10, 0.0)
        call hbookb(-1013,'t - Q3', 10, binsxst10, 0.0)
        call hbookb(-1014,'t - Q4', 10, binsxst10, 0.0)	
        call hbookb(-1015,'t - Q5', 9,  binsxst9, 0.0)
        call hbookb(-1016,'t - Q6', 9,  binsxst9, 0.0)
        call hbookb(-1017,'t - Q7', 7,  binsxst7, 0.0)
        call hpak (-1011,Xsec8)
        call hpak (-1012,Xsec9)
        call hpak (-1013,Xsec10)
        call hpak (-1014,Xsec11)
        call hpak (-1015,Xsec12)
        call hpak (-1016,Xsec13)
        call hpak (-1017,Xsec14)
        call hpake (-1011,Err8)
        call hpake (-1012,Err9)
        call hpake (-1013,Err10)
        call hpake (-1014,Err11)
        call hpake (-1015,Err12)
        call hpake (-1016,Err13)
        call hpake (-1017,Err14)
		
	! Broadening - xsec 
        call hbookb(-1021,'B - Q1', 9, binsxsb9, 0.0)
        call hbookb(-1022,'B - Q2', 9, binsxsb9, 0.0)
        call hbookb(-1023,'B - Q3', 9, binsxsb9, 0.0)
        call hbookb(-1024,'B - Q4', 9, binsxsb9, 0.0)	
        call hbookb(-1025,'B - Q5', 9, binsxsb9, 0.0)
        call hbookb(-1026,'B - Q6', 9, binsxsb9, 0.0)
        call hbookb(-1027,'B - Q7', 7, binsxsb7, 0.0)
        call hpak (-1021,Xsec15)
        call hpak (-1022,Xsec16)
        call hpak (-1023,Xsec17)
        call hpak (-1024,Xsec18)
        call hpak (-1025,Xsec19)
        call hpak (-1026,Xsec20)
        call hpak (-1027,Xsec21)
        call hpake (-1021,Err15)
        call hpake (-1022,Err16)
        call hpake (-1023,Err17)
        call hpake (-1024,Err18)
        call hpake (-1025,Err19)
        call hpake (-1026,Err20)
        call hpake (-1027,Err21)
	
	! Rho - xsec
        call hbookb(-1031,'rho - Q1', 8,  binsxsm8, 0.0)
        call hbookb(-1032,'rho - Q2', 8,  binsxsm8, 0.0)
        call hbookb(-1033,'rho - Q3', 8,  binsxsm8, 0.0)
        call hbookb(-1034,'rho - Q4', 8,  binsxsm8, 0.0)	
        call hbookb(-1035,'rho - Q5', 8,  binsxsm8, 0.0)
        call hbookb(-1036,'rho - Q6', 8,  binsxsm8, 0.0)
        call hbookb(-1037,'rho - Q7', 7,  binsxsm7, 0.0)
        call hpak (-1031,Xsec22)
        call hpak (-1032,Xsec23)
        call hpak (-1033,Xsec24)
        call hpak (-1034,Xsec25)
        call hpak (-1035,Xsec26)
        call hpak (-1036,Xsec27)
        call hpak (-1037,Xsec28)
        call hpake (-1031,Err22)
        call hpake (-1032,Err23)
        call hpake (-1033,Err24)
        call hpake (-1034,Err25)
        call hpake (-1035,Err26)
        call hpake (-1036,Err27)
        call hpake (-1037,Err28)
	
	! C-parameter - xsec	
        call hbookb(-1041,'C - Q1', 10,  binsxsc10, 0.0)
        call hbookb(-1042,'C - Q2', 10,  binsxsc10, 0.0)
        call hbookb(-1043,'C - Q3', 10,  binsxsc10, 0.0)
        call hbookb(-1044,'C - Q4', 10,  binsxsc10, 0.0)	
        call hbookb(-1045,'C - Q5', 10,  binsxsc10, 0.0)
        call hbookb(-1046,'C - Q6', 10,  binsxsc10, 0.0)
        call hbookb(-1047,'C - Q7', 9,   binsxsc9, 0.0)
        call hpak (-1041,Xsec29)
        call hpak (-1042,Xsec30)
        call hpak (-1043,Xsec31)
        call hpak (-1044,Xsec32)
        call hpak (-1045,Xsec33)
        call hpak (-1046,Xsec34)
        call hpak (-1047,Xsec35)
        call hpake (-1041,Err29)
        call hpake (-1042,Err30)
        call hpake (-1043,Err31)
        call hpake (-1044,Err32)
        call hpake (-1045,Err33)
        call hpake (-1046,Err34)
        call hpake (-1047,Err35)



	!mean values
        call hbookb(-1100,'mean(Tc)', 7,  binQ, 0.0)
        call hbookb(-1110,'mean(T)', 7,  binQ, 0.0)
        call hbookb(-1120,'mean(B)', 7,  binQ, 0.0)
        call hbookb(-1130,'mean(Rho)', 7,  binQ, 0.0)	
        call hbookb(-1140,'mean(C)', 7,  binQ, 0.0)
        call hpak (-1100,Mean1)
        call hpak (-1110,Mean2)
        call hpak (-1120,Mean3)
        call hpak (-1130,Mean4)
        call hpak (-1140,Mean5)
        call hpake (-1100,ErrMean1)
        call hpake (-1110,ErrMean2)
        call hpake (-1120,ErrMean3)
        call hpake (-1130,ErrMean4)
        call hpake (-1140,ErrMean5)
	

*****book cross-sections histos for different Q

	! thrust_c 
        call hbookb(1001,'t_c - Q1', 9, binsxstc9, 0.0)
        call hbookb(1002,'t_c - Q2', 9, binsxstc9, 0.0)
        call hbookb(1003,'t_c - Q3', 9, binsxstc9, 0.0)
        call hbookb(1004,'t_c - Q4', 9, binsxstc9, 0.0)	
        call hbookb(1005,'t_c - Q5', 8, binsxstc8, 0.0)
        call hbookb(1006,'t_c - Q6', 7, binsxstc7, 0.0)
        call hbookb(1007,'t_c - Q7', 7, binsxstc7, 0.0)

	! thrust 
        call hbookb(1011,'t - Q1', 10, binsxst10, 0.0)
        call hbookb(1012,'t - Q2', 10, binsxst10, 0.0)
        call hbookb(1013,'t - Q3', 10, binsxst10, 0.0)
        call hbookb(1014,'t - Q4', 10, binsxst10, 0.0)	
        call hbookb(1015,'t - Q5', 9,  binsxst9, 0.0)
        call hbookb(1016,'t - Q6', 9,  binsxst9, 0.0)
        call hbookb(1017,'t - Q7', 7,  binsxst7, 0.0)
		
	! Broadening 
        call hbookb(1021,'B - Q1', 9, binsxsb9, 0.0)
        call hbookb(1022,'B - Q2', 9, binsxsb9, 0.0)
        call hbookb(1023,'B - Q3', 9, binsxsb9, 0.0)
        call hbookb(1024,'B - Q4', 9, binsxsb9, 0.0)	
        call hbookb(1025,'B - Q5', 9, binsxsb9, 0.0)
        call hbookb(1026,'B - Q6', 9, binsxsb9, 0.0)
        call hbookb(1027,'B - Q7', 7, binsxsb7, 0.0)
	
	! Rho
        call hbookb(1031,'rho - Q1', 8,  binsxsm8, 0.0)
        call hbookb(1032,'rho - Q2', 8,  binsxsm8, 0.0)
        call hbookb(1033,'rho - Q3', 8,  binsxsm8, 0.0)
        call hbookb(1034,'rho - Q4', 8,  binsxsm8, 0.0)	
        call hbookb(1035,'rho - Q5', 8,  binsxsm8, 0.0)
        call hbookb(1036,'rho - Q6', 8,  binsxsm8, 0.0)
        call hbookb(1037,'rho - Q7', 7,  binsxsm7, 0.0)
	
	! C-parameter	
        call hbookb(1041,'C - Q1', 10,  binsxsc10, 0.0)
        call hbookb(1042,'C - Q2', 10,  binsxsc10, 0.0)
        call hbookb(1043,'C - Q3', 10,  binsxsc10, 0.0)
        call hbookb(1044,'C - Q4', 10,  binsxsc10, 0.0)	
        call hbookb(1045,'C - Q5', 10,  binsxsc10, 0.0)
        call hbookb(1046,'C - Q6', 10,  binsxsc10, 0.0)
        call hbookb(1047,'C - Q7', 9,   binsxsc9, 0.0)
	
*****book histos for average values as a function of Q
        do i=0,4
	  call hbookb(1100+i*10,'', 7,  binQ, 0.0)
	enddo
	call hbookb(1200,'Q', 7,  binQ, 0.0)
 	
***** Init sums for average values
	do i=1,7
	  taucsum(i)= 0.0
          tausum(i) = 0.0
	  Bsum(i)   = 0.0
	  rhosum(i) = 0.0
	  Csum(i)   = 0.0
	
	  NeventQ(i) = 0.0
	  Nevtauc(i) = 0.0
	  Nevtau(i)  = 0.0
	  NevB(i)    = 0.0
	  Nevrho(i)  = 0.0
	  NevC(i)    = 0.0
	enddo 
	 
*********************************************************
*							*
*                      Event Processing			*
*							*
*********************************************************

      else if (iflag.eq.2) then

***** move to the correct sub-directory in PAWC
         call hcdir('//PAWC/'//xxxx,' ')

***** event counter
	Nevent=Nevent+wtx

***** get the beams
        ierr=HZIBEAM(ipbeam,iebeam)
        if (.not.(ierr.eq.1)) then
          write(6,*) 'HZ'//xxxx,' beams not found ! '
          return
        else
	  do i=1,4
	    pbeam(i)=PHEP(i,ipbeam)
	    ebeam(i)=PHEP(i,iebeam)
	  enddo
	endif

***** get the scattered lepton
        iel=HZIDELEC(idum)
        if (iel.eq.-1) then
           write(*,*) 'Hz'//xxxx,' Sc.electron not found!'
           return
        else
	  do i=1,4
	    pelec(i)=PHEP(i,iel)
	  enddo
        endif


***** get Q and y 
        Q = sqrt(HzDiskin(1)) 
        y = HZDISKIN(3)
*        Ee=PHEP(4,iel)
*	The=HZPHMANG(PHEP(3,iel), sqrt(PHEP(1,iel)**2+PHEP(2,iel)
*     +  **2))
*        Q = sqrt(4*Ee*pelec(4)*cos(The/2.)*cos(The/ 2.))
*        y = (pelec(4)-(Ee*sin(The/2.)*sin(The/ 2.)))/pelec(4)

***** cut	
	if(Q.lt.14.or.Q.gt.200.) goto 777
	if(y.lt.0.1.or.y.gt.0.7) goto 777
	
***** determine Q bin
	do i=1,7
	  if(Q.gt.dble(binQ(i)).and.Q.le.dble(binQ(i+1))) ibinQ = i	 
	enddo

***** get the boson vector
          ierr=HZIPGAM(pgam)            
          if (ierr.eq.-1) then
            write(6,*) 'HZ'//xxxx,' boson vector not found ! '
            return
          endif

***** init boost to Breit frame
*         print*,'pbeam',pbeam(1),pbeam(2),pbeam(3),pbeam(4)
*         print*,'pgam',pgam(1),pgam(2),pgam(3),pgam(4)
         call HzBrtini(pbeam, pgam, ierr)
         if (ierr.eq.-1) then
            write(6,*) 'HZ'//xxxx,' problem with boost to Breit frame !'
            return
         endif
	

***** boost particles to breit and save current hemisphere particles in array
	ncurhem=0
	nypart=0
	do ihep=1,nhep
	  
	  !only stable particles, and not the lepton or beams
	  if(isthep(ihep).ne.1) goto 666
	  if(ihep.eq.iel) goto 666
          if(ihep.eq.ipbeam.or.ihep.eq.iebeam) goto 666
	
	  !boost particle to brt
          do i = 1, 4
             plab(i) = PHEP(i, ihep)
          enddo
          call HzBrt(plab, pbrt, ierr)

          if (ierr.eq.-1) then
             write(6,*) 'HZ'//xxxx,
     &            ' problem with boost to Breit frame !'
             return
          endif
		  
	  !only use current hemisphere
	  if (.not.(pbrt(3).lt.0.d0)) goto 666
	    ncurhem=ncurhem+1	  
	    do i=1,4
	      pchbrt(i,ncurhem)=pbrt(i)
	      QPI(i,ncurhem)=pbrt(i)
	    enddo 
	  	    
	
666     CONTINUE
	enddo
	
	
*****loop particles in current hemisphere, calculate event shape variables
	psum=0
	Esum=0
	pxsum=0
	pysum=0
	pzsum=0
	ptsum=0
	cpr=0
	do ii=1,ncurhem

	  
	  !p-sum	  
	  psum=psum + sqrt(pchbrt(1,ii)*pchbrt(1,ii)+
     &		           pchbrt(2,ii)*pchbrt(2,ii)+
     &		           pchbrt(3,ii)*pchbrt(3,ii))
	  !px-sum
	  pxsum=pxsum+pchbrt(1,ii)
	  
	  !pz-sum
	  pysum=pysum+pchbrt(2,ii)
	  
	  !pz-sum
	  pzsum=pzsum+pchbrt(3,ii)

	  !pt-sum
	  ptsum=ptsum+sqrt(pchbrt(1,ii)*pchbrt(1,ii)+
     &		           pchbrt(2,ii)*pchbrt(2,ii))

	  !E-sum
	  Esum=Esum+pchbrt(4,ii)


	  !polar angel-momentum sum for C-Parameter 	
          do ij = ii+1,ncurhem          
               pnorm = 
     &	         sqrt(pchbrt(1,ii)**2+pchbrt(2,ii)**2+pchbrt(3,ii)**2)*
     &           sqrt(pchbrt(1,ij)**2 +pchbrt(2,ij)**2 +pchbrt(3,ij)**2)               
	       costheta =
     &	         (pchbrt(1,ii)*pchbrt(1,ij)
     &           +pchbrt(2,ii)*pchbrt(2,ij)
     &           +pchbrt(3,ii)*pchbrt(3,ij))  /  pnorm
               cpr = cpr + pnorm*(1.e0-costheta**2)
          enddo

	enddo !end particle loop


***** apply cut	
	if (Esum.gt.Q/10) then

***** calculate event shape variables
          tau=1-ABS(pzsum)/psum
	  B=abs(ptsum)/(2*psum)
	  rho=(PSUM*PSUM-PXSUM*PXSUM-PYSUM*PYSUM-PZSUM*PZSUM) / 
     &      (4.E0*PSUM**2)
	  C=3.0 * cpr /(PSUM**2)   !--- factor of half cancelled by not doing symmetrized sum

***** calculate tau_c by using dgthru (with the commonblock)
          JTBI=ncurhem
	  CALL DGTHRU
	  tauc=1.0 - QTBOR(1,1)

***** fill histos      
	  call hfill(1000+ibinQ, real(tauc),0.0, wtx)			
	  call hfill(1010+ibinQ, real(tau), 0.0, wtx)
	  call hfill(1020+ibinQ, real(B),   0.0, wtx)
	  call hfill(1030+ibinQ, real(rho), 0.0, wtx)
	  call hfill(1040+ibinQ, real(C),   0.0, wtx)			

***** calc sums for average 	
	  taucsum(ibinQ)= taucsum(ibinQ) + tauc*dble(wtx)
	  tausum(ibinQ) = tausum(ibinQ)  + tau*dble(wtx)
	  Bsum(ibinQ)   = Bsum(ibinQ)    + B*dble(wtx)
	  rhosum(ibinQ) = rhosum(ibinQ)  + rho*dble(wtx)
	  Csum(ibinQ)   = Csum(ibinQ)    + C*dble(wtx)			  
	  Qsum(ibinQ)   = Qsum(ibinQ)	 + Q*dble(wtx)

***** calc square sums for error 	
	  taucsqsum(ibinQ)= taucsqsum(ibinQ) + (tauc**2)*dble(wtx)
	  tausqsum(ibinQ) = tausqsum(ibinQ)  + (tau**2)*dble(wtx)
	  Bsqsum(ibinQ)   = Bsqsum(ibinQ)    + (B**2)*dble(wtx)
	  rhosqsum(ibinQ) = rhosqsum(ibinQ)  + (rho**2)*dble(wtx)
	  Csqsum(ibinQ)   = Csqsum(ibinQ)    + (C**2)*dble(wtx)			  
	  Qsqsum(ibinQ)   = Qsqsum(ibinQ)    + (Q**2)*dble(wtx)

*****  calc number of events for each variable and Q bin	
	  NeventQ(ibinQ)= NeventQ(ibinQ) + dble(wtx)
	  
	  if (tauc.gt.0.0.and.tauc.lt.taucmaxQ(ibinQ)) 
     &			Nevtauc(ibinQ)= Nevtauc(ibinQ) + wtx
     
	  if (tau.gt.0.0.and.tau.lt.taumaxQ(ibinQ)) 
     &			Nevtau(ibinQ)= Nevtau(ibinQ) + wtx
     
	  if (B.gt.0.0.and.B.lt.BmaxQ(ibinQ)) 
     &			NevB(ibinQ)= NevB(ibinQ) + wtx
     
	  if (rho.gt.0.0.and.rho.lt.rhomaxQ(ibinQ)) 
     &			Nevrho(ibinQ)= Nevrho(ibinQ) + wtx
     
	  if (C.gt.0.0.and.C.lt.CmaxQ(ibinQ)) 
     &			NevC(ibinQ)= NevC(ibinQ) + wtx

	
	endif


777 	CONTINUE

*********************************************************
*							*
*                      Termination			*
*						    	*
*********************************************************

      else if(iflag.eq.3) then

***** move to the correct sub-directory in PAWC
        call hcdir('//PAWC/'//xxxx,' ')         

***** calc avegare event shape variabels for different Q bins
        do i=1,7
          
	  !values
	  taucav(i)=real(taucsum(i)/Nevtauc(i))
          tauav(i)=real(tausum(i)/Nevtau(i))
          Bav(i)=real(Bsum(i)/NevB(i))
          rhoav(i)=real(rhosum(i)/Nevrho(i))
          Cav(i)=real(Csum(i)/NevC(i))
          Qav(i)=real(Qsum(i)/NeventQ(i))
        
	  !errors	  
 	  taucerr(i)=real((taucsqsum(i)/NeventQ(i)-taucav(i)**2)/
     &							(NeventQ(i)-1))
          tauerr(i)=real((tausqsum(i)/NeventQ(i)-tauav(i)**2)/
     &							(NeventQ(i)-1))
          Berr(i)=real((Bsqsum(i)/NeventQ(i)-Bav(i)**2)/(NeventQ(i)-1))
	  rhoerr(i)=real((rhosqsum(i)/NeventQ(i)-rhoav(i)**2)/
     &							(NeventQ(i)-1))
          Cerr(i)=real((Csqsum(i)/NeventQ(i)-Cav(i)**2)/(NeventQ(i)-1))
		
	enddo


***** Fill average values and errors
	call hpak(1100, taucav)
        call hpak(1110, tauav)
        call hpak(1120, Bav)
        call hpak(1130, rhoav)
        call hpak(1140, Cav)
        call hpake(1100, taucerr)
        call hpake(1110, tauerr)
        call hpake(1120, Berr)
        call hpake(1130, rhoerr)
        call hpake(1140, Cerr)

***** calc lumi


***** normalize to lumi and Q-bin-xsec (the lumi cancel)
         do i=1,7 !Q-bins
	  call hopera(1000+i, '+E', 1000+i,1000+i, 1./Nevtauc(i), 0.0)
	  call hopera(1010+i, '+E', 1010+i,1010+i, 1./Nevtau(i) , 0.0)
	  call hopera(1020+i, '+E', 1020+i,1020+i, 1./NevC(i)   , 0.0)
	  call hopera(1030+i, '+E', 1030+i,1030+i, 1./Nevrho(i) , 0.0)
	  call hopera(1040+i, '+E', 1040+i,1040+i, 1./NevC(i)   , 0.0)
         enddo

      
      endif  !end iflag

      return
      end
