      SUBROUTINE HZh0603014(iflag)
c     -----------------------------------------------------------------------
c     
C     Authors: Kaloyan Krastev (DESY, H1)
c     This photoproduction routine describes INgo Strauch thesis results
c     i.e. high pT jats in gammaP at H1, HERA
c     Ref:  hep-ex/0603014 Phys. Lett. B 639 (2006) 21
c     
c     Code conventions:
c     +10000 to run on parton showers instead of final state particles.
c     +100000 is used by NLO intergration options (not available yet).
c     
c     iflag = 1,2 or 3 depending on whether it is the initialization,
c     processing or termination phase (respectively)
c     
c     The default jetfinder is KTCLUS(
c     To change jetfinder use these values to call the initialisation
c     phase.
c     1 + 10 c (jetfinder number)
c
c     All these graphs will be meaningless unless Xsec and Ntot are set
c     before calling the termination routine.
c     (Xsec - total cross section returned by MC)
c     (Ntot - number of events passed to this routine)
c     
c     Cuts :
c     Q2 < 1 GeV^2
c     0.1 < y_bj < 0.9
c     Et > 15, 25 GeV
c     -0.5 < eta < 2.75
c
c
c     Experimental data histograms:     
c     id 11, d\sigma/dM_jj
c     id 12, d\sigma/dET(mean)
c     id 13, d\sigma/dET(max)
c     id 14, d\sigma/detabar, 0.1<y<0.5, 25<ET(max)<35 GeV
c     id 15, d\sigma/detabar, 0.1<y<0.5, 35<ET(max)<80 GeV
c     id 16, d\sigma/detabar, 0.5<y<0.9, 25<ET(max)<35 GeV
c     id 17, d\sigma/detabar, 0.5<y<0.9, 35<ET(max)<80 GeV
c
c     Figure 2 =====================
c     id 18, d\sigma/dxgamma, xp<0.1
c     id 19, d\sigma/dxgamma, xp>0.1
c     ==============================
c
c     Figure 3 =========================
c     id 20, d\sigma/dxp,     xgamma<0.8
c     id 21, d\sigma/dxp,     xgamma>0.8
c     ==================================
c
c     id 22, d\sigma/dxgamma, 25<ET(max)<35 GeV
c     id 23, d\sigma/dxgamma, 35<ET(max)<80 GeV
c
c     Figure 1 ===============================
c     id 24, d\sigma/d\cos(theta*), xgamma<0.8
c     id 25, d\sigma/d\cos(theta*), xgamma>0.8
c     id 26, d\sigma/d\cos(theta*), xgamma<0.8, M_jj>65 GeV
c     id 27, d\sigma/d\cos(theta*), xgamma>0.8, M_jj>65 GeV     
c     =====================================================
c
c     Figure 4 =========== now ====================
c     id 28, d\sigma/d\pT(max), xgamma<0.8, eta_12<1
c     id 29, d\sigma/d\pT(max), xgamma>0.8, eta_12<1
c     id 30, d\sigma/d\pT(max), xgamma<0.8, eta_i<1, eta_j>1
c     id 31, d\sigma/d\pT(max), xgamma>0.8, eta_i<1, eta_j>1
c     id 32, d\sigma/d\pT(max), xgamma<0.8, eta_12>1
c     id 33, d\sigma/d\pT(max), xgamma>0.8, eta_12>1
c     =============================================
c
c     Figure 5 =========== now ===============
c     id 34, d\sigma/dxp, xgamma<0.8, eta_12<1
c     id 35, d\sigma/dxp, xgamma>0.8, eta_12<1
c     id 36, d\sigma/dxp, xgamma<0.8, eta_i<1, eta_j>1
c     id 37, d\sigma/dxp, xgamma>0.8, eta_i<1, eta_j>1
c     id 38, d\sigma/dxp, xgamma<0.8, eta_12>1
c     id 39, d\sigma/dxp, xgamma>0.8, eta_12>1
c     ========================================
c
c
c     Monte Carlo:
c     id + 100
c              
c     NLO:     ..
c     id + 200 ~~
c
c
c     =====================================================
c     Author: Kaloyan Krastev (kaloyan@mail.desy.de)
c     Reference: ??????????
c     Version 1.0, july/2005
c     ====================================================
c     Template: hz01225 by Matthew Wing (wing@mail.desy.de) 
c     Version 1.0, 21/05/02,  Stat. errors only
c     Version 2.0, 12/07/02,  Stat.+syst. (uncorrelated) errors
c     
c     ---------------------------------------------------------------
*     
*     HERA Tuning defined commons

      IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"

      CHARACTER*6 jetf
      
      INTEGER NJET
c      DOUBLE PRECISION Jets(50,8)
      DOUBLE PRECISION CONER
      
      DOUBLE PRECISION elecE
      INTEGER myil,myip

c     Iterators
      INTEGER i_etcut, i_jet, j_jet, i_plot, i_bin

      DOUBLE PRECISION xgamcut
      double precision xpcut
      DOUBLE PRECISION phi(2),px(2),py(2),pz(2),ptsq(2)
      DOUBLE PRECISION ppsq(2),energy(2),mass(2)
      DOUBLE PRECISION rap(2),Et(2)
      DOUBLE PRECISION ybj,q2

C     --- cuts arrays
      DOUBLE PRECISION ybjcut(3),q2cut(2)
      DOUBLE PRECISION rapcut(2),mjjcut,etabarcut(2)
      double precision etcut(4)
c     photoproduction cuts
      DATA ybjcut / 0.1, 0.5, 0.9 /
      DATA q2cut / 0., 1. /
      DATA xgamcut / 0.8 /
      DATA xpcut / 0.1 /
      DATA mjjcut / 65. /
      data rapcut / -0.5, 2.75 /
      data etcut / 15., 25., 35., 80. /

c     Unit to read data file from
      INTEGER ipunit
      PARAMETER(ipunit=8)

c     The data
c     --------------------------------------
c     Notes:
c
c     stat. and syst. errors given as % in paper. Convert to pb later.
c     Total uncertainty given in paper includes stat., syst., and energy scale 
c     uncertainties.
c     Remove energy scale uncertainty from total uncertainty for now and 
c     assume it is a ~10% normalisation uncertainty. See later.  (MW, 12/07/02)
c     
c --  Mjj cross section (TABLE 2)
      integer n_mjjbins
      parameter (n_mjjbins=6)
      real mjjbins(n_mjjbins+1)
      real mjjdat(n_mjjbins)
      real mjjstat(n_mjjbins)
      real mjjtotp(n_mjjbins)
      real mjjescalep(n_mjjbins)
      real mjjtotm(n_mjjbins)
      real mjjescalem(n_mjjbins)
      real mjjerr(n_mjjbins)
c --
      data mjjbins    /45.0,  58,    70.0,  90.0,  110.0, 135.0, 180.0 /
      data mjjdat     / 5.747, 4.793, 2.002, 0.3599, 0.153, 0.020 /
c ++
      data mjjstat    / 2.5, 2.9, 3.7, 6.9, 12.0, 27.1 /
      data mjjtotp   / 13., 11., 13., 12., 19., 32. /
      data mjjtotm   / 14., 11., 11., 16., 18., 30. /
      data mjjescalep / 8., 9., 11., 8., 13., 14. /
      data mjjescalem / 8., 8., 8., 12., 11., 9. /
      real mjj_bw(n_mjjbins), mjj_zero(n_mjjbins)
      data mjj_zero / n_mjjbins*0. /

c --  ET(mean) cross section (TABLE 2)
      integer n_etmeanbins
      parameter (n_etmeanbins=4)
      real etmeanbins(n_etmeanbins+1)
      real etmeandat(n_etmeanbins)
      real etmeanstat(n_etmeanbins)
      real etmeantotp(n_etmeanbins)
      real etmeanescalep(n_etmeanbins)
      real etmeantotm(n_etmeanbins)
      real etmeanescalem(n_etmeanbins)
      real etmeanerr(n_etmeanbins)
      data etmeanbins    / 20.0, 30.0, 45.0, 60.0, 80.0 /
      data etmeandat     / 10.65, 2.41, 0.166, 0.0192 /
      data etmeanstat    / 1.8, 3.1, 11.6, 29.2 /
      data etmeantotp   / 12., 12., 16., 37. /
      data etmeantotm   / 12., 11., 17., 34. / 
      data etmeanescalep /  9., 10., 10., 22. / 
      data etmeanescalem /  8.,  8., 11., 16. /
      real etmean_bw(n_etmeanbins), etmean_zero(n_etmeanbins)
      data etmean_zero / n_etmeanbins*0. /

c --  ET(max) cross section (TABLE 2)
      integer n_etmaxbins
      parameter (n_etmaxbins=4)
      real etmaxbins(n_etmaxbins+1)
      real etmaxdat(n_etmaxbins)
      real etmaxstat(n_etmaxbins)
      real etmaxtotp(n_etmaxbins)
      real etmaxescalep(n_etmaxbins)
      real etmaxtotm(n_etmaxbins)
      real etmaxescalem(n_etmaxbins)
      real etmaxerr(n_etmaxbins)
      data etmaxbins    / 25.0, 35.0, 45.0, 60.0, 80.0 /
      data etmaxdat     / 12.36, 1.82, 0.252, 0.0198 /
      data etmaxstat    / 1.8, 3.9, 8.4, 27.4 /
      data etmaxtotp   / 11., 12., 15., 33. /
      data etmaxtotm   / 12., 12., 15., 32. /
      data etmaxescalep /  9.,  9., 11., 17. /
      data etmaxescalem /  8.,  9., 11., 15./
      real etmax_bw(n_etmaxbins), etmax_zero(n_etmaxbins)
      data etmax_zero / n_etmaxbins*0. /

c --  etabar cross sections (TABLE 3)
c     1: 0.1<y<0.5, 25<ET(max)<35 GeV 
c     2: 0.1<y<0.5, 35<ET(max)<80 GeV 
c     3: 0.5<y<0.9, 25<ET(max)<35 GeV 
c     4: 0.5<y<0.9, 35<ET(max)<80 GeV 
      integer n_etabarbins
      parameter (n_etabarbins=6)
      real etabarbins(n_etabarbins+1)
      real etabardat1(n_etabarbins), etabardat2(n_etabarbins)
      real etabarstat1(n_etabarbins), etabarstat2(n_etabarbins)
      real etabarerr1(n_etabarbins), etabarerr2(n_etabarbins)
      real etabartotp1(n_etabarbins), etabartotp2(n_etabarbins)
      real etabartotm1(n_etabarbins), etabartotm2(n_etabarbins)
      real etabarescalep1(n_etabarbins), etabarescalep2(n_etabarbins)
      real etabarescalem1(n_etabarbins), etabarescalem2(n_etabarbins)
      real etabardat3(n_etabarbins), etabardat4(n_etabarbins)
      real etabarstat3(n_etabarbins), etabarstat4(n_etabarbins)
      real etabarerr3(n_etabarbins), etabarerr4(n_etabarbins)
      real etabartotp3(n_etabarbins), etabartotp4(n_etabarbins)
      real etabartotm3(n_etabarbins), etabartotm4(n_etabarbins)
      real etabarescalep3(n_etabarbins), etabarescalep4(n_etabarbins)
      real etabarescalem3(n_etabarbins), etabarescalem4(n_etabarbins)
      data etabarbins     / .0,    .6,    .9,   1.3,   1.7,   2.1, 2.5 /
      data etabardat1     / .0,  12.162,46.925,65.668,50.080,22.267 /
      data etabarstat1    / .0,  10.0,   4.4,   4.2,   5.4,  10.1 /
      data etabartotp1    / .0,  20.0,  11.0,  10.0,  12.0,  17.0 /
      data etabartotm1    / .0,  22.0,  15.0,  11.0,  11.0,  18.0 /
      data etabarescalep1 / .0,   7.0,   7.0,   7.0,  10.0,  10.0 /
      data etabarescalem1 / .0,  11.0,  12.0,   8.0,   7.0,  10.0 /
      data etabardat2     / .0,    .0,   3.381,11.207,10.543, 3.566/
      data etabarstat2    / .0,    .0,  14.0,   8.0,   9.9,  21.0 /
      data etabartotp2    / .0,    .0,  20.0,  14.0,  14.0,  24.0 /
      data etabartotm2    / .0,    .0,  20.0,  13.0,  16.0,  24.0 /
      data etabarescalep2 / .0,    .0,   8.0,  10.0,   9.0,   9.0 /
      data etabarescalem2 / .0,    .0,   9.0,   8.0,  11.0,  10.0 /
      data etabardat3   / 32.246,70.321,56.114,32.641,16.002, 5.629 /
      data etabarstat3   / 4.7,   4.5,   4.9,   7.4,  11.7,  24.8 /
      data etabartotp3  / 17.0,  13.0,  14.0,  13.0,  14.0,  30.0 / 
      data etabartotm3  / 14.0,  13.0,  11.0,  16.0,  14.0,  29.0 /
      data etabarescalep3/11.0,   9.0,  11.0,  10.0,   6.0,   9.0 /
      data etabarescalem3 /6.0,   8.0,   6.0,  13.0,   4.0,   3.0 /
      data etabardat4    / 1.247,12.172,17.720,11.017, 4.043, 0.0/
      data etabarstat4   / 0.0,   9.7,   7.5,   9.3,  18.1,   0.0 /
      data etabartotp4   / 0.0,  20.0,  13.0,  15.0,  24.0,   0.0 /
      data etabartotm4   / 0.0,  20.0,  13.0,  15.0,  24.0,   0.0 /
      data etabarescalep4 /0.0,  10.0,  10.0,  10.0,  13.0,   0.0 /
      data etabarescalem4 /0.0,  11.0,   8.0,   9.0,  13.0,   0.0 /
      real etabar_bw(n_etabarbins), etabar_zero(n_etabarbins)
      data etabar_zero / n_etabarbins*0. /

c --  xgamma cross sections (Table 3 and TABLE 6)
c     1: xproton<0.1
c     2: xproton>0.1
c     3: 25<ET(max)<35 GeV 
c     4: 35<ET(max)<80 GeV 
      integer n_xgammabins
      parameter (n_xgammabins=5)   
      real xgammabins(n_xgammabins+1)
      real xgammadat1(n_xgammabins), xgammadat2(n_xgammabins)
      real xgammaErU1(n_xgammabins), xgammaErU2(n_xgammabins)
      real xgammaErD1(n_xgammabins), xgammaErD2(n_xgammabins)
      real xgammaerr1(n_xgammabins), xgammaerr2(n_xgammabins)
      real xgammadat3(n_xgammabins), xgammadat4(n_xgammabins)
      real xgammaErU3(n_xgammabins), xgammaErU4(n_xgammabins)
      real xgammaErD3(n_xgammabins), xgammaErD4(n_xgammabins)
      real xgammastat3(n_xgammabins), xgammastat4(n_xgammabins) 
      real xgammaerr3(n_xgammabins), xgammaerr4(n_xgammabins) 
      real xgammatotp3(n_xgammabins), xgammatotp4(n_xgammabins)
      real xgammaescalep3(n_xgammabins), xgammaescalep4(n_xgammabins)
      real xgammatotm3(n_xgammabins), xgammatotm4(n_xgammabins)
      real xgammaescalem3(n_xgammabins), xgammaescalem4(n_xgammabins)
      data xgammabins     / 0.1, 0.3, 0.5, 0.7, 0.85, 1.0 /
c     
      data xgammadat1     / 0.0, 19.11, 46.43, 94.58, 220.74/     
      data xgammaErU1     / 0.0, 11.9 , 12.0 , 10.3 ,   8.4 /     
      data xgammaErD1     / 0.0, 10.4 , 10.3 ,  9.1 ,   8.5 /     
c 
      data xgammadat2     / 55.51, 90.88, 103.82, 148.74, 276.52 /  
      data xgammaErU2     /14.1, 12.4 , 10.9 ,  9.9 ,   8.6 /     
      data xgammaErD2     / 8.0,  6.4 ,  6.4 ,  7.2 ,   6.0 /     

      data xgammadat3     / 40.1, 77.3, 101.1, 173.6, 361.7 / 
      data xgammastat3    / 8.4, 5.3, 4.3, 3.9, 2.5 /
      data xgammatotp3   / 17., 11., 14., 14., 11. /
      data xgammatotm3   / 18., 13., 14., 11., 12. /
      data xgammaescalep3 /  7.,  8., 11., 12.,  7. /
      data xgammaescalem3 /  8.,  9., 11.,  8.,  7. /

      data xgammadat4     / 2.34, 9.83, 19.7, 28.7, 78.1 / 
      data xgammastat4    / 27.5, 12.1, 8.5, 8.0, 4.9 /
      data xgammatotp4   / 34., 17., 14., 15., 13. /
      data xgammatotm4   / 31., 18., 15., 16., 12. /
      data xgammaescalep4 / 16.,  9., 10., 11., 10. /
      data xgammaescalem4 /  9., 10., 11., 10.,  8. /
      real xgamma_bw(n_xgammabins), xgamma_zero(n_xgammabins)
      data xgamma_zero / n_xgammabins*0. /

c --  xproton cross sections (TABLE 5)
c     1: xgamma<0.8
c     2: xgamma>0.8
      integer n_xpbins
      parameter (n_xpbins=6)
      real xpbins(n_xpbins+1)
      real xpdat1(n_xpbins),  xpdat2(n_xpbins)
      real xpstat1(n_xpbins), xpstat2(n_xpbins)
      real xptotp1(n_xpbins), xptotp2(n_xpbins)
      real xpescalep1(n_xpbins), xpescalep2(n_xpbins)
      real xptotm1(n_xpbins), xptotm2(n_xpbins)
      real xpescalem1(n_xpbins), xpescalem2(n_xpbins)
      real xperr1(n_xpbins), xperr2(n_xpbins)
      data xpbins       / 0.05,  0.10,  0.15,  0.22, 0.32, 0.45, 0.7 /
      data xpdat1     / 402.39,460.98,365.17,180.53,33.86, 2.05 /
      data xpstat1     /  5.3,   4.8,   4.4,   5.6, 11.8, 10.0  /
      data xptotp1     / 15.0,  13.0,   9.0,  19.0, 17.0, 17.0 /
      data xptotm1     / 17.0,  11.0,  15.0,  12.0, 19.0, 19.0 /
      data xpescalep1  /  8.0,   9.0,   5.0,  17.0, 11.0, 11.0 /
      data xpescalem1  / 10.0,   5.0,  12.0,   9.0, 13.0, 13.0 /
      data xpdat2     / 646.11,441.90,262.53,113.32,20.03,1.89 /
      data xpstat2      / 3.5,   4.3,   4.7,   6.5, 12.7, 12.0 /
      data xptotp2     / 12.0,  12.0,  11.0,  13.0, 19.0, 19.0 /
      data xptotm2     / 14.0,  14.0,  12.0,  13.0, 18.0, 18.0 /
      data xpescalep2  /  7.0,   8.0,   8.0,  10.0, 12.0, 12.0 /
      data xpescalem2  /  9.0,   9.0,   8.0,   9.0, 11.0, 11.0 /
      real xp_bw(n_xpbins), xp_zero(n_xpbins)
      data xp_zero / n_xpbins*0. /



c --  now xproton cross sections (Table 4)
c     1: xgamma<0.8, eta_12<1
c     2: xgamma>0.8, eta_12<1
c     3: xgamma<0.8, eta_i<1, eta_j>1
c     4: xgamma>0.8, eta_i<1, eta_j>1
c     5: xgamma<0.8, eta_12>1
c     6: xgamma>0.8, eta_12>1
      integer n_xxpbins
      parameter (n_xxpbins=6)
      real xxpbins(n_xxpbins+1)
      real xxpdat1(n_xxpbins),  xxpdat2(n_xxpbins)
      real xxpErU1(n_xxpbins),  xxpErU2(n_xxpbins)
      real xxpErD1(n_xxpbins),  xxpErD2(n_xxpbins)
      real xxperr1(n_xxpbins), xxperr2(n_xxpbins)

      real xxpdat3(n_xxpbins),  xxpdat4(n_xxpbins)
      real xxpErU3(n_xxpbins),  xxpErU4(n_xxpbins)
      real xxpErD3(n_xxpbins),  xxpErD4(n_xxpbins)
      real xxperr3(n_xxpbins), xxperr4(n_xxpbins)

      real xxpdat5(n_xxpbins),  xxpdat6(n_xxpbins)
      real xxpErU5(n_xxpbins),  xxpErU6(n_xxpbins)
      real xxpErD5(n_xxpbins),  xxpErD6(n_xxpbins)
      real xxperr5(n_xxpbins), xxperr6(n_xxpbins)

      data xxpbins       / 0.05,  0.10,  0.15,  0.22, 0.32, 0.45, 0.7 /

      data xxpdat1     /  60.60,  1.18,   .0 ,   .0 ,  .0,   .0  /
      data xxpErU1     /  11.9 , 57.9 ,   .0 ,   .0 ,  .0,   .0  /
      data xxpErD1     /  11.8 , 57.2 ,   .0 ,   .0 ,  .0,   .0  /

      data xxpdat3     / 276.22,235.36,131.95, 31.80, 2.38, .02  /
      data xxpErU3     /  10.9 , 13.1 , 13.3 , 16.4 ,28.1,115.3  /
      data xxpErD3     /   9.0 ,  8.0 ,  8.1 , 12.0 ,24.3,119.5  /

      data xxpdat6     /  41.84,175.08,205.50,122.74,24.69, 1.35 /
      data xxpErU6     /  11.4 , 11.2 , 10.5 , 13.7 ,14.0, 25.5  /
      data xxpErD6     /  11.8 ,  8.5 ,  7.3 ,  6.6 ,10.1, 21.1  /

      data xxpdat2     / 180.53,  5.17,   .0 ,   .0 ,  .0 ,  .0  /
      data xxpErU2     /   8.1 , 27.4 ,   .0 ,   .0 ,  .0,   .0  /
      data xxpErD2     /   7.8 , 28.9 ,   .0 ,   .0 ,  .0,   .0  /

      data xxpdat4     / 401.94,269.6, 120.07, 32.43, 4.27,  .07 /
      data xxpErU4     /   8.7 ,  9.0 ,  9.9 , 12.3 ,21.3,107.9  /
      data xxpErD4     /   8.4 ,  6.6 ,  7.3 ,  8.5 ,19.8,104.3  /

      data xxpdat5     /  28.58,118.59,126.72, 76.27,13.0,  1.2  /
      data xxpErU5     /  11.2 ,  7.4 , 10.7 , 11.9 ,14.5, 27.4  /
      data xxpErD5     /  15.5 , 11.8 ,  9.8 , 10.0 ,12.9, 26.2  /

      real xxp_bw(n_xxpbins), xxp_zero(n_xxpbins)
      data xxp_zero / n_xxpbins*0. /


c --  now pT,max cross sections (Table 4) to be continued...
c     1: xgamma<0.8, eta_12<1
c     2: xgamma>0.8, eta_12<1
c     3: xgamma<0.8, eta_i<1, eta_j>1
c     4: xgamma>0.8, eta_i<1, eta_j>1
c     5: xgamma<0.8, eta_12>1
c     6: xgamma>0.8, eta_12>1
      integer n_pTbins
      parameter (n_pTbins=4)
      real pTbins(n_pTbins+1)
      real pTdat1(n_pTbins),  pTdat2(n_pTbins)
      real pTErU1(n_pTbins),  pTErU2(n_pTbins)
      real pTErD1(n_pTbins),  pTErD2(n_pTbins)
      real pTerr1(n_pTbins), pTerr2(n_pTbins)
      data pTbins       /25.0,  29.0,  35.0,  50.0, 80.0 /

      data pTdat1        / .796,  .217,  .022,  .0 /
      data pTErU1      / 16.5  ,12.5  ,22.7  ,  .0 /
      data pTErD1      / 14.3  ,13.7  ,21.1  ,  .0 /

      data pTdat2       / 2.664,  .872,  .131,  .002 /
      data pTErU2      /  8.8  , 9.6  ,12.0  ,54.2 /
      data pTErD2      /  8.8  , 9.3  ,11.2  ,49.2 /

      real pT_bw(n_pTbins), pT_zero(n_pTbins)
      data pT_zero / n_pTbins*0. /

      integer n_pT2bins
      parameter (n_pT2bins=6)
      real pT2bins(n_pT2bins+1)

      real pT2dat3(n_pT2bins),  pT2dat4(n_pT2bins)
      real pT2ErU3(n_pT2bins),  pT2ErU4(n_pT2bins)
      real pT2ErD3(n_pT2bins),  pT2ErD4(n_pT2bins)
      real pT2err3(n_pT2bins), pT2err4(n_pT2bins)

      real pT2dat5(n_pT2bins),  pT2dat6(n_pT2bins)
      real pT2ErU5(n_pT2bins),  pT2ErU6(n_pT2bins)
      real pT2ErD5(n_pT2bins),  pT2ErD6(n_pT2bins)
      real pT2err5(n_pT2bins), pT2err6(n_pT2bins)

      data pT2bins      / 25.0,  29.0,  35.0,  42.0,  50.0,  57.0,80.0 /

      data pT2dat3       / 5.5,   1.989,  .517,  .110,  .023,  .001 / 
      data pT2ErU3      / 11.0  ,13.1  ,15.4  ,20.3,  33.8,  90.6 /
      data pT2ErD3      /  8.0  , 8.3  ,10.7  ,14.9,  33.9,  88.2 /

      data pT2dat4       / 5.751, 2.4,    .854,  .249,  .089,  .015 / 
      data pT2ErU4      /  9.0  , 8.6  ,10.2  ,13.4,  19.1,  24.2 /
      data pT2ErD4      /  7.6  , 7.4  , 7.6  ,10.4,  16.6,  23.7 /

      data pT2dat5       / 5.691, 2.031,  .631,  .181,  .034,  .008 / 
      data pT2ErU5      / 11.6  ,11.1  ,12.4  ,15.6,  29.5,  32.7 /
      data pT2ErD5      /  6.6  , 8.0  , 8.3  ,12.5,  26.9,  29.3 /

      data pT2dat6       / 2.833, 1.474,  .487,  .167,  .081,  .019 / 
      data pT2ErU6      /  7.9  ,11.0  ,11.7  ,14.0,  18.3,  19.8 /
      data pT2ErD6      / 10.7  ,10.4  ,12.4  ,13.7,  16.7,  19.7 /

      real pT2_bw(n_pT2bins), pT2_zero(n_pT2bins)
      data pT2_zero / n_pT2bins*0. /


c --  cos(theta*) cross sections (Table 2)
c     1: xgamma<0.8
c     2: xgamma>0.8
c     3: xgamma<0.8, mjj>65GeV
c     4: xgamma>0.8, mjj>65GeV
      integer n_ctbins
      parameter (n_ctbins=8)
      real ctbins(n_ctbins+1)
      real ctdat1(n_ctbins), ctdat2(n_ctbins)
      real ctErU1(n_ctbins), ctErU2(n_ctbins)
      real ctErD1(n_ctbins), ctErD2(n_ctbins)
      real cterr1(n_ctbins), cterr2(n_ctbins)
      real ctdat3(n_ctbins), ctdat4(n_ctbins)
      real ctErU3(n_ctbins), ctErU4(n_ctbins)
      real ctErD3(n_ctbins), ctErD4(n_ctbins)
      real cterr3(n_ctbins), cterr4(n_ctbins)
      data ctbins  / 0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.85 /
      data ctdat1  / 104.92, 109.98, 105.18, 111.92, 110.97
     +, 107.18, 91.65, 63.34/
      data ctErU1  / 10.7, 9.6, 11.1, 10.5, 10.8, 15.2, 14.0, 14.9 /
      data ctErD1  /  8.1, 9.2,  7.7,  7.5,  6.8,  7.8,  7.0, 12.2 /
      data ctdat2  / 121.64, 120.34, 124.26, 117.96, 114.16
     +, 105.41, 95.70, 60.91 /
      data ctErU2  /  7.5, 7.4,  8.0,  7.4,  8.5, 12.4,  9.1, 13.9 /
      data ctErD2  /  9.0,10.0,  5.7,  6.7,  6.0,  8.3,  6.8, 11.5 /
      data ctdat3  / 15.19, 13.51, 14.38, 18.15, 23.61
     +, 35.90, 56.38, 56.58 /
      data ctErU3  / 14.7,15.7, 16.3, 13.8, 13.3, 16.5, 15.1, 14.6 /
      data ctErD3  / 12.3,12.8, 11.2, 13.3, 10.3,  9.6,  6.8, 11.3 /
      data ctdat4  / 30.99, 29.54, 33.16, 35.60
     +, 40.79, 53.04, 74.15, 58.86 /
      data ctErU4  / 10.4,12.2, 10.3, 10.5, 10.4, 12.4,  9.4, 12.8 /
      data ctErD4  / 11.0,12.1,  8.8,  8.7,  8.2,  7.6,  7.0, 10.1 /
      real ct_bw(n_ctbins), ct_zero(n_ctbins)
      data ct_zero / n_ctbins*0. /

      real xp, xgam
      REAL costheta, deltaeta, mjj, etabar, etmean, etmax

      INTEGER intiflag, chjet, iproc, iflag
      SAVE chjet
C     --- choose same jet finder as in the paper --- KTCLUS
      DATA chjet/3/
      INTEGER hzflag
     
      LOGICAL lp, first
      INTEGER LENOCC      
      DATA lp, first /.false., .true./
      Character*8 xxxx
      Data xxxx /'H0603014'/      
      INTEGER survived 
      IF (FIRST) THEN

         survived = 0
         DO i_bin=1,n_mjjbins-1
            mjj_bw(i_bin)=1.0/(mjjbins(i_bin+1)-mjjbins(i_bin))
         ENDDO

         DO i_bin=1,n_etmaxbins
            etmax_bw(i_bin)=1.0/(etmaxbins(i_bin+1)-etmaxbins(i_bin))
         ENDDO

         DO i_bin=1,n_etmeanbins
            etmean_bw(i_bin)=1.0/(etmeanbins(i_bin+1)-etmeanbins(i_bin))
         ENDDO

         DO i_bin=1,n_etabarbins
            etabar_bw(i_bin)=1.0/(etabarbins(i_bin+1)-etabarbins(i_bin))
         ENDDO

         DO i_bin=1,n_xgammabins
            xgamma_bw(i_bin)=1.0/(xgammabins(i_bin+1)-xgammabins(i_bin))
         ENDDO

         DO i_bin=1,n_xpbins
            xp_bw(i_bin)=1.0/(xpbins(i_bin+1)-xpbins(i_bin))
         ENDDO

         DO i_bin=1,n_ctbins
            ct_bw(i_bin)=1.0/(ctbins(i_bin+1)-ctbins(i_bin))
         ENDDO

         DO i_bin=1,n_xxpbins
            xxp_bw(i_bin)=1.0/(xxpbins(i_bin+1)-xxpbins(i_bin))
         ENDDO

         DO i_bin=1,n_pTbins
            pT_bw(i_bin)=1.0/(pTbins(i_bin+1)-pTbins(i_bin))
         ENDDO

         DO i_bin=1,n_pT2bins
            pT2_bw(i_bin)=1.0/(pT2bins(i_bin+1)-pT2bins(i_bin))
         ENDDO

         FIRST=.FALSE.
         CLOSE(IPUNIT)

      ENDIF

      intiflag=iflag

      
C     Choose direct/resolved
C     --- recommend 1 - DIRECT, 2 - RESOLVED
      IF (intiflag.gt.1000) THEN
         iproc=INT(intiflag/1000)
         intiflag=MOD(intiflag,1000)
      ENDIF
      
      chjet=ABS(chjet)


      IF (intiflag.eq.1) THEN
C     *** Initialisation run ***
C     *** Create directories ***

         CALL HCDIR('//PAWC',' ')
         call hmdir(xxxx,'S')


         DO iproc=0,2
c     Book the M_jj plot.
            CALL HBOOKB(iproc*100+11,'Mjj'
     &           ,n_mjjbins,mjjbins,0.)
c     Book the ET(mean) plot.
            CALL HBOOKB(iproc*100+12,'ET(mean)'
     &           ,n_etmeanbins,etmeanbins,0.)
c     Book the ET(max) plot.
            CALL HBOOKB(iproc*100+13,'ET(max)'
     &           ,n_etmaxbins,etmaxbins,0.)
c     Book the etabar plot.
            CALL HBOOKB(iproc*100+14
     &           ,'etabar,0.1<y<0.5, 25<ET(max)<35GeV'
     &           ,n_etabarbins, etabarbins,0.)
            CALL HBOOKB(iproc*100+15
     &           ,'etabar,0.1<y<0.5 ,35<ET(max)<80GeV'
     &           ,n_etabarbins, etabarbins, 0.)
            CALL HBOOKB(iproc*100+16
     &           ,'etabar,0.5<y<0.9, 25<ET(max)<35GeV'
     &           ,n_etabarbins, etabarbins, 0.)
            CALL HBOOKB(iproc*100+17
     &           ,'etabar,0.5<y<0.9, 35<ET(max)<80GeV'
     &           ,n_etabarbins, etabarbins, 0.)
c     Book the xgamma and xp plot.
            CALL HBOOKB(iproc*100+18,'xg, xp<0.1'
     &           ,n_xgammabins,xgammabins,0.)
            CALL HBOOKB(iproc*100+19,'xg, xp>0.1'
     &           ,n_xgammabins,xgammabins,0.)
            CALL HBOOKB(iproc*100+20,'xp, xg<0.8'
     &           ,n_xpbins,xpbins,0.)
            CALL HBOOKB(iproc*100+21,'xp, xg>0.8'
     &           ,n_xpbins,xpbins,0.)
            CALL HBOOKB(iproc*100+22,'xg, 25<ET(max)<35 GeV'
     &           ,n_xgammabins,xgammabins,0.)
            CALL HBOOKB(iproc*100+23,'xg, 35<ET(max)<80 GeV'
     &           ,n_xgammabins,xgammabins,0.)
c     Book the costheta* plots.
            CALL HBOOKB(iproc*100+24,'cos(theta*), xg<0.8'
     &           ,n_ctbins,ctbins,0.)
            CALL HBOOKB(iproc*100+25,'cos(theta*), xg>0.8'
     &           ,n_ctbins,ctbins,0.)
            CALL HBOOKB(iproc*100+26,'cos(theta*), xg<0.8, M_jj>65 GeV'
     +           ,n_ctbins, ctbins, 0.)
            CALL HBOOKB(iproc*100+27,'cos(theta*), xg>0.8, M_jj>65 GeV'
     +           ,n_ctbins, ctbins, 0.)
c
c
c     here starts the change
            CALL HBOOKB(iproc*100+28, 'pT, xgamma<0.8, eta_12<1'
     &           ,n_pTbins,pTbins,0.)
            CALL HBOOKB(iproc*100+29, 'pT, xgamma>0.8, eta_12<1'
     &           ,n_pTbins,pTbins,0.)
            CALL HBOOKB(iproc*100+30, 'pT, xgamma<0.8, eta_i<1, eta_j>1'
     &           ,n_pT2bins,pT2bins,0.)
            CALL HBOOKB(iproc*100+31, 'pT, xgamma>0.8, eta_i<1, eta_j>1'
     &           ,n_pT2bins,pT2bins,0.)
            CALL HBOOKB(iproc*100+32, 'pT, xgamma<0.8, eta_12>1'
     &           ,n_pT2bins,pT2bins,0.)
            CALL HBOOKB(iproc*100+33, 'pT, xgamma>0.8, eta_12>1'
     &           ,n_pT2bins,pT2bins,0.)
c
c
c     Book the now xp plot.
            CALL HBOOKB(iproc*100+34, 'xp, xgamma<0.8, eta_12<1'
     &           ,n_xxpbins,xxpbins,0.)
            CALL HBOOKB(iproc*100+35, 'xp, xgamma>0.8, eta_12<1'
     &           ,n_xxpbins,xxpbins,0.)
            CALL HBOOKB(iproc*100+36, 'xp, xgamma<0.8, eta_i<1, eta_j>1'
     &           ,n_xxpbins,xxpbins,0.)
            CALL HBOOKB(iproc*100+37, 'xp, xgamma>0.8, eta_i<1, eta_j>1'
     &           ,n_xxpbins,xxpbins,0.)
            CALL HBOOKB(iproc*100+38, 'xp, xgamma<0.8, eta_12>1'
     &           ,n_xxpbins,xxpbins,0.)
            CALL HBOOKB(iproc*100+39, 'xp, xgamma>0.8, eta_12>1'
     &           ,n_xxpbins,xxpbins,0.)
         ENDDO

         CALL HIDOPT(0,'stat')
         CALL HBARX(0)
         
C     --- standard cone radius is 1.0
         CALL HZJETRAD(2,CONER)
         IF (CONER.lt.0.0) THEN
            CONER=1.0
         ENDIF
         CALL HZJTNAME(chjet,jetf)
         
         WRITE(6,*)'**********************************************'
         WRITE(6,*)'* HZ05555 called, histograms will be output. *'
         WRITE(6,*)'*     Jet finder selected : ',jetf,'           *'
         WRITE(6,1001)'*        with cone radius : ',CONER,'   *'
         WRITE(6,*)'**********************************************'

 1001    FORMAT(A29,F5.2,A13)

      ELSE IF (intiflag.eq.2) THEN
C     *** Filling runs ***
C     *** Change directory to our graphs ***
         iproc=1

c         CALL HCDIR('//PAWC/d05555',' ')
         call hcdir('//PAWC/'//xxxx,' ')

C         q2=hzphokin(1)
         q2=real(HZDISKIN(1))
         IF (q2.eq.-1) THEN
            PRINT*,'HZ05555: HZPHOKIN returns error for Q2'
         ELSEIF (q2.lt.0) THEN
            q2=abs(q2)
         ENDIF
C         ybj=hzphokin(3)
         ybj =real(HZDISKIN(3))
         IF (ybj.eq.-1) THEN
            PRINT*,'HZ05555: HZPHOKIN returns error for ybj'
         ENDIF



         
C     *** Fill histograms ***
        
C     All events must pass the y cut and the Q2 cut.
         IF ((ybj.ge.ybjcut(1)).and.(ybj.le.ybjcut(3))
     &        .and.(q2.lt.q2cut(2))) Then
c     Get the initla state electron energy.
            hzflag=HZIBEAM(myip,myil)
            IF (hzflag.ge.0) Then
               elecE=abs(PHEP(4,myil))
            ELSE
               write(6,*) 'HZ05555: Electron not found. E=0.001'
               write(6,*) '         event will be rejected'
               RETURN
            ENDIF
            
c     Find jets
            CALL HZJTFIND(chjet,CONER,NJET,Jets)
            
            IF (lp) THEN
               WRITE(*,*) 'Njet=',NJET,'Q2=',q2,'ybj=',ybj 
            ENDIF

            DO i_jet=1,2
               et(i_jet)=0.0
            ENDDO

c     If there aren't at least 2 jets, don't bother doing anything else
            IF (njet.GE.2) THEN
               i_jet=0
               DO j_jet = 1,njet
c     Find the two highest ET jets in the rapidity interval.
c     Assumes jets are ordered in ET.
                  IF ( jets(j_jet,1).gt.rapcut(1).and.jets(j_jet,1).lt
     &                 .rapcut(2)) THEN
                     i_jet=i_jet+1
                     IF (i_jet.LE.2) THEN
                        Et(i_jet)=Jets(j_jet,3)
                        rap(i_jet)=Jets(j_jet,1)
                        phi(i_jet)=Jets(j_jet,2)
                     ENDIF

                  ENDIF
               ENDDO

c     All events must satisfy at least the lowest ET cut.
               IF (et(2).gt.etcut(1).AND.et(1).gt.etcut(2)) THEN
                  survived = survived + 1
                  PRINT*,'survived = ', survived
            
c     Calculate fractions and quantities
                  xgam = (Et(1)*exp(-1.*rap(1))+Et(2)*exp(-1.*rap(2)))
     &                 / (2.0*ybj*elecE)
                  xp   = (Et(1)*exp(rap(1))+Et(2)*exp(rap(2)))
     &                 / (1640.)
                  deltaeta = abs(rap(1)-rap(2))
                  etabar   = (rap(1)+rap(2))/2.0
                  costheta = abs(tanh(deltaeta/2))
                  etmax    = real(et(1))
                  etmean   = (et(1)+et(2))/2.0
c     TODO: is phi wrapping dealt with ok here??
                  mjj = sqrt(2.0*et(1)*et(2)*(cosh(deltaeta)
     +              -cos(phi(1)-phi(2))))

c     Fill mjj, etmean, etmax
                  call hfill(iproc*100+11,mjj,0.,wtx)
                  call hfill(iproc*100+12,etmean,0.,wtx)
                  call hfill(iproc*100+13,etmax,0.,wtx)
c     Fill etabar
                  if (ybj.lt.ybjcut(2).and.et(1).lt.etcut(3)) then
                     call hfill(iproc*100+14,etabar,0.,wtx)
                  end if
                  if (ybj.lt.ybjcut(2).and.et(1).gt.etcut(3)
     .                                .and.et(1).lt.etcut(4)) then
                     call hfill(iproc*100+15,etabar,0.,wtx)
                  end if
                  if (ybj.gt.ybjcut(2).and.et(1).lt.etcut(3)) then
                     call hfill(iproc*100+16,etabar,0.,wtx)
                  end if
                  if (ybj.gt.ybjcut(2).and.et(1).gt.etcut(3)
     .                                .and.et(1).lt.etcut(4)) then
                     call hfill(iproc*100+17,etabar,0.,wtx)
                  end if
c     Fill xgamma with xp cuts and rapidity staff
                  if (xp.lt.xpcut) then
                     call hfill(iproc*100+18,real(xgam),0.,wtx)
                  else
                     call hfill(iproc*100+19,real(xgam),0.,wtx)
                  end if
c     Fill xp with xgamma cuts
                  if (xgam.lt.xgamcut) then
                     call hfill(iproc*100+20,real(xp),0.,wtx)
                     if (rap(1).lt.1.and.rap(2).lt.1) then
                        call hfill(iproc*100+34,real(xp),0.,wtx)
                        call hfill(iproc*100+28,real(et(1)),0.,wtx)
                     else if (rap(1).gt.1.and.rap(2).gt.1) then
                        call hfill(iproc*100+38,real(xp),0.,wtx)
                        call hfill(iproc*100+32,real(et(1)),0.,wtx)
                     else 
                        call hfill(iproc*100+36,real(xp),0.,wtx)
                        call hfill(iproc*100+30,real(et(1)),0.,wtx)
                     end if
                  else
                     call hfill(iproc*100+21,real(xp),0.,wtx)
                     if (rap(1).lt.1.and.rap(2).lt.1) then
                        call hfill(iproc*100+35,real(xp),0.,wtx)
                        call hfill(iproc*100+29,real(et(1)),0.,wtx)
                     else if (rap(1).gt.1.and.rap(2).gt.1) then
                        call hfill(iproc*100+39,real(xp),0.,wtx)
                        call hfill(iproc*100+33,real(et(1)),0.,wtx)
                     else 
                        call hfill(iproc*100+37,real(xp),0.,wtx)
                        call hfill(iproc*100+31,real(et(1)),0.,wtx)
                     end if
                  end if
c     Fill xgamma with ET cuts
                  if (et(1).lt.etcut(3)) then
                     call hfill(iproc*100+22,real(xgam),0.,wtx)
                  end if
                  if (et(1).gt.etcut(3).and.et(1).lt.etcut(4)) then
                     call hfill(iproc*100+23,real(xgam),0.,wtx)
                  end if
c      Fill cos(theta*)
                  if (xgam.lt.xgamcut) then
                     call hfill(iproc*100+24,real(costheta),0.,wtx)
                     if (mjj.gt.mjjcut) then
                        call hfill(iproc*100+26,real(costheta),0.,wtx)
                     end if
                  else
                     call hfill(iproc*100+25,real(costheta),0.,wtx)
                     if (mjj.gt.mjjcut) then
                        call hfill(iproc*100+27,real(costheta),0.,wtx)
                     end if
                  end if
                  
                  IF (lp) THEN
                     WRITE(*,*) 'xgamma,wtx=',xgam,wtx
                  ENDIF
               end if
            end if
         end if
      ELSE IF (intiflag.eq.3) THEN
C     *** Termination run ***
C     *** Change directory ***
         call hcdir('//PAWC/'//xxxx,' ')
         
C     *** Finish off histograms ***
         Xsec = 8209.0
         Xsec = 1.077E-05     
c     Xsec in mb
         Xsec =  Xsec * 1.0E9
c     Now pb

c     this appears as  1.070E-05 in pythia steering
         Ntot = 999999
c         Ntot = 199999
c        Ntot = 99999
         Ntot = 9999
c        Ntot = 999


         IF (Xsec.eq.0) THEN
            PRINT*,'HZ05555: termination called with zero cross section'
            PRINT*,'          cross section graph meaningless'
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            PRINT*,'HZ05555: termination called with no total events'
            PRINT*,'          cross section graph meaningless'
            Ntot=1
         ENDIF
         


c     Create the bin width histograms
         DO i_plot=11,11
            CALL HBOOKB(50+i_plot,'bin width',n_mjjbins
     &           ,mjjbins,0.)
            CALL HPAK(50+i_plot,mjj_bw)
            CALL HPAKE(50+i_plot,mjj_zero)
         ENDDO
         DO i_plot=12,12
            CALL HBOOKB(50+i_plot,'bin width',n_etmeanbins
     &           ,etmeanbins,0.)
            CALL HPAK(50+i_plot,etmean_bw)
            CALL HPAKE(50+i_plot,etmean_zero)
         ENDDO
         DO i_plot=13,13
            CALL HBOOKB(50+i_plot,'bin width',n_etmaxbins
     &           ,etmaxbins,0.)
            CALL HPAK(50+i_plot,etmax_bw)
            CALL HPAKE(50+i_plot,etmax_zero)
         ENDDO
         DO i_plot=14,17
            CALL HBOOKB(50+i_plot,'bin width',n_etabarbins
     &           ,etabarbins,0.)
            CALL HPAK(50+i_plot,etabar_bw)
            CALL HPAKE(50+i_plot,etabar_zero)
         ENDDO
         DO i_plot=18,19
            CALL HBOOKB(50+i_plot,'bin width',n_xgammabins
     &           ,xgammabins,0.)
            CALL HPAK(50+i_plot,xgamma_bw)
            CALL HPAKE(50+i_plot,xgamma_zero)
         ENDDO
         DO i_plot=20,21
            CALL HBOOKB(50+i_plot,'bin width',n_xpbins
     &           ,xpbins,0.)
            CALL HPAK(50+i_plot,xp_bw)
            CALL HPAKE(50+i_plot,xp_zero)
         ENDDO
         DO i_plot=22,23
            CALL HBOOKB(50+i_plot,'bin width',n_xgammabins
     &           ,xgammabins,0.)
            CALL HPAK(50+i_plot,xgamma_bw)
            CALL HPAKE(50+i_plot,xgamma_zero)
         ENDDO
         DO i_plot=24,27
            CALL HBOOKB(50+i_plot,'bin width',n_ctbins
     &           ,ctbins,0.)
            CALL HPAK(50+i_plot,ct_bw)
            CALL HPAKE(50+i_plot,ct_zero)
         ENDDO
c     change
         DO i_plot=28,29
            CALL HBOOKB(50+i_plot,'bin width', n_pTbins
     &           ,pTbins,0.)
            CALL HPAK(50+i_plot,pT_bw)
            CALL HPAKE(50+i_plot,pT_zero)
         ENDDO
         DO i_plot=30,33
            CALL HBOOKB(50+i_plot,'bin width', n_pT2bins
     &           ,pT2bins,0.)
            CALL HPAK(50+i_plot,pT2_bw)
            CALL HPAKE(50+i_plot,pT2_zero)
         ENDDO
         DO i_plot=34,39
            CALL HBOOKB(50+i_plot,'bin width', n_xxpbins
     &           ,xxpbins,0.)
            CALL HPAK(50+i_plot,xxp_bw)
            CALL HPAKE(50+i_plot,xxp_zero)
         ENDDO


c
c --  Convert errors from percentage into pb.
c --  Remove energy scale uncertainty from total systematic error for now and 
c --  assume it is a ~10% normalisation uncertainty.  (MW, 12/07/02)
c
         do i_bin = 1, n_mjjbins
            mjjerr(i_bin) = sqrt(
     +           (0.25/10000.)*mjjdat(i_bin)**2
     +           *(sqrt(mjjtotp(i_bin)**2-mjjescalep(i_bin)**2)
     +           + sqrt(mjjtotm(i_bin)**2-mjjescalem(i_bin)**2))**2)
         end do
         do i_bin = 1, n_etmeanbins
            etmeanerr(i_bin) = sqrt(
     +           (0.25/10000.)*etmeandat(i_bin)**2
     +           *(sqrt(etmeantotp(i_bin)**2-etmeanescalep(i_bin)**2)
     +           + sqrt(etmeantotm(i_bin)**2-etmeanescalem(i_bin)**2)
     +           )**2)
         end do
         do i_bin = 1, n_etmaxbins
            etmaxerr(i_bin) = sqrt(
     +           (0.25/10000.)*etmaxdat(i_bin)**2
     +           *(sqrt(etmaxtotp(i_bin)**2-etmaxescalep(i_bin)**2)
     +           + sqrt(etmaxtotm(i_bin)**2-etmaxescalem(i_bin)**2))**2)
         end do
         do i_bin = 1, n_etabarbins
            etabarerr1(i_bin) = sqrt(
     +           (0.25/10000.)*etabardat1(i_bin)**2
     +           *(sqrt(etabartotp1(i_bin)**2-etabarescalep1(i_bin)**2)
     +           + sqrt(etabartotm1(i_bin)**2-etabarescalem1(i_bin)**2)
     +           )**2)
            etabarerr2(i_bin) = sqrt(
     +           (0.25/10000.)*etabardat2(i_bin)**2
     +           *(sqrt(etabartotp2(i_bin)**2-etabarescalep2(i_bin)**2)
     +           + sqrt(etabartotm2(i_bin)**2-etabarescalem2(i_bin)**2))
     +           **2)
            etabarerr3(i_bin) = sqrt(
     +           (0.25/10000.)*etabardat3(i_bin)**2
     +           *(sqrt(etabartotp3(i_bin)**2-etabarescalep3(i_bin)**2)
     +           + sqrt(etabartotm3(i_bin)**2-etabarescalem3(i_bin)**2)
     +           )**2)
            etabarerr4(i_bin) = sqrt(
     +           (0.25/10000.)*etabardat4(i_bin)**2
     +           *(sqrt(etabartotp4(i_bin)**2-etabarescalep4(i_bin)**2)
     +           + sqrt(etabartotm4(i_bin)**2-etabarescalem4(i_bin)**2)
     +           )**2)
         end do
         do i_bin = 1, n_xgammabins
            xgammaerr1(i_bin)=xgammadat1(i_bin)*xgammaErU1(i_bin)/100.0
            xgammaerr2(i_bin)=xgammadat2(i_bin)*xgammaErU2(i_bin)/100.0
            xgammaerr3(i_bin)=xgammadat3(i_bin)*xgammaErU3(i_bin)/100.0
            xgammaerr4(i_bin)=xgammadat4(i_bin)*xgammaErU4(i_bin)/100.0
         end do
         do i_bin = 1, n_xpbins
            xperr1(i_bin) = sqrt(
     +           (0.25/10000.)*xpdat1(i_bin)**2
     +           *(sqrt(xptotp1(i_bin)**2-xpescalep1(i_bin)**2)
     +           + sqrt(xptotm1(i_bin)**2-xpescalem1(i_bin)**2))**2)
            xperr2(i_bin) = sqrt(
     +           (0.25/10000.)*xpdat2(i_bin)**2
     +           *(sqrt(xptotp2(i_bin)**2-xpescalep2(i_bin)**2)
     +           + sqrt(xptotm2(i_bin)**2-xpescalem2(i_bin)**2))**2)
         end do
         do i_bin = 1, n_ctbins
            cterr1(i_bin) = ctdat1(i_bin) * ctErU1(i_bin) / 100.0
            cterr2(i_bin) = ctdat2(i_bin) * ctErU2(i_bin) / 100.0
            cterr3(i_bin) = ctdat3(i_bin) * ctErU3(i_bin) / 100.0
            cterr4(i_bin) = ctdat4(i_bin) * ctErU4(i_bin) / 100.0
         end do
c     change
         do i_bin = 1, n_pTbins
            pTerr1(i_bin) = pTdat1(i_bin) * pTErU1(i_bin) / 100.0 
            pTerr2(i_bin) = pTdat2(i_bin) * pTErU2(i_bin) / 100.0 
         end do
         do i_bin = 1, n_pT2bins
            pT2err3(i_bin) = pT2dat3(i_bin) * pT2ErU3(i_bin) / 100.0
            pT2err4(i_bin) = pT2dat4(i_bin) * pT2ErU4(i_bin) / 100.0
            pT2err5(i_bin) = pT2dat5(i_bin) * pT2ErU4(i_bin) / 100.0
            pT2err6(i_bin) = pT2dat6(i_bin) * pT2ErU5(i_bin) / 100.0
         end do
         do i_bin = 1, n_xxpbins
            xxperr1(i_bin) = xxpdat1(i_bin) * xxpErU1(i_bin) / 100.0
            xxperr2(i_bin) = xxpdat2(i_bin) * xxpErU2(i_bin) / 100.0
            xxperr3(i_bin) = xxpdat3(i_bin) * xxpErU3(i_bin) / 100.0
            xxperr4(i_bin) = xxpdat4(i_bin) * xxpErU4(i_bin) / 100.0
            xxperr5(i_bin) = xxpdat5(i_bin) * xxpErU5(i_bin) / 100.0
            xxperr6(i_bin) = xxpdat6(i_bin) * xxpErU6(i_bin) / 100.0
         end do

         CALL HPAK( 11,mjjdat)
         CALL HPAKE(11,mjjerr)
         CALL HPAK( 12,etmeandat)
         CALL HPAKE(12,etmeanerr)
         CALL HPAK( 13,etmaxdat)
         CALL HPAKE(13,etmaxerr)
         CALL HPAK( 14,etabardat1)
         CALL HPAKE(14,etabarerr1)
         CALL HPAK( 15,etabardat2)
         CALL HPAKE(15,etabarerr2)
         CALL HPAK( 16,etabardat3)
         CALL HPAKE(16,etabarerr3)
         CALL HPAK( 17,etabardat4)
         CALL HPAKE(17,etabarerr4)
         CALL HPAK( 18,xgammadat1)
         CALL HPAKE(18,xgammaerr1)
         CALL HPAK( 19,xgammadat2)
         CALL HPAKE(19,xgammaerr2)
         CALL HPAK( 20,xpdat1)
         CALL HPAKE(20,xperr1)
         CALL HPAK( 21,xpdat2)
         CALL HPAKE(21,xperr2)
         CALL HPAK( 22,xgammadat3)
         CALL HPAKE(22,xgammaerr3)
         CALL HPAK( 23,xgammadat4)
         CALL HPAKE(23,xgammaerr4)
         CALL HPAK( 24,ctdat1)
         CALL HPAKE(24,cterr1)
         CALL HPAK( 25,ctdat2)
         CALL HPAKE(25,cterr2)
         CALL HPAK( 26,ctdat3)
         CALL HPAKE(26,cterr3)
         CALL HPAK( 27,ctdat4)
         CALL HPAKE(27,cterr4)
c     change
         CALL HPAK( 28,pTdat1)
         CALL HPAKE(28,pTerr1)
         CALL HPAK( 29,pTdat2)
         CALL HPAKE(29,pTerr2)
         CALL HPAK( 30,pT2dat3)
         CALL HPAKE(30,pT2err3)
         CALL HPAK( 31,pT2dat4)
         CALL HPAKE(31,pT2err4)
         CALL HPAK( 32,pT2dat5)
         CALL HPAKE(32,pT2err5)
         CALL HPAK( 33,pT2dat6)
         CALL HPAKE(33,pT2err6)

         CALL HPAK( 34,xxpdat1)
         CALL HPAKE(34,xxperr1)
         CALL HPAK( 35,xxpdat2)
         CALL HPAKE(35,xxperr2)
         CALL HPAK( 36,xxpdat3)
         CALL HPAKE(36,xxperr3)
         CALL HPAK( 37,xxpdat4)
         CALL HPAKE(37,xxperr4)
         CALL HPAK( 38,xxpdat5)
         CALL HPAKE(38,xxperr5)
         CALL HPAK( 39,xxpdat6)
         CALL HPAKE(39,xxperr6)

C     --- normalise histograms

         DO i_plot=11,39
            CALL HOPERA(i_plot+100,'*e',i_plot+50,i_plot+100
     &           ,REAL(Xsec/Ntot),1.)
         ENDDO

         
      ELSE
C     *** End ***
         PRINT*,'HZ05555:Please run routine with iflag set to 1,2 or 3'
      ENDIF

      RETURN
      END
