      SUBROUTINE HZTEMPLATE(IFLAG)
****************************************************************************
*     [For subroutine naming conventions, see the hztool manual.]
*
*     Purpose: This routine reproduces the data from Fig.Xa, Yb, Yc and
*     Table Za from [reference], and produces histograms of simulated data
*     which can be compared to them.
*
*     Initial state: XX GeV things vs YY GeV other things.
*
*     Reference: [reference]
*     Arguments: iflag=10000*[A]+1000*[B]+100*[C]+10*[D]+1 initialisation
*                iflag=10000*[A]+1000*[B]+100*[C]+10*[D]+2 filling
*                iflag=10000*[A]+1000*[B]+100*[C]+10*[D]+3 termination
*
*     [A]= IPS   >0 to run on the final state of the parton shower, 0 to run on
*                final state particles. 
*     [B]= IPROC decides which process is being run when more than one is 
*                generated (e.g. single/double resolved and direct or 
*                diffractive and non-diffractive). Leave this to 0 if there is 
*                only one process type.
*     [C]= IRUN  =0 rerun jetfinder, =1 use results already stored.
*     [D]= JETF  decides jet finder type. Ignored if IRUN=1. If set to 0 the 
*                finder set by chjet below is used. This should be the finder
*                used in the paper (only necessary if using jets!).
*     See manual section "The analysis routines" for more details.
*     NB If you do not implement all these options, change the comments to say 
*     so!
*
*     It is assumed you run either (a) with iproc = 0
*                               or (b) with iproc = 1, 2... in order.
*     This example has only 2 processes (iproc=1,2) but others may be added.
*
*     Histograms: -N data for Fig.Xa
*                  N simulation for Fig.Xa
*                  ... etc.
*
*     written by: [me] on [today]
****************************************************************************
      IMPLICIT NONE

****** These included files contain function declarations and various 
****** parameter declarations which are required by the hztool routines.
****** In particular Xsec and Ntot (global variables) are defined in 
****** 'heracmn.inc'

#include "hzfunc.inc"
****** for histograms
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
****** (only necessary if using jets!)
#include "hzjetcmn.inc"

****** declare variables
      INTEGER ihep,iflag,chjet,iproc,njet,iloop
      REAL ptdata(10),ptdataerr(10),pt,rapdata(10),rapdataerr(10),rap,
     +     rapcut(2),q2,q2cut
      DOUBLE PRECISION coner
      SAVE coner
      CHARACTER*8 xxxx

****** initialize variables
      DATA q2cut/100./
      DATA rapcut/-2.,2./
****** save variables that retain their values between different subroutines
      SAVE chjet

****** (only necessary if using jets!)
****** Set the default jet finder as that in the paper.
****** (see the hztool manual section "The Jet Finders" for the different jet
******  finders available).
      DATA chjet/3/

****** Initialize data points and errors from plots/tables
      DATA ptdata    /10.,9.,8.,7.,6.,5.,4.,3.,2.,1./
      DATA ptdataerr /1., 1.,1.,1.,1.,1.,1.,1.,1.,1./
      DATA rapdata   /1.,2.,3.,4.,4.,5.,5.,6.,6.,6./
      DATA rapdataerr/1.,1.,1.,1.,1.,1.,1.,1.,1.,1./
****** Set xxxx equal to the arXiv:hep-ex number.
******  [For alternative subroutine naming conventions, see the manual.]
      DATA xxxx /'hyymmnnn'/

****** set the jet cone radius. (only necessary if using jets!)
****** CALL hzjetrad(1,coner) sets the radius to coner
****** CALL hzjetrad(2,coner) sets coner to the radius, or -1. 
******  check if the radius has been set eleswhere
      CALL hzjetrad(2,coner)
******  if not set it to the value used in the paper.
      IF (coner.LT.0.0) THEN
	      coner=1.0
      ENDIF

****** Determine the process type (single/double resolved etc)
****** (only necessary if > 1 process types!)
      iproc=(mod(iflag,10000)/1000)
      IF ((iproc.LT.0).OR.(iproc.GT.2)) THEN
         WRITE(HZUNIT,*) xxxx,': illegal IPROC:',iproc
         RETURN
      ENDIF

****** choose jet finder (only necessary if using jets!)
      IF ((MOD(iflag,100)/10.GT.0).AND.MOD(iflag,10).eq.1) THEN
         chjet=INT(MOD(iflag,100)/10)
      ENDIF

****** Decide if we are running on parton showers or final state particles.
      IF (MOD(iflag,100000)/10000.GT.0) THEN
****** use a different paw directory for parton showers.
         xxxx = 'PS'//xxxx
****** tell jet finder that we running on parton showers 
****** (only necessary if using jets!)
         chjet=-1*ABS(chjet)
      ENDIF
******************************************************************************
********************************  Initialise ********************************* 
      IF (MOD(iflag,10).EQ.1) THEN
****** Initialisation: The following MUST always be done
******     (i) make subdirectory in PAWC
******     - use the name as the xxxxxx in HZHxxxxxx subroutine
******     (i) make subdirectory in o/p file
         
         CALL hcdir('//PAWC',' ')
         IF (iproc.LE.1) THEN
             CALL hmdir(xxxx,'S')
         ELSE 
             CALL hcdir(xxxx,' ')
         ENDIF
 
         CALL hcdir('//HISTO',' ')
         IF (iproc.LE.1) THEN
             CALL hmdir(xxxx,'S')
         ELSE 
             CALL hcdir(xxxx,' ')
         ENDIF
     
******  book your histograms using:
******   1D histo
******     CALL hbook1(long key, char* title, long bins,  float xmin,  
******                 float xmax, float VMX) 
******   2D histo
******     CALL hbook2(long key, char* title, long bins1, float xmin1, 
******                 float xmax1, long bins2, float xmin2, float xmax2, 
******                 float VMX) 
******  1D histo with variable bin width
******     CALL hbookb(long key, char* title, long bins,  float* xvalues, 
******                 float VMX)
******
******  VMX upper limit of single channel content, set to 0.0 which  means 1 
******  word per channel (no packing).  

******  Give key a different value depending on which process you are running 
******  (only necessary if > 1 process types!)
        CALL hbook1(1000*iproc+11,'1/N dn/d-pt(MC)',10,0.0,20.,0.)
        CALL hbook1(1000*iproc+12,'1/N dn/d-rap(MC)',10,-2.0,2.0,0.)

******************************************************************************
**********************  Loop over events (fill) ******************************
      ELSE IF(MOD(iflag,10).EQ.2) THEN
******  Filling: The following MUST always be done
******    (i) move to the correct sub-directory in PAWC
         CALL hcdir('//PAWC/'//xxxx,' ')
******  example cut...
******  hzdiskin(x) returns the DIS kinematic variables 
******  (see hztool manual for details)
         q2  = hzdiskin(1)
         IF(q2.GT.q2cut) THEN
******  Loop through all particles in the events     
            DO 30 ihep = 1, nhep
****** The HEPEVT standard is used for the Monte Carlo output, see the
****** hztool manual for details.
****** If the particle is a final state particle
              IF (ISTHEP(ihep).EQ.1) THEN
******  calculate pT of the particle 
                  pt = sqrt(phep(1,ihep)**2+phep(2,ihep)**2)
******  Fill your histograms
                  CALL hfill(1000*iproc+11,pt,0.,wtx)
              ENDIF
 30         CONTINUE

****** run the hzjtfind jet finder (only necessary if using jets!)
******  chjet     : gives the type of jet finder to be used
******  coner     : gives the cone radius
******  njet      : gets set to the number of jets (maximum is 50)
******  jets(50,x): gets set to info about jets with
******             x=1,2 -> eta,phi of jets axis; x=3 -> eT of jet
******             x=4-8 -> e,px,py,pz,m of jet axis  
****** NB/ jets is declared in heracmn.inc
            CALL hzjtfind(chjet,coner,njet,jets)
****** loop through all jets in the event
            DO 20 iloop = 1, njet
******  get rapidity of jet
		 rap=Jets(iloop,1)
******  if the jet passes rapidity cuts
		 IF (rap.GE.rapcut(1).AND.rap.LE.rapcut(2)) THEN
******  Fill your histograms
		       CALL hfill(1000*iproc+12,rap,0.,wtx)
                 ENDIF
 20         CONTINUE
         ENDIF

*******************************************************************************
*******************************  Terminate ************************************
         
      ELSE IF(MOD(iflag,10).EQ.3) THEN

******  Termination: The following MUST always be done
******     (i) Move to the correct PAW subdirectory     
         CALL hcdir('//PAWC/'//xxxx,' ')

******  write the number of events and xsec for the process just finished.
         WRITE(6,*) xxxx,' X-section is',xsec, ' and number of events ' 
     +    ,ntot, ' for process ',iproc
  
****** normalise the histograms using hzhinrm(iid,iidnew,nevt,iifl)
****** with arguments: iid = histogram id, 
******                 iidnew = new histo id (if 0, old histo is modified)
******                 nevt = normalization factor
******                 iifl =1: normalize errors too, otherwise they are zeroed

         CALL hzhinrm(1000*iproc+11,0,real(ntot/xsec),1)
         CALL hzhinrm(1000*iproc+12,0,real(ntot/xsec),1)

******  Add the histos from different iproc values in the last call 
******  (only necessary if > 1 process types!)
         IF (iproc.EQ.2) THEN
******  add your histograms using:
******     CALL HOPERA (ID1,CHOPER,ID2,ID3,C1,C2) 
******      Action: Fills an histogram I3 with values such that 
******              ID3 = C1 * ID1 (OPERATION) C2 * ID2 
******      Parameters: IDn = histogram identifiers, 
******                  CHOPER =  Character variable specifying the kind of 
******                  operation to be performed (+,-,*,/) and 'E' for errors.
******                   C1,C2 = Multiplicative constants. 
           CALL hopera(1011,'+e',2011,11,1.,1.)
           CALL hopera(1012,'+e',2012,12,1.,1.)
         ENDIF

******  Deal with data plots once only
         IF (iproc.EQ.0.OR.iproc.EQ.2) THEN
****** book histograms
            CALL hbook1(-11,'1/N dn/d-pt(data)',10,0.0,20.,0.)
            CALL hbook1(-12,'1/N dn/d-rap(data)',10,-2.0,2.0,0.)
****** fill data points
            CALL hpak(-11,ptdata)
            CALL hpak(-12,rapdata)
****** fill data errors
            CALL hpake(-11,ptdataerr)
            CALL hpake(-12,rapdataerr)        
         ENDIF

      ENDIF

      RETURN

      END

