      SUBROUTINE HZPN443(iflag)
**********************************************************************
*     Dijet production in untagged photon-photon collisions 
*     as measured by OPAL at average sqrt(s)=194 GeV
*
* iflag = 1,2 or 3 depending on whether it is the initialization,
*		processing or termination phase (respectively)
*
* For this routine use
*    0+iflag   for direct + single + double
* 1000+iflag   for direct events
* 2000+iflag   for single resolved
* 3000+iflag   for double resolved
*
* Initialization: only has to be called once (but may be called three 
*                 times as well without harm
* Processing: 1002 for direct, 2002 for sngl res., 3002 for dble res.
*             a set of histogram (see below) is filled in addition 
*             in every case and may be used in generators which 
*             provide all processes in one go.
* Termination: call once with 1003, 2003, and 3003 respectively and
*              be careful to set Xsec to the appropriate value in the
*              driver routine
* 
* Produces graphs corresponding to Figures 11 & 12 in reference
*
* Cuts : Anti-tag on scattered beam particles (< 33 mrad)
*        Et_mean > 5 GeV
*        |Et1-Et2|/(Et1+Et2) < 1/4
*        |jet rapidity| < 2.0
*        In events with more than two jets, only the two jets with the
*        highest Et values are taken       
*              
* Histograms ---------------------------------------------------------
*                        DATA (OPAL Prel. - PN443)
*  stat    total  error
* --------------------------------------------------------------------
* -20211 (-30211) di-jet x-sect vs. Et-mean 
* -20313 (-30313) di-jet x-sect vs. x_g for 5 < Et-mean < 7 GeV
* -20314 (-30314) di-jet x-sect vs. log10(x_g) for 5 < Et-mean < 7 GeV
* -20323 (-30323) di-jet x-sect vs. x_g for 7 < Et-mean < 11 GeV
* -20333 (-30333) di-jet x-sect vs. x_g for 11 < Et-mean < 25 GeV
* ---------------------------------------------------------------------
*
*                                MC 
* ---------------------------------------------------------------------
*            
*       *** All Processes (from filling all processes at once) ***
* 10211 di-jet x-sect vs. Et-mean 
* 10313 di-jet x-sect vs. x_g for 5 < Et-mean < 7 GeV 
* 10314 di-jet x-sect vs. log10(x_g) for 5 < Et-mean < 7 GeV
* 10323 di-jet x-sect vs. x_g for 7 < Et-mean < 11 GeV
* 10333 di-jet x-sect vs. x_g for 11 < Et-mean < 25 GeV
*
*       *** All Processes (from summing contributions below) ***
* 11211 di-jet x-sect vs. Et-mean 
* 11313 di-jet x-sect vs. x_g for 5 < Et-mean < 7 GeV 
* 11314 di-jet x-sect vs. log10(x_g) for 5 < Et-mean < 7 GeV
* 11323 di-jet x-sect vs. x_g for 7 < Et-mean < 11 GeV
* 11333 di-jet x-sect vs. x_g for 11 < Et-mean < 25 GeV
*
* The second set of the histos above is intended for cases when the 
* contributions have to be calculated separately
*
*                     *** Direct Processes ***
* 12211 di-jet x-sect vs. Et-mean 
* 12313 di-jet x-sect vs. x_g for 5 < Et-mean < 7 GeV 
* 12314 di-jet x-sect vs. log10(x_g) for 5 < Et-mean < 7 GeV
* 12323 di-jet x-sect vs. x_g for 7 < Et-mean < 11 GeV 
* 12333 di-jet x-sect vs. x_g for 11 < Et-mean < 25 GeV
*
*                 *** Single Resolved  Processes ***
* 13211 di-jet x-sect vs. Et-mean                    
* 13313 di-jet x-sect vs. x_g for 5 < Et-mean < 7 GeV 
* 13314 di-jet x-sect vs. log10(x_g) for 5 < Et-mean < 7 GeV 
* 13323 di-jet x-sect vs. x_g for 7 < Et-mean < 11 GeV 
* 13333 di-jet x-sect vs. x_g for 11 < Et-mean < 25 GeV 
*
*                 *** Double Resolved  Processes ***
* 14211 di-jet x-sect vs. Et-mean 
* 14313 di-jet x-sect vs. x_g for 5 < Et-mean < 7 GeV 
* 14314 di-jet x-sect vs. log10(x_g) for 5 < Et-mean < 7 GeV 
* 14323 di-jet x-sect vs. x_g for 7 < Et-mean < 11 GeV 
* 14333 di-jet x-sect vs. x_g for 11 < Et-mean < 25 GeV 
*
*
* Author : Thorsten Wengler
*
* Reference: OPAL PN443
*            
* last change: 5 Oct 2000
*
* Modified :
* J. Monk (allow to be called with iproc=0) 14/3/2006
*
**********************************************************************
	IMPLICIT NONE
*
* HEP event prime common ---------------------------------------------
      Integer NMXHEP
      PARAMETER (NMXHEP=4000)
      Integer NEVHEP,NHEP,ISTHEP,IDHEP
      Integer JMOHEP,JDAHEP
      Double Precision PHEP,VHEP
      COMMON/HEPEVTP/NEVHEP,NHEP,ISTHEP(NMXHEP),IDHEP(NMXHEP),
     & JMOHEP(2,NMXHEP),JDAHEP(2,NMXHEP),PHEP(5,NMXHEP),VHEP(4,NMXHEP)
*
* HERA common --------------------------------------------------------
*
*     GEN: Name of generator
*     XSEC: total cross section (in pb)
*     IHCHRG: charge of particle/parton times 3
*     NTOT : Number of total events
*     WTX  : event weight
*
      Character*8 Gen
      Double Precision Xsec
      Integer ihchrg
      Real    wtx, Ntot
      Common /HERACMN/ Xsec, Gen, ihchrg(nmxhep), Ntot,wtx
*
c --- cut definition -------------------------------------------------
      real etmin
      real metmi1,metmi2,metmi3
      real metma1,metma2,metma3
      real jetasy
      real etamax
      parameter(
     +     etmin=3.0,           ! min jet Et-cut after jet finding
     +     metmi1=5.0,          ! mean Et min region 1
     +     metma1=7.0,          ! mean Et max region 1
     +     metmi2=7.0,          ! mean Et min region 2
     +     metma2=11.0,         ! mean Et max region 2
     +     metmi3=11.0,         ! mean Et min region 3
     +     metma3=25.0,         ! mean Et max region 3
     +     jetasy=0.25,          ! max.asym: abs(Et1-Et2)/(Et1+Et2)
     +     etamax=2.0           ! |etajet| max
     +     )
      double precision scelze,scelpi
      parameter(
     +     scelze=0.033D0,      ! max angle scat. el at ZERO
     +     scelpi=3.1085927D0   ! max angle scat. el at PI
     +     )

c --- definitions for histogram booking ------------------------------
c     The Et-mean distribution
      INTEGER etut1n          
      PARAMETER (etut1n=6)
      REAL etut1b(etut1n+1)
      DATA etut1b/ 5.0, 6.54, 8.55, 11.18, 14.62, 19.12, 
     +     25.00 / 

c     x-gamma distribution for 5 < Et-mean < 7 GeV
      INTEGER xgut1n                          ! x-gamma linear scale
      PARAMETER (xgut1n=8)
      REAL xgut1b(xgut1n+1)
      DATA xgut1b/ 0.0, 0.125, 0.25, 0.375, 0.5, 0.625, 0.75, 0.875, 
     +     1.0 / 
      INTEGER xgut1sn                          ! x-gamma log scale
      PARAMETER (xgut1sn=8)
      REAL xgut1sb(xgut1sn+1)
      DATA xgut1sb/ -2.0, -1.75, -1.5, -1.25, -1.0, -0.75, -0.5,  
     +     -0.25, 0.0 / 

c     x-gamma distribution for 7 < Et-mean < 11 GeV
      INTEGER xgut2n                          
      PARAMETER (xgut2n=8)
      REAL xgut2b(xgut2n+1)
      DATA xgut2b/ 0.0, 0.125, 0.25, 0.375, 0.5, 0.625, 0.75, 0.875, 
     +     1.0 / 

c     x-gamma distribution for 11 < Et-mean < 25 GeV
      INTEGER xgut3n                          ! x-gamma
      PARAMETER (xgut3n=8)
      REAL xgut3b(xgut3n+1)
      DATA xgut3b/ 0.0, 0.125, 0.25, 0.375, 0.5, 0.625, 0.75, 0.875, 
     +     1.0 / 
c --- end definitions for histogram booking

      DOUBLE PRECISION mhxsec(3)
      INTEGER mhntot(3)
      REAL norm
      INTEGER initflag, iproc, iflag
      DOUBLE PRECISION angle

c local work variables
      integer i,j,maxtrk
      PARAMETER (maxtrk=1000)
      integer veto1,veto2,kbeam,i1,i2
      integer maxjet,nktin
      parameter (maxjet=1000)
      double precision zero
      PARAMETER (ZERO   =  0.D0)
      integer nftrk,nscel
      integer stflag,termfl
      data stflag/0/
      data termfl/0/

c     local variables for ktclus
      double precision vin(4,maxtrk),vpre(4,maxjet)
      integer vpart(maxjet),njet
      real yc(maxjet)
      real jtheta,jeta
      real ktinpx(maxjet),ktinpy(maxjet),ktinpz(maxjet),ktinen(maxjet)
      real pihalf,pival,wght
      parameter (pihalf = 1.5707963, pival = 3.1415927)
      real etj1,etj2,etjm,thj1,thj2,etaj1,etaj2,jesum,jpzsum,xgpl,xgmi
      real etatmp,etajm,detaj,etcom
      real mcpxhv,mcpyhv,mcpzhv,mcenhv

c
c --- functions ------------------------------------------------------
      DOUBLE PRECISION hzeekin
      INTEGER hzeebeam

c --- define the data
c     Et-mean distribution
      real detm(6),detmse(6),detmte(6)
      data detm/ 10.90, 3.907, 1.555, 0.4799, 0.1447,0.0365 /
      data detmse/ 0.275, 0.135, 0.0741, 0.0360, 0.0175, 0.0079 /
      data detmte/ 0.684, 0.252, 0.111, 0.0496,  0.0315, 0.00944/

c     x-gamma distribution region 1
      real dxg1(8),dxg1se(8),dxg1te(8)
      data dxg1/ 43.98 ,42.05 ,34.23 ,27.56,24.78,33.02,
     +     68.33, 42.72/
      data dxg1se/2.414 ,1.992 ,1.922 ,1.750 ,1.645 ,1.907 ,
     +     2.811 ,2.547/
      data dxg1te/5.171 ,4.318 ,4.791 ,3.801 ,2.303 ,3.688 ,
     +     4.178 ,6.025/

c     x-gamma distribution region 1 (log scale)
      real dxg2(8),dxg2se(8),dxg2te(8)
      data dxg2/ 0.519 ,2.338 ,4.769 ,9.435 ,14.535 ,22.21 ,
     +     25.86 ,79.91/
      data dxg2se/ 0.115 ,0.288 ,0.409 ,0.583 ,0.712 ,0.923 ,
     +     0.994 ,2.326/
      data dxg2te/ 0.147 ,0.334 ,0.561 ,1.074 ,1.634 ,2.568 ,
     +     2.485 ,4.072/

c     x-gamma distribution region 2
      real dxg3(8),dxg3se(8),dxg3te(8)
      data dxg3/ 12.156 ,14.643 ,11.792 ,10.561 ,10.251 ,12.189 ,
     +     31.484 ,45.938/
      data dxg3se/ 0.993 ,1.049 ,0.880 ,0.826 ,0.821 ,0.843 ,
     +     1.482 ,2.196/
      data dxg3te/ 1.761 ,1.662 ,1.390 ,1.138 ,1.085 ,1.363 ,
     +     2.918 ,3.465/

c     x-gamma distribution region 3
      real dxg4(8),dxg4se(8),dxg4te(8)
      data dxg4/ 1.600 ,2.494 ,3.136 ,2.562 ,2.536 ,3.593 ,
     +     11.703 ,20.440/
      data dxg4se/ 0.638 ,0.597 ,0.735 ,0.664 ,0.725 ,0.794 ,
     +     1.727 ,1.790/ 
      data dxg4te/ 0.745 ,0.761 ,0.926 ,0.697 ,0.937 ,1.0535 ,
     +     2.375 ,3.181 /


C     --- choose direct/resolved -------------------------------------
C     ---1 - All  2 - DIRECT, 3 - single-RESOLVED, 4 - double-RESOLVED

c     change so that 0 = All, 1 = Direct, 2 = Single Resolved and 3 = Double

      iflag = iflag + 1000;

      IF (iflag.gt.1000) THEN
        iproc=INT(iflag*0.001)
        initflag=MOD(iflag,1000)
      ELSE
        PRINT*,'HZPN443 : initflag < 1000 ???'
      ENDIF
      IF ((iproc.gt.4).or.(iproc.lt.1)) THEN
        PRINT*,'HZPN443 : Badly called routine'
        PRINT*,'           set initflag +1000, +2000 or +3000'
      ENDIF
      
c --- Initialization -------------------------------------------------
      IF (initflag.eq.1) THEN
	IF (stflag.eq.0) THEN
         stflag=1
C --- Create directories 
         CALL HCDIR('//HISTO',' ')
         CALL HMDIR('PN443','S')
         CALL HCDIR('//PAWC',' ')
         CALL HMDIR('PN443','S')
         CALL HCDIR('//HISTO/PN443',' ')
         CALL HCDIR('//PAWC/PN443',' ')
        
c --- book the MC histograms
         CALL HBOOKB(10211,'(E?T!&&^jet1!+ E?T!&&^jet2!)/2 />G<E>V',
     +        etut1n,etut1b,0.)
         CALL HBOOKB(10313,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(10314,'log10(<x?[g]!)',xgut1sn,xgut1sb,0.)
         CALL HBOOKB(10323,'<x?[g]!',xgut2n,xgut2b,0.)
         CALL HBOOKB(10333,'<x?[g]!',xgut3n,xgut3b,0.)
         
         CALL HBOOKB(12211,'(E?T!&&^jet1!+ E?T!&&^jet2!)/2 />G<E>V',
     +        etut1n,etut1b,0.)
         CALL HBOOKB(12313,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(12314,'log10(<x?[g]!)',xgut1sn,xgut1sb,0.)
         CALL HBOOKB(12323,'<x?[g]!',xgut2n,xgut2b,0.)
         CALL HBOOKB(12333,'<x?[g]!',xgut3n,xgut3b,0.)
         
         CALL HBOOKB(13211,'(E?T!&&^jet1!+ E?T!&&^jet2!)/2 />G<E>V',
     +        etut1n,etut1b,0.)
         CALL HBOOKB(13313,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(13314,'log10(<x?[g]!)',xgut1sn,xgut1sb,0.)
         CALL HBOOKB(13323,'<x?[g]!',xgut2n,xgut2b,0.)
         CALL HBOOKB(13333,'<x?[g]!',xgut3n,xgut3b,0.)
         
         CALL HBOOKB(14211,'(E?T!&&^jet1!+ E?T!&&^jet2!)/2 />G<E>V',
     +        etut1n,etut1b,0.)
         CALL HBOOKB(14313,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(14314,'log10(<x?[g]!)',xgut1sn,xgut1sb,0.)
         CALL HBOOKB(14323,'<x?[g]!',xgut2n,xgut2b,0.)
         CALL HBOOKB(14333,'<x?[g]!',xgut3n,xgut3b,0.)
         
         
c --- book the data histograms
c --- with statistical error only
         CALL HBOOKB(-20211,'OPAL Prel (w/stat. err)"J# (E?T!&&^jet1!+ E 
     +?T!&&^jet2!)/2 />G<E>V',etut1n,etut1b,0.)
         CALL HBOOKB(-20313,'OPAL Prel (w/stat. err)"J#<x?[g]!',
     +        xgut1n,xgut1b,0.)
         CALL HBOOKB(-20314,'OPAL Prel (w/stat. err)"J#log10(<x?[g]!)',
     +        xgut1sn,xgut1sb,0.)
         CALL HBOOKB(-20323,'OPAL Prel (w/stat. err)"J#<x?[g]!',xgut2n,
     +        xgut2b,0.)
         CALL HBOOKB(-20333,'OPAL Prel (w/stat. err) "J#<x?[g]!',xgut3n,
     +        xgut3b,0.)
         
c     --- with statistical total error
         CALL HBOOKB(-30211,'OPAL Prel (w/total err)"J# (E?T!&&^jet1!+ E 
     +?T!&&^jet2!)/2 />G<E>V',etut1n,etut1b,0.)
         CALL HBOOKB(-30313,'OPAL Prel (w/total err)"J#<x?[g]!',xgut1n,
     +        xgut1b,0.)
         CALL HBOOKB(-30314,'OPAL Prel (w/total err)"J#log10(<x?[g]!)',
     +        xgut1sn,xgut1sb,0.)
         CALL HBOOKB(-30323,'OPAL Prel (w/total err)"J#<x?[g]!',xgut2n,
     +        xgut2b,0.)
         CALL HBOOKB(-30333,'OPAL Prel (w/total err)"J#<x?[g]!',xgut3n,
     +        xgut3b,0.)
         
         
         CALL HBARX(0)
         
c --- fill data into histograms
         CALL HPAK(-20211,detm)
         CALL HPAKE(-20211,detmse)
         CALL HPAK(-20313,dxg1)
         CALL HPAKE(-20313,dxg1se)
         CALL HPAK(-20314,dxg2)
         CALL HPAKE(-20314,dxg2se)
         CALL HPAK(-20323,dxg3)
         CALL HPAKE(-20323,dxg3se)
         CALL HPAK(-20333,dxg4)
         CALL HPAKE(-20333,dxg4se)
         
         CALL HPAK(-30211,detm)
         CALL HPAKE(-30211,detmte)
         CALL HPAK(-30313,dxg1)
         CALL HPAKE(-30313,dxg1te)
         CALL HPAK(-30314,dxg2)
         CALL HPAKE(-30314,dxg2te)
         CALL HPAK(-30323,dxg3)
         CALL HPAKE(-30323,dxg3te)
         CALL HPAK(-30333,dxg4)
         CALL HPAKE(-30333,dxg4te)

	ENDIF	! stflag=0        
        
         WRITE(6,*)'**************************************************'
         WRITE(6,*)'* HZPN443 called, histograms will be generated   *'
         WRITE(6,*)'* for dijet production in untagged photon-photon *'
         WRITE(6,*)'* collisions. See OPAL PN 443 for reference.     *'
         WRITE(6,*)'*           This DATA is PRELIMINARY!            *'
         WRITE(6,*)'**************************************************'
         
c --- run the event loop ---------------------------------------------
      ELSE IF (initflag.eq.2) THEN

c --- reset variables
	 veto1=0
	 veto2=0
	 i1=0
	 i2=0
         etj1  = 0.0
         etj2  = 0.0
         etjm  = 0.0
         etcom = 0.0
         thj1  = 0.0
         thj2  = 0.0
         etaj1 = 0.0
         etaj2 = 0.0
         etatmp= 0.0
         etajm = 0.0
         detaj = 0.0
         jesum = 0.0
         jpzsum= 0.0
         mcpxhv = 0.0
         mcpyhv = 0.0
         mcpzhv = 0.0
         mcenhv = 0.0

         xgpl= 0.0
         xgmi= 0.0
         njet=0
         
         do 10 i=1,maxtrk
            vin(1,i)=0.0D0
            vin(2,i)=0.0D0
            vin(3,i)=0.0D0
            vin(4,i)=0.0D0
 10      continue
         do 11 i=1,maxjet
            do 12 j=1,4
               vpre(j,i)=0.0D0
 12         continue
            ktinpx(i)=0.0
            ktinpy(i)=0.0
            ktinpz(i)=0.0
            ktinen(i)=0.0
            vpart(i)=0.0
            yc(i)=0
 11      continue

         nftrk = 0

         wght = 1.0
         angle=1.D0

         CALL HCDIR('//PAWC/PN443',' ')
        
         angle=hzeekin(2)
         IF ((angle.ne.-1D0).and. ! anti-tag cut
     +        ((angle.lt.scelze).or.(angle.gt.scelpi))) Then 

C --- Make sure scattered electrons are not included in hadronic
c     final state. (NB. Not robust against changes in positions of
c     electrons in HEP common for a given generator).
            IF (GEN(1:3).eq.'HRW') THEN
              veto1=1
              veto2=2
            ELSEIF (GEN(1:3).eq.'PYT') THEN
              kbeam= hzeebeam(i1,i2) ! find e- and e+ position
              If ((i1.eq.1).and.(i2.eq.2)) Then
                veto1=3
                veto2=4
              ElseIf ((i1.eq.2).and.(i2.eq.1)) Then
                veto1=4
                veto2=3
              EndIf
            ELSEIF (GEN(1:3).eq.'PHO') THEN
              veto1=3
              veto2=5
            ELSE
              write(6,*)'HZPN443: Unknown generator'
            ENDIF
            IF ((abs(IDHEP(veto1)).ne.11).or.
     +		      (abs(IDHEP(veto2)).ne.11)) then
	      print*,'HZPN443: electrons not correctly found!'
	    ENDIF
            
            DO 100 I=1,NHEP
C --- Another check that electrons found correctly
	       IF((ISTHEP(I).eq.1).and.((JMOHEP(1,I).eq.veto1)
     +		 .or.(JMOHEP(1,I).eq.veto2))
     +		 .and.(abs(IDHEP(I)).ne.11)) THEN
		  print*,'HZPN443: electrons not correctly found!'
	       ENDIF
C --- select final particles excluding electrons
               IF((ISTHEP(I).EQ.1).and.(JMOHEP(1,I).ne.veto1)
     +		 .and.(JMOHEP(1,I).ne.veto2)) THEN
c     hadronic final state - prepare input array for jet finders
                     nftrk=nftrk+1
                     vin(1,nftrk) = PHEP(1,I)
                     vin(2,nftrk) = PHEP(2,I)
                     vin(3,nftrk) = PHEP(3,I)
                     vin(4,nftrk) = PHEP(4,I)
                     mcpxhv=mcpxhv+sngl(PHEP(1,I))
                     mcpyhv=mcpyhv+sngl(PHEP(2,I))
                     mcpzhv=mcpzhv+sngl(PHEP(3,I))
                     mcenhv=mcenhv+sngl(PHEP(4,I))
               ENDIF
 100        CONTINUE

c --- call jet finders
c     kt_incl
            call ktclur(4212,vin,nftrk,1.0,1.0,yc,*999)
            call ktincl(2,vin,nftrk,vpre,vpart,njet,*998)

c --- apply pt & eta jet cut and order with largest pt first
            nktin=0
            do 30 i=njet,1,-1
               if(sngl(vpre(1,i)**2 + vpre(2,i)**2) 
     +              .gt. etmin**2) then
                  jtheta=acos(sngl(vpre(3,i))/sngl(vpre(4,i)))
                  jeta=-log(tan(jtheta/2))
                  if(abs(jeta).lt.etamax) then
                     nktin=nktin+1
                     ktinpx(nktin) = sngl(vpre(1,i))
                     ktinpy(nktin) = sngl(vpre(2,i))
                     ktinpz(nktin) = sngl(vpre(3,i))
                     ktinen(nktin) = sngl(vpre(4,i))
                  endif
               endif
 30         continue
c     

*     calculate jet quantities
            etj1  = sqrt(ktinpx(1)**2+ktinpy(1)**2)
            etj2  = sqrt(ktinpx(2)**2+ktinpy(2)**2)
            etjm  = (etj1+etj2)/2
            etcom = abs(etj1-etj2)/(etj1+etj2)
            thj1  = atan2(sqrt(ktinpx(1)**2+ktinpy(1)**2),ktinpz(1))
            thj2  = atan2(sqrt(ktinpx(2)**2+ktinpy(2)**2),ktinpz(2))
            etaj1 = -log(tan(thj1/2))
            etaj2 = -log(tan(thj2/2))
            etajm=(abs(etaj1)+abs(etaj2))/2
            detaj=abs(etaj1-etaj2)
            jesum = ktinen(1)+ktinen(2)
            jpzsum= ktinpz(1)+ktinpz(2)
            
            xgpl=( jesum+jpzsum ) / ( mcenhv+mcpzhv )
            xgmi=( jesum-jpzsum ) / ( mcenhv-mcpzhv )
            
c     apply the jet-cuts and fill the histos
c

            if(etjm.gt.metmi1 .and. etcom.lt.jetasy) then

c --- fill the Et-mean distribution
               call hf1(10211,etjm,wght)          ! All
               if(iproc.eq.1) then                ! direct
                  call hf1(12211,etjm,wght)
               elseif(iproc.eq.2) then            ! single resolved
                  call hf1(13211,etjm,wght)
               elseif(iproc.eq.3) then            ! double resolved
                  call hf1(14211,etjm,wght)
               endif

c --- fill the x-gamma distributions for region 1
               if(etjm.gt.metmi1 .and. etjm.le.metma1) then 
                  call hf1(10313,xgpl,wght)       ! All    
                  call hf1(10313,xgmi,wght)
                  call hf1(10314,log10(xgpl),wght)
                  call hf1(10314,log10(xgmi),wght)
                  if(iproc.eq.1) then             ! direct
                     call hf1(12313,xgpl,wght)
                     call hf1(12313,xgmi,wght)
                     call hf1(12314,log10(xgpl),wght)
                     call hf1(12314,log10(xgmi),wght)
                  elseif(iproc.eq.2) then         ! single resolved
                     call hf1(13313,xgpl,wght)
                     call hf1(13313,xgmi,wght)
                     call hf1(13314,log10(xgpl),wght)
                     call hf1(13314,log10(xgmi),wght)
                  elseif(iproc.eq.3) then         ! double resolved
                     call hf1(14313,xgpl,wght)
                     call hf1(14313,xgmi,wght)
                     call hf1(14314,log10(xgpl),wght)
                     call hf1(14314,log10(xgmi),wght)
                  endif
               endif
            
c --- fill the x-gamma distributions for region 2
               if(etjm.gt.metmi2 .and. etjm.le.metma2) then 
                  call hf1(10323,xgpl,wght)     ! All
                  call hf1(10323,xgmi,wght)
                  if(iproc.eq.1) then           ! direct
                     call hf1(12323,xgpl,wght)
                     call hf1(12323,xgmi,wght)
                  elseif(iproc.eq.2) then       ! single resolved
                     call hf1(13323,xgpl,wght)
                     call hf1(13323,xgmi,wght)
                  elseif(iproc.eq.3) then       ! double resolved
                     call hf1(14323,xgpl,wght)
                     call hf1(14323,xgmi,wght)
                  endif
               endif
               
c --- fill the x-gamma distributions for region 2
               if(etjm.gt.metmi3 .and. etjm.le.metma3) then 
                  call hf1(10333,xgpl,wght)     ! All 
                  call hf1(10333,xgmi,wght)
                  if(iproc.eq.1) then           ! direct
                     call hf1(12333,xgpl,wght)
                     call hf1(12333,xgmi,wght)
                  elseif(iproc.eq.2) then       ! single resolved
                     call hf1(13333,xgpl,wght)
                     call hf1(13333,xgmi,wght)
                  elseif(iproc.eq.3) then       ! double resolved
                     call hf1(14333,xgpl,wght)
                     call hf1(14333,xgmi,wght)
                  endif
               endif
            endif
            
         ENDIF                  
        
c --- run termination ------------------------------------------------
      ELSE IF (initflag.eq.3) THEN

C --- Change directory 
         CALL HCDIR('//PAWC/PN443',' ')

C --- check x-sections
         IF (Xsec.eq.0) THEN
            PRINT*,'HZPN443: term. called with zero cross section'
            PRINT*,'          cross section graph meaningless'
            PRINT*,'	      in process :',(iproc-1)*1000
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            PRINT*,'HZPN443: term. called with no total events'
            PRINT*,'          cross section graph meaningless'
            PRINT*,'          in process :',(iproc-1)*1000
            Ntot=1
         ENDIF
         
C --- store Xsec and Ntot for both direct and resolved events.
        
         mhXsec(iproc)=1000*Xsec
         mhntot(iproc)=Ntot

         norm=0.
         norm=REAL(mhntot(iproc)/mhXsec(iproc))
 

c --- All
         if(termfl.eq.0) then
            termfl=1
            call HzHinrm(10211,0,norm,1) ! Et: metmi1
            call HzHinrm(10313,0,norm,1) ! xg : metmi1
            call HzHinrm(10314,0,norm,1) ! xg : metmi1 log
            call HzHinrm(10323,0,norm,1) ! xg : metmi2
            call HzHinrm(10333,0,norm,1) ! xg : metmi3     
         endif
c --- direct 
         if(iproc.eq.1) then
            call HzHinrm(12211,0,norm,1) ! Et: metmi1
            call HzHinrm(12313,0,norm,1) ! xg : metmi1
            call HzHinrm(12314,0,norm,1) ! xg : metmi1 log
            call HzHinrm(12323,0,norm,1) ! xg : metmi2
            call HzHinrm(12333,0,norm,1) ! xg : metmi3     
         endif                 
                               
c --- single resolved          
         if(iproc.eq.2) then   
            call HzHinrm(13211,0,norm,1) ! Et: metmi1
            call HzHinrm(13313,0,norm,1) ! xg : metmi1
            call HzHinrm(13314,0,norm,1) ! xg : metmi1 log
            call HzHinrm(13323,0,norm,1) ! xg : metmi2
            call HzHinrm(13333,0,norm,1) ! xg : metmi3     
            
            CALL HOPERA(13211,'+e',12211,15211,1.,1.)
            CALL HOPERA(13313,'+e',12313,15313,1.,1.)
            CALL HOPERA(13314,'+e',12314,15314,1.,1.)
            CALL HOPERA(13323,'+e',12323,15323,1.,1.)
            CALL HOPERA(13333,'+e',12333,15333,1.,1.)

         endif

c --- double resolved
         if(iproc.eq.3) then
            call HzHinrm(14211,0,norm,1) ! Et: metmi1
            call HzHinrm(14313,0,norm,1) ! xg : metmi1
            call HzHinrm(14314,0,norm,1) ! xg : metmi1 log
            call HzHinrm(14323,0,norm,1) ! xg : metmi2
            call HzHinrm(14333,0,norm,1) ! xg : metmi3     
            
            CALL HOPERA(15211,'+e',14211,11211,1.,1.)
            CALL HOPERA(15313,'+e',14313,11313,1.,1.)
            CALL HOPERA(15314,'+e',14314,11314,1.,1.)
            CALL HOPERA(15323,'+e',14323,11323,1.,1.)
            CALL HOPERA(15333,'+e',14333,11333,1.,1.)

            CALL HDELET(15211)
            CALL HDELET(15313)
            CALL HDELET(15314)
            CALL HDELET(15323)
            CALL HDELET(15333)

         endif

        
      ELSE
C     *** End ***
         PRINT*,'HZPN443:Please run with iflag set to 0, 1,2 or 3'
      ENDIF
      
      RETURN
      
c --- jet finder jump out
 999  continue
      print *,'HZPN443: KTCLUR ERROR'
      return
 998  continue
      print *,'HZPN443: KTINCL ERROR'
      return
c --- end of routine HZPN443 -----------------------------------------      
      END










